package com.codes_roots.imakeapp.create_vendor_driver_account.viewmodel

import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.repository.homepage.RemoteDataSource
import com.satafood.core.entities.add_driver_request.DriverRequestModel
import io.reactivex.disposables.CompositeDisposable
import kotlinx.coroutines.*
import javax.inject.Inject

class CreateDriverAccViewModel @Inject constructor(private val Datasources: RemoteDataSource,
) : ViewModel() {

    @Inject
    lateinit var Pref: PreferenceHelper


    private var job: Job? = null
    var mCompositeDisposable = CompositeDisposable()
    var processVisibility = MutableLiveData(false)

    var rateJob: Job? = null
    var authLD: MutableLiveData<DriverRequestModel>? = null

    val errorMessage = MutableLiveData<String>()
    val loading = MutableLiveData<Boolean>()

    val coroutineExceptionHandler = CoroutineExceptionHandler{_, throwable ->
        throwable.printStackTrace()
    }

    init {
        authLD = MutableLiveData()
    }


    fun driverRequestModel(driverRequestModel: DriverRequestModel) {
        val coroutineExceptionHandler = CoroutineExceptionHandler { _, throwable ->
            throwable.printStackTrace()
        }
        job = CoroutineScope(Dispatchers.Default + coroutineExceptionHandler).launch {
            val response = Datasources.addDriverRequest(driverRequestModel!!)
            withContext(Dispatchers.Main + coroutineExceptionHandler) {
                if (response.isSuccessful) {
                    authLD?.postValue(response.body())

                } else {
                    onError("Error : ${response.message()} ")
                }
            }
        }

    }



    private fun onError(message: String) {
        errorMessage.value = message
        loading.value = false
    }

    override fun onCleared() {
        super.onCleared()
        mCompositeDisposable.dispose()
        mCompositeDisposable.clear()
        rateJob?.cancel()
    }
}