package com.codes_roots.imakeapp.di

import android.content.Context
import cn.pedant.SweetAlert.SweetAlertDialog
import com.google.firebase.messaging.FirebaseMessaging

import com.codes_roots.imakeapp.repository.wallet.GetUserCreditUseCase
import com.codes_roots.imakeapp.repository.wallet.GetWalletListUseCase
import com.codes_roots.imakeapp.repository.wallet.WalletDataSource
import com.codes_roots.imakeapp.repository.wallet.WalletRepository
import com.codes_roots.imakeapp.datalayer.APIServices
import com.codes_roots.imakeapp.datalayer.ExceptionsMapper
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.repository.RestaurantsList.RestListRemoteDataSource
import com.codes_roots.imakeapp.repository.RestaurantsList.RestaurantListDataSource
import com.codes_roots.imakeapp.repository.cart.CartDataSource
import com.codes_roots.imakeapp.repository.cart.CartRemoteDataSource
import com.codes_roots.imakeapp.repository.gallery.GalleryDataSource
import com.codes_roots.imakeapp.repository.gallery.GalleryRemoteDataSource
import com.codes_roots.imakeapp.repository.google.GoogleDataSource
import com.codes_roots.imakeapp.repository.google.GoogleRemoteDataSource
import com.codes_roots.imakeapp.repository.home.GetItemDetailsUseCase
import com.codes_roots.imakeapp.repository.home.HomeDataSourceImpl
import com.codes_roots.imakeapp.repository.home.HomeRepositoryImpl
import com.codes_roots.imakeapp.repository.homepage.DataSource
import com.codes_roots.imakeapp.repository.homepage.RemoteDataSource
import com.codes_roots.imakeapp.repository.main.FCMRepositoryImpl
import com.codes_roots.imakeapp.repository.main.MainDataSource
import com.codes_roots.imakeapp.repository.support.SupportRepositoryImpl
import com.codes_roots.imakeapp.repository.searchRepo.SearchDataSource
import com.codes_roots.imakeapp.repository.searchRepo.SearchRemoteDataSource
import com.codes_roots.imakeapp.repository.splash.SplashDataSource
import com.codes_roots.imakeapp.repository.splash.SplashRemoteDataSource
import com.codes_roots.imakeapp.repository.support.SupportDataSource
import com.codes_roots.imakeapp.repository.wallet_details.WalletDetailsRemoteDataSource
import com.satafood.core.entities.favourite.FavouriteModelItem
import com.satafood.core.entities.myorders.MyOrderModelItem
import com.satafood.core.entities.restaurantdetails.MenuCategoriesItem
import com.satafood.core.entities.restaurantdetails.StoreDetailsModel
import com.satafood.core.mapper.main.GetQueriesUseCase
import com.satafood.core.mapper.main.SubscribeTopicUseCase
import com.satafood.core.mapper.main.UpdateUserTokenUseCase
import dagger.Module
import dagger.Provides
import kotlinx.coroutines.CoroutineDispatcher
import javax.inject.Singleton

@Module
class AppModule() {

    @Singleton
    @Provides
    fun provideTasksRemoteDataSource(apiService: APIServices): DataSource {
        return RemoteDataSource(apiService)
    }
    @Singleton
    @Provides
    fun provideTasksRemoteDataSourceGoogle(api: APIServices): GoogleDataSource {
        return GoogleRemoteDataSource(api)
    }

    @Singleton
    @Provides
    fun provideTasksRemoteDataSourceGallery(api: APIServices): GalleryDataSource {
        return GalleryRemoteDataSource(api)
    }

    @Singleton
    @Provides
    fun provideTasksRemoteSearchRemoteDataSource(api: APIServices): SearchDataSource {
        return SearchRemoteDataSource(api)
    }

    @Singleton
    @Provides
    fun provideTasksRemoteSplashRemoteDataSource(api: APIServices): SplashDataSource {
        return SplashRemoteDataSource(api)
    }
    @Singleton
    @Provides
    fun provideTasksRemoteWalletDetailsRemoteDataSource(api: APIServices): com.codes_roots.imakeapp.repository.wallet_details.WalletDataSource {
        return WalletDetailsRemoteDataSource(api)
    }

    @Singleton
    @Provides
    fun provideTasksRemoteCartRemoteDataSource(api: APIServices): CartDataSource {
        return CartRemoteDataSource(api)
    }




    @Singleton
    @Provides
    fun provideTasksRestaurantListDataSource(
        apiService: APIServices
    ): RestaurantListDataSource {
        return RestListRemoteDataSource(apiService)
    }


    @Provides
    fun provideMenuCategoriesItem(): MenuCategoriesItem {
        return MenuCategoriesItem()
    }

    @Provides
    fun provideRestaurantDetailsModel(): StoreDetailsModel {
        return StoreDetailsModel()
    }
    @Provides
    fun MyOrderModelItem(): MyOrderModelItem {
        return MyOrderModelItem()
    }

    @Provides
    fun FavouriteViewModel(): FavouriteModelItem {
        return FavouriteViewModel()
    }



    @Provides
    fun providePreferenceHelper(context: Context): PreferenceHelper {
        return PreferenceHelper(context)
    }
    @Provides
    fun provideSweetAlertDialog(context: Context): SweetAlertDialog {
        return SweetAlertDialog(context)
    }


    @Singleton
    @Provides
    fun provideWalletDataSource(context: Context, apiService: APIServices,
                                exceptionsMapper: ExceptionsMapper
    ): WalletDataSource {
        return WalletDataSource(apiService, context, exceptionsMapper)
    }

    @Singleton
    @Provides
    fun provideMainDataSource(context: Context, apiService: APIServices,
                                exceptionsMapper: ExceptionsMapper
    ): MainDataSource {
        return MainDataSource(apiService, context, exceptionsMapper)
    }

    @Singleton
    @Provides
    fun provideSupportDataSource(context: Context, apiService: APIServices,
                                exceptionsMapper: ExceptionsMapper
    ): SupportDataSource {
        return SupportDataSource(apiService, context, exceptionsMapper)
    }

    @Singleton
    @Provides
    fun provideHomeDataSource(context: Context, apiService: APIServices,
                                exceptionsMapper: ExceptionsMapper
    ): HomeDataSourceImpl {
        return HomeDataSourceImpl(apiService, context, exceptionsMapper)
    }

    @Singleton
    @Provides
    fun provideWalletRepository(walletDataSource: WalletDataSource, @IoDispatcher ioDispatcher: CoroutineDispatcher): WalletRepository {
        return WalletRepository(walletDataSource, ioDispatcher)
    }

    @Singleton
    @Provides
    fun provideFCMRepository(mainDataSource: MainDataSource, exceptionsMapper: ExceptionsMapper, firebaseMessaging: FirebaseMessaging): FCMRepositoryImpl {
        return FCMRepositoryImpl(mainDataSource, exceptionsMapper, firebaseMessaging)
    }

    @Singleton
    @Provides
    fun provideSupportRepository(supportDataSource: SupportDataSource): SupportRepositoryImpl {
        return SupportRepositoryImpl(supportDataSource)
    }

    @Singleton
    @Provides
    fun provideHomeRepository(homeDataSource: HomeDataSourceImpl): HomeRepositoryImpl {
        return HomeRepositoryImpl(homeDataSource)
    }

    @Singleton
    @Provides
    fun provideGetWalletListUseCase(walletRepository: WalletRepository): GetWalletListUseCase {
        return GetWalletListUseCase(walletRepository)
    }

    @Singleton
    @Provides
    fun provideGetUserCreditUseCase(walletRepository: WalletRepository): GetUserCreditUseCase {
        return GetUserCreditUseCase(walletRepository)
    }

    @Singleton
    @Provides
    fun provideSubscribeTopicUseCase(fcmRepository: FCMRepositoryImpl): SubscribeTopicUseCase {
        return SubscribeTopicUseCase(fcmRepository)
    }

    @Singleton
    @Provides
    fun provideUpdateUserTokenUseCase(fcmRepository: FCMRepositoryImpl): UpdateUserTokenUseCase {
        return UpdateUserTokenUseCase(fcmRepository)
    }

    @Singleton
    @Provides
    fun provideGetQueriesUseCase(supportRepositoryImpl: SupportRepositoryImpl): GetQueriesUseCase {
        return GetQueriesUseCase(supportRepositoryImpl)
    }



    @Singleton
    @Provides
    fun provideGetItemDetailsUseCase(homeRepositoryImpl: HomeRepositoryImpl): GetItemDetailsUseCase {
        return GetItemDetailsUseCase(homeRepositoryImpl)
    }
}
