package com.codes_roots.imakeapp.di

import android.content.Context
import com.haroldadmin.cnradapter.NetworkResponseAdapterFactory
import com.codes_roots.dagger.di.module.ActivityBuildersModule
import com.codes_roots.imakeapp.BuildConfig
import com.codes_roots.imakeapp.create_vendor_driver_account.create_driver_account_fragment.CreateDriverAccountFragment
import com.codes_roots.imakeapp.create_vendor_driver_account.create_vendor_account_fragment.CreateVendorAccountFragment
import com.codes_roots.imakeapp.datalayer.APIServices
import com.codes_roots.imakeapp.datalayer.ExceptionsMapper
import com.codes_roots.imakeapp.datalayer.NetworkConnectionInterceptor
import com.codes_roots.imakeapp.helper.*
import com.codes_roots.imakeapp.helper.Constants.Companion.BASE_URL
import com.codes_roots.imakeapp.helper.Constants.Companion.SocketURL
import com.codes_roots.imakeapp.helper.Constants.Companion.TIMEOUT
import com.codes_roots.imakeapp.paymob_activity.PaymobActivity
import com.codes_roots.imakeapp.presentaion.add_address_fragment.AddAdressFragment
import com.codes_roots.imakeapp.presentaion.cartfragment.CartFragment
import com.codes_roots.imakeapp.presentaion.confirmedfragment.ConfirmedFragment
import com.codes_roots.imakeapp.presentaion.contactus.ContactUsFragment
import com.codes_roots.imakeapp.presentaion.edit_address_fragment.EditAddressFragment
import com.codes_roots.imakeapp.presentaion.favouritefragment.FavouriteFragment
import com.codes_roots.imakeapp.presentaion.itemdetails.ItemDetailsBottomSheet
import com.codes_roots.imakeapp.presentaion.itemdetails.ItemDetailsFragment
import com.codes_roots.imakeapp.presentaion.mainactivity.MainActivity
import com.codes_roots.imakeapp.presentaion.mainactivity.di.MainModule
import com.codes_roots.imakeapp.presentaion.map_activity.MapActivity
import com.codes_roots.imakeapp.presentaion.menufragment.MenuFragment
import com.codes_roots.imakeapp.presentaion.menufragment.staticfragment.StaticFragment
import com.codes_roots.imakeapp.presentaion.myaddress.AddressBottomFragment
import com.codes_roots.imakeapp.presentaion.myaddress.UserAddressesFragment
import com.codes_roots.imakeapp.presentaion.myaddress.cities.CitiesActivity
import com.codes_roots.imakeapp.presentaion.myorderDetails.AnswerFragment
import com.codes_roots.imakeapp.presentaion.myorders.MyOrdersFragment
import com.codes_roots.imakeapp.presentaion.mypoints.PointsFragment
import com.codes_roots.imakeapp.presentaion.mywallet.WalletFragment
import com.codes_roots.imakeapp.presentaion.notification.NotificationFragment
import com.codes_roots.imakeapp.presentaion.paymentfragment.PaymentFragment
import com.codes_roots.imakeapp.presentaion.ratefragment.RateFragment
import com.codes_roots.imakeapp.presentaion.register.RegisterActivity
import com.codes_roots.imakeapp.presentaion.register.loginfragment.LoginFragment
import com.codes_roots.imakeapp.presentaion.register.otp_fragment.EmailVerificationFragment
import com.codes_roots.imakeapp.presentaion.register.otp_fragment.MobileVerificationFragment
import com.codes_roots.imakeapp.presentaion.register.registerfragment.RegisterFragment
import com.codes_roots.imakeapp.presentaion.vendordetailsfragment.VendorDetailsFragment
import com.codes_roots.imakeapp.presentaion.stores_fragment.StoresFragment
import com.codes_roots.imakeapp.presentaion.sortfragment.SortFragment
import com.codes_roots.imakeapp.presentaion.support.SupportFragment
import com.codes_roots.imakeapp.presentaion.support.answerfragment.ComplaintsFragment
import dagger.BindsInstance
import dagger.Component
import dagger.Module
import dagger.Provides
import dagger.android.AndroidInjectionModule
import dagger.android.AndroidInjector
import dagger.android.DaggerApplication
import okhttp3.logging.HttpLoggingInterceptor
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import java.net.URISyntaxException

import java.util.concurrent.TimeUnit
import javax.inject.Singleton
import com.codes_roots.imakeapp.presentaion.filterfragment.FilterFragment
import com.codes_roots.imakeapp.presentaion.gallery_fragment.GalleryFragment
import com.codes_roots.imakeapp.presentaion.homefragment.HomeFragment
import com.codes_roots.imakeapp.presentaion.menufragment.MenuFragmentCompose
import com.codes_roots.imakeapp.presentaion.myorderDetails.MyOrdersDetailsFragment
import com.codes_roots.imakeapp.presentaion.profilefragment.NewProfileFragment
import com.codes_roots.imakeapp.presentaion.register.forgetpassword.ForgetPasswordFragment
import com.codes_roots.imakeapp.presentaion.reviews_fragment.ReviewsVendorFragment
import com.codes_roots.imakeapp.presentaion.search_fragment.SearchFragment
import com.codes_roots.imakeapp.presentaion.wallet_details.WalletDetailsFragment
import io.socket.client.IO
import io.socket.client.Socket
import okhttp3.*
import okhttp3.internal.http2.ConnectionShutdownException
import java.io.IOException
import java.net.SocketTimeoutException
import java.net.UnknownHostException


/**
 *
 * @author juancho.
 */
@Singleton
@Component(
    modules =
    [DispatcherModule::class,
        AndroidInjectionModule::class,
        APIModule::class,
        AppModule::class,
        FirebaseModule::class,
        FragmentFactoryModule::class,
        MainModule::class,
        ActivityBuildersModule::class,
        FragmentFactoryModule::class,
        ViewModelBuilderModule::class,
        SocketModule::class
    ]
)
interface AppComponent : AndroidInjector<DaggerApplication> {

    @Component.Factory
    interface Factory {
        fun create(@BindsInstance applicationContext: Context): AppComponent
    }


    fun inject(app: CitiesActivity)
    fun inject(app: MainActivity)
    fun inject(app: MapActivity)
    fun inject(app: HomeFragment)
    fun inject(app: EditAddressFragment)
    fun inject(app: StoresFragment)
    fun inject(app: SortFragment)
    fun inject(app: VendorDetailsFragment)
    fun inject(app: ForgetPasswordFragment)
    fun inject(app: MyOrdersDetailsFragment)
    fun inject(app: EmailVerificationFragment)
    fun inject(app: CartFragment)
    fun inject(app: ItemDetailsBottomSheet)
    fun inject(app: CreateDriverAccountFragment)
    fun inject(app: CreateVendorAccountFragment)
    fun inject(app: ItemDetailsFragment)
    fun inject(app: ConfirmedFragment)
    fun inject(app: PaymentFragment)
    fun inject(app: FilterFragment)
    fun inject(app: MobileVerificationFragment)
    fun inject(app: UserAddressesFragment)
    fun inject(app: GalleryFragment)
    fun inject(app: AddressBottomFragment)
    fun inject(app: AddAdressFragment)
    fun inject(app: MyOrdersFragment)
    fun inject(app: SearchFragment)
    fun inject(app: PointsFragment)
    fun inject(app: FavouriteFragment)
    fun inject(app: ReviewsVendorFragment)
    fun inject(app: RateFragment)
    fun inject(app: LoginFragment)
    fun inject(app: RegisterActivity)
    fun inject(app: RegisterFragment)
    fun inject(app: NotificationFragment)
    fun inject(app: MenuFragment)
    fun inject(app: StaticFragment)
    fun inject(app: ContactUsFragment)
    fun inject(app: WalletFragment)
    fun inject(app: SupportFragment)
    fun inject(app: AnswerFragment)
    fun inject(app: MenuFragmentCompose)
    fun inject(app: ComplaintsFragment)
    fun inject(app: NewProfileFragment)
    fun inject(app: WalletDetailsFragment)

    fun inject(app: PaymobActivity)

}

@Module
class APIModule constructor() {

    @Provides
    @Singleton
    fun provideInterceptor(): HttpLoggingInterceptor {
        return HttpLoggingInterceptor().apply {
            level = HttpLoggingInterceptor.Level.BODY
        }
    }

    @Provides
    @Singleton
    fun provideNetworkConnectionInterceptor(context: Context): NetworkConnectionInterceptor {
        return NetworkConnectionInterceptor(context)
    }


    @Singleton
    @Provides
    fun provideHttpClient(
        context: Context,
        loggingInterceptor: HttpLoggingInterceptor,
        networkConnectionInterceptor: NetworkConnectionInterceptor,
    ): OkHttpClient {
        try {
            val client = OkHttpClient()
                .newBuilder()
                .readTimeout(TIMEOUT.toLong(), TimeUnit.SECONDS)
                .writeTimeout(TIMEOUT.toLong(), TimeUnit.SECONDS)

                .addInterceptor { chain: Interceptor.Chain ->
                    try {
                        val originalRequest = chain.request()
                        var preferenceHelper = PreferenceHelper(context)
                        val builder = originalRequest.newBuilder()

                        builder.addHeader("lang", preferenceHelper.lang!!)
                        builder.addHeader("vendor_code", BuildConfig.VendorCode)
                        builder.addHeader("Content-Type", "application/json")
                        val shouldAddAuthHeaders =
                            originalRequest.headers["isAuthorizable"] != "false"
                        if (shouldAddAuthHeaders) {
                            builder.addHeader("Authorization", "Bearer " + preferenceHelper.token)
                        }
                        val newRequest = builder.build()
                        val response = chain.proceed(newRequest)
                        if (response.code == 403) {

                        }
                        response
                    } catch (e: IOException) {
                        throw  e

                    }
                }

                .connectionSpecs(listOf(ConnectionSpec.CLEARTEXT,
                    ConnectionSpec.Builder(ConnectionSpec.MODERN_TLS)
                        .allEnabledTlsVersions()
                        .allEnabledCipherSuites()
                        .build()))
                .addInterceptor(loggingInterceptor)

            return client.build()
        } catch (e: IOException) {
            throw  e
        }
    }

    @Singleton
    @Provides
    fun provideConverterFactory(): GsonConverterFactory =
        GsonConverterFactory.create()

    @Singleton
    @Provides
    fun provideRetrofit(
        okHttpClient: OkHttpClient,
        gsonConverterFactory: GsonConverterFactory,
    ): Retrofit {
        try {
            kotlin.runCatching {
                return Retrofit.Builder()
                    .baseUrl(BuildConfig.API_URL)
                    .client(okHttpClient)
                    .addConverterFactory(gsonConverterFactory)
                    .addCallAdapterFactory(NetworkResponseAdapterFactory())

                    .build()
            }.getOrElse { throw it }
        } catch (e: IOException) {
            throw  e
        }
    }

    @Singleton
    @Provides
    fun apiServices(provideRetrofit: Retrofit): APIServices {
        try {
            kotlin.runCatching {
                return provideRetrofit.create(APIServices::class.java)
            }.getOrElse { throw it }
        } catch (e: IOException) {
            throw  e
        }
    }

    @Singleton
    @Provides
    fun provideExceptionsMapper(context: Context) = ExceptionsMapper(context)
}

class LoggingInterceptor : Interceptor {

    @Throws(Exception::class)
    override fun intercept(chain: Interceptor.Chain): Response {
        val request = chain.request()
        try {
            val response = chain.proceed(request)

            val bodyString = response.body!!.string()

            return response.newBuilder()
                .body(ResponseBody.create(response.body?.contentType(), bodyString))
                .build()
        } catch (e: Exception) {
            e.printStackTrace()
            var msg = ""
            when (e) {
                is SocketTimeoutException -> {
                    msg = "Timeout - Please check your internet connection"
                }
                is UnknownHostException -> {
                    msg = "Unable to make a connection. Please check your internet"
                }
                is ConnectionShutdownException -> {
                    msg = "Connection shutdown. Please check your internet"
                }
                is IOException -> {
                    msg = "Server is unreachable, please try again later."
                }
                is IllegalStateException -> {
                    msg = "${e.message}"
                }
                else -> {
                    msg = "${e.message}"
                }
            }

            return Response.Builder()
                .request(request)
                .protocol(Protocol.HTTP_1_1)
                .code(999)
                .message(msg)
                .body(ResponseBody.create(null, "{${e}}")).build()
        }
    }
}

@Module
class SocketModule constructor() {

    @Singleton
    @Provides
    fun provideSocket(
    ): Socket {
        return try {
//creating socket instance
            IO.socket(SocketURL)
        } catch (e: URISyntaxException) {
            throw RuntimeException(e)
        }
    }


}