package com.codes_roots.imakeapp.helper

import android.content.Context
import android.util.Log
import com.codes_roots.imakeapp.datalayer.ExceptionsMapper
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import com.satafood.core.base.DataState
//import dagger.hilt.android.qualifiers.ApplicationContext
import retrofit2.Response

abstract class BaseDataSource(
    val context: Context,
    private val exceptionsMapper: ExceptionsMapper
) {

    protected suspend fun <T> getResult(call: suspend () -> Response<T>): DataState<T> {
        try {
            val response = call()
            if (response.isSuccessful) {
                val body = response.body()
                if (body != null) return DataState.success(body)
            }

            val gson = Gson()
            val type = object : TypeToken<Response<Any>>() {}.type
            var errorResponse: Response<Any>? =
                gson.fromJson(response.errorBody()?.charStream(), type)


            return error(
                "Error",
                response.code(),
                response.body()
            )
        } catch (e: Exception) {
            Log.d("TAG", "getResult: Exception" + e.toString())
            return error(exceptionsMapper.mapFromException(e))
        }
    }

    private fun <T> error(
        throwableMessage: String?,
        errorCode: Int = 0,
        data: T? = null
    ): DataState<T> {
        return DataState.error(
            DataState.HttpError(throwableMessage, errorCode),
            data
        )
    }

}