package com.codes_roots.imakeapp.helper


import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentFactory
import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import com.codes_roots.imakeapp.presentaion.add_address_fragment.AddAdressFragment
import com.codes_roots.imakeapp.presentaion.favouritefragment.FavouriteFragment
import com.codes_roots.imakeapp.presentaion.homefragment.HomeFragment
import com.codes_roots.imakeapp.presentaion.itemdetails.ItemDetailsBottomSheet
import com.codes_roots.imakeapp.presentaion.itemdetails.ItemDetailsFragment
import com.codes_roots.imakeapp.presentaion.menufragment.MenuFragment
import com.codes_roots.imakeapp.presentaion.myorderDetails.MyOrdersDetailsFragment
import com.codes_roots.imakeapp.presentaion.paymentfragment.PaymentFragment
import com.codes_roots.imakeapp.presentaion.vendordetailsfragment.VendorDetailsFragment
import com.codes_roots.imakeapp.presentaion.search_fragment.SearchFragment
import com.codes_roots.imakeapp.presentaion.stores_fragment.StoresFragment
import com.codes_roots.imakeapp.presentaion.sortfragment.SortFragment
import dagger.Binds
import dagger.MapKey
import dagger.Module
import dagger.multibindings.IntoMap
import javax.inject.Inject
import javax.inject.Provider
import kotlin.reflect.KClass


class ViewModelFactory @Inject constructor(
    private val creators: @JvmSuppressWildcards Map<Class<out ViewModel>, Provider<ViewModel>>,
) : ViewModelProvider.Factory {
    override fun <T : ViewModel> create(modelClass: Class<T>): T {
        var creator: Provider<out ViewModel>? = creators[modelClass]
        if (creator == null) {
            for ((key, value) in creators) {
                if (modelClass.isAssignableFrom(key)) {
                    creator = value
                    break
                }
            }
        }
        if (creator == null) {
            throw IllegalArgumentException("Unknown model class: $modelClass")
        }
        try {
            @Suppress("UNCHECKED_CAST")
            return creator.get() as T
        } catch (e: Exception) {
            throw RuntimeException(e)
        }
    }
}

@Module
abstract class ViewModelBuilderModule {

    @Binds
    abstract fun bindViewModelFactory(
        factory: ViewModelFactory, // Extends ViewModelProvider.Factory
    ): ViewModelProvider.Factory // Android Lifecycle ViewModel


}

// This key will enforce using a class that extends ViewModel
@Target(
    AnnotationTarget.FUNCTION, AnnotationTarget.PROPERTY_GETTER, AnnotationTarget.PROPERTY_SETTER
)
@Retention(AnnotationRetention.RUNTIME)
@MapKey
annotation class ViewModelKey(val value: KClass<out ViewModel>)

class MyFragmentFactory @Inject constructor(
    private val creator: Map<Class<out Fragment>, @JvmSuppressWildcards Provider<Fragment>>,
) : FragmentFactory() {

    override fun instantiate(classLoader: ClassLoader, className: String): Fragment {
        val fragmentClass = loadFragmentClass(classLoader, className)
        return creator[fragmentClass]?.get() ?: super.instantiate(classLoader, className)
    }

}

@Module
abstract class FragmentFactoryModule {

    @Binds
    abstract fun bindFragmentFactroy(factory: MyFragmentFactory): FragmentFactory

    @Binds
    @IntoMap
    @FragmentKey(HomeFragment::class)
    abstract fun bindMainFragment(fragment: HomeFragment): Fragment

    @Binds
    @IntoMap
    @FragmentKey(StoresFragment::class)
    abstract fun bindRestaurantFragment(fragment: StoresFragment): Fragment

    @Binds
    @IntoMap
    @FragmentKey(SortFragment::class)
    abstract fun bindSortFragment(fragment: SortFragment): Fragment

    @Binds
    @IntoMap
    @FragmentKey(VendorDetailsFragment::class)
    abstract fun bindRestaurantDetailsFragment(fragment: VendorDetailsFragment): Fragment

    @Binds
    @IntoMap
    @FragmentKey(ItemDetailsBottomSheet::class)
    abstract fun bindItemDetailsBottomSheet(bottomSheet: ItemDetailsBottomSheet): Fragment

    @Binds
    @IntoMap
    @FragmentKey(ItemDetailsFragment::class)
    abstract fun bindItemDetailsFragment(bottomSheet: ItemDetailsFragment): Fragment

    @Binds
    @IntoMap
    @FragmentKey(AddAdressFragment::class)
    abstract fun bindAddAddressFragment(bottomSheet: AddAdressFragment): Fragment

    @Binds
    @IntoMap
    @FragmentKey(PaymentFragment::class)
    abstract fun bindPaymentFragment(bottomSheet: PaymentFragment): Fragment
    @Binds
    @IntoMap
    @FragmentKey(SearchFragment::class)
    abstract fun bindSearchFragment(searchFragment: SearchFragment): Fragment

    @Binds
    @IntoMap
    @FragmentKey(MyOrdersDetailsFragment::class)
    abstract fun bindMyOrdersDetailsFragment(bottomSheet: MyOrdersDetailsFragment): Fragment

    @Binds
    @IntoMap
    @FragmentKey(FavouriteFragment::class)
    abstract fun bindMyFavouriteFragment(bottomSheet: FavouriteFragment): Fragment


    @Binds
    @IntoMap
    @FragmentKey(MenuFragment::class)
    abstract fun bindMenuFragment(menuFragment: MenuFragment): Fragment

}

@MapKey
annotation class FragmentKey(val clazz: KClass<out Fragment>)