package com.codes_roots.imakeapp.presentaion.cartfragment

import android.graphics.Color
import android.graphics.PorterDuff
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.view.isVisible
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.databinding.CartFragmentBinding
import com.codes_roots.imakeapp.helper.BaseApplication
import com.codes_roots.imakeapp.helper.Error_MotionToast
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.presentaion.ClickHandler
import com.codes_roots.imakeapp.presentaion.cartfragment.adapter.CartAdapter
import com.codes_roots.imakeapp.presentaion.cartfragment.mvi.CartIntents
import com.codes_roots.imakeapp.presentaion.cartfragment.viewmodel.CartViewModel
import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.presentaion.mainactivity.MainActivity
import kotlinx.coroutines.flow.collect
import javax.inject.Inject

class CartFragment : Fragment() {

    private lateinit var view: CartFragmentBinding

    @Inject
    lateinit var viewModelFactory: ViewModelProvider.Factory

    @Inject
    lateinit var Pref: PreferenceHelper

    val viewModel by viewModels<CartViewModel> { viewModelFactory }

    var reorderSubtotal: Double? = null
    lateinit var cartAdapter: CartAdapter

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        BaseApplication.appComponent.inject(this)
    }

    fun categoryRecycleView() {
        cartAdapter =
            CartAdapter(viewModel.state, viewModel.intents, requireContext() as MainActivity)
        view.cartRecycle.apply {
            adapter = cartAdapter
            layoutManager = LinearLayoutManager(requireContext())
            setHasFixedSize(true)
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?,
    ): View {
        view = DataBindingUtil.inflate(inflater, R.layout.cart_fragment, container, false)
        view.context = context as MainActivity
        view.viewModel = viewModel
        view.discountVal = 0.0
        try {
            val args = arguments
            reorderSubtotal = args!!.getDouble("subtotal")
            if (reorderSubtotal != null) {
                reorderSubtotal = requireArguments().getDouble("subtotal")
                view.total = reorderSubtotal
            } else {
                view.total = 0.0
            }
        } catch (e: Exception) {
        }
        view.continueBtn.background.setColorFilter(
            Color.parseColor(Pref.appPrimaryColor),
            PorterDuff.Mode.SRC_ATOP
        )

        view.backBtn.setOnClickListener { (context as MainActivity).onBackPressed() }
        try {
        //call server data
        if (Pref.retriveCartItemsValue().isEmpty())
            viewModel.intents.trySend(CartIntents.NoItem(viewModel.state.value!!.copy()))
        else
            viewModel.intents.trySend(
                CartIntents
                    .LoadData(
                        viewModel.state.value!!.copy(
                            CartItem = Pref.retriveCartItemsValue() as ArrayList,
                            noItemInCart = false
                        ), Pref.restaurantId, Pref.areaId, arrayOf(25, 10)

                    )
            )
        }catch (e:Exception){

        }

        categoryRecycleView()
        //preferenceHelper.clearCart()
        //    viewModel.intents.trySend(CartIntents.AddItem(viewModel.state.value, MenuCategoriesItem()))
        //عند عمل trysend من داخل الادابتر التاكد من تغيير الايتمز حتي يتم ارسال الintent من الجزء الخاص بال MenuDiffCallback
        //flow matuable لا تقوم باي تحديث عندما تكون الداتا غير مختلفة
        lifecycleScope.launchWhenStarted {
//            view.progress.isVisible = true
            viewModel.state.collect { cartViewState ->
                kotlin.runCatching {
                    val restaurantDetailsData = cartViewState?.ItemCartData?.restaurantDetails
                    //   val cartConfigurations = cartViewState!!.CartConfigurationsData!!.Confirgrations


                    if (cartViewState?.Errors != null) {
                        cartViewState.Errors.let {
                            when (it) {
                                is UserError.NetworkError -> Error_MotionToast(
                                    it.throwable.cause.toString(),
                                    requireActivity()
                                )
                            }
                        }
                        viewModel.intents.send(CartIntents.DisplayErrors(cartViewState))
                    } else {
                        try {
                        view.progresss.isVisible = cartViewState?.progress!!

                        if (Pref.retriveCartItemsValue().isEmpty()) {
                            view.noItemInCartView.isVisible = cartViewState.noItemInCart
                            view.animationView.playAnimation()
                        } else
                            view.noItemInCartView.isVisible = cartViewState!!.noItemInCart

                        if (reorderSubtotal != null) {
                            view.total = reorderSubtotal
                        } else {
                            view.total = (cartViewState.SubTotal)
                        }
                        view.totalCartValue.text =
                            cartViewState?.SubTotal.toString() + Pref.currency ?: ""

                        if (restaurantDetailsData != null) {
                            view.listener = ClickHandler()

                            view.data = cartViewState.ItemCartData
                            viewModel.data = restaurantDetailsData
                            cartAdapter.submitList(cartViewState.CartItem)

                            if (view.description.text == "")
                                view.description.apply {
                                    isSelected = true
                                    restaurantDetailsData.first()
                                        .cuisines!!.forEachIndexed { index, cuisine ->
                                            if (restaurantDetailsData.first().cuisines!!.lastIndex != index)
                                                view.description.append(cuisine.name + " ~")
                                            else view.description.append(cuisine.name)
                                        };
                                }
                        }

              }catch (e:Exception){

              }

              }
                }.getOrElse {


                    //     view.progres.isVisible = true
                    //  Error_MotionToast(it.cause.toString(),requireActivity())
                }

            }
        }

        return view.root
    }


}