package com.codes_roots.imakeapp.presentaion.cartfragment.adapter

import android.annotation.SuppressLint
import android.content.Context
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.databinding.DataBindingUtil
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.databinding.CartItemAdapterBinding
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.helper.WARN_MotionToast
import com.codes_roots.imakeapp.presentaion.ClickHandler
import com.codes_roots.imakeapp.presentaion.cartfragment.mvi.CartIntents
import com.codes_roots.imakeapp.presentaion.cartfragment.mvi.CartViewState
import com.codes_roots.imakeapp.presentaion.mainactivity.MainActivity
import com.satafood.core.entities.restaurantdetails.MenuCategoriesItem
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.flow.MutableStateFlow

class CartAdapter(
    var viewModel: MutableStateFlow<CartViewState?>,
    var Intent: Channel<CartIntents>,
    var context: MainActivity?
) : ListAdapter<MenuCategoriesItem, MenuViewHolder>(MenuDiffCallback()) {

    var Pref: PreferenceHelper? = null

    val scope = CoroutineScope(Dispatchers.Main)

    override fun onBindViewHolder(holder: MenuViewHolder, position: Int) {
        holder.bind(context, Intent, currentList[position])

        try {
            val state = viewModel.value
            holder.binding.quintityPlus.setOnClickListener {
                Intent.trySend(CartIntents.AddItem(state, currentList[position], position))
                notifyItemChanged(position)
                notifyDataSetChanged()

            }
            holder.binding.quintityMinus.setOnClickListener {
                if (currentList[position].quintity > 1){
                    Intent.trySend(CartIntents.MinusItem(state, currentList[position], position))
                   notifyItemChanged(position)
                    notifyDataSetChanged()


                }else{
                    WARN_MotionToast(context!!.getString(R.string.minimum1),context!!)

                }
            }

            holder.binding.RemoveCell.setOnClickListener {
                try {
                    Intent.trySend(
                        CartIntents.RemoveItem(
                            (state!!.copy(noItemInCart = false, progress = true)),
                            currentList[position], position))

                    context?.Pref?.removeItemFromCart(position)
                    notifyItemRemoved(position)
                    notifyDataSetChanged()

                }catch (e:Exception){

                }

            }
            /// item options
        if (holder.binding.itemOptions.text == "")
            currentList[position]!!.menu_options?.forEachIndexed { index, value ->
                if (currentList[position].menu_options!!.lastIndex != index)
                    holder.binding.itemOptions.append(value.name + "-")
                else holder.binding.itemOptions.append(value.name)
            }
        } catch (e: Throwable) {

        }
    }

    override fun onCreateViewHolder(p0: ViewGroup, p1: Int): MenuViewHolder {
        val binding: CartItemAdapterBinding = DataBindingUtil.inflate(
            LayoutInflater.from(p0.context),
            R.layout.cart_item_adapter, p0, false
        )
        return MenuViewHolder(binding)
    }
}

private class MenuDiffCallback : DiffUtil.ItemCallback<MenuCategoriesItem>() {

    override fun areItemsTheSame(oldItem: MenuCategoriesItem, newItem: MenuCategoriesItem) =
        oldItem.quintity == newItem.quintity

    @SuppressLint("DiffUtilEquals")
    override fun areContentsTheSame(oldItem: MenuCategoriesItem, newItem: MenuCategoriesItem) =
        oldItem == newItem
}

class MenuViewHolder(
    val binding: CartItemAdapterBinding
) : RecyclerView.ViewHolder(binding.root) {


    fun bind(context: Context?, viewModel: Channel<CartIntents>, data: MenuCategoriesItem) {
        binding.listener = ClickHandler()
        binding.data = data
        binding.context = context as MainActivity?
    }


}
