package com.codes_roots.imakeapp.presentaion.cartfragment.mvi

import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.repository.cart.CartDataRepo
import com.satafood.core.entities.restaurantdetails.MenuCategoriesItem
import com.satafood.core.entities.restaurantdetails.StoreDetailsModel
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first
import kotlin.math.roundToInt


suspend fun mapIntentToViewState(
    intent: CartIntents,
    Datarepo: CartDataRepo,
    loadMainData: suspend () -> Flow<Result<StoreDetailsModel>> = {
        Datarepo.getCartList(intent.restaurantId!!, intent.areaId!!, intent.ItemList!!)
    },

) = when (intent) {
    is CartIntents.LoadData -> proceedWithInitialize(loadMainData, intent)
    is CartIntents.DisplayErrors -> intent.viewState.copy(Errors = null)
    is CartIntents.AddItem -> updateAddItem(intent, intent.item.quintity + 1)
    is CartIntents.MinusItem -> updateMinusItem(intent, intent.item.quintity - 1)
    is CartIntents.RemoveItem -> RemoveItem(intent)
    is CartIntents.NoItem -> intent.viewState.copy(noItemInCart = true)
}

private suspend fun proceedWithInitialize(
    loadCart: suspend () -> Flow<Result<StoreDetailsModel>>,
    intent: CartIntents,
): CartViewState {
    val response = loadCart()
    val data = response.first()



    return runCatching {
        val totalSum = MakeTotalPluseOptions(intent.viewState!!.CartItem!!)
        intent.viewState!!.copy(ItemCartData = data.getOrThrow(),
            SubTotal = (totalSum * 10.0).roundToInt() / 10.0,
            delivery_cost = data.getOrNull()!!.restaurantDetails?.first()?.delivery_cost!! ,
            progress = false,
            AllTaxes = totalSum * data.getOrNull()!!.restaurantDetails?.first()?.taxes!! / 100)
    }
        .getOrElse {
            intent.viewState!!.copy(Errors = UserError.NetworkError(it as Throwable))
        }
}


fun updateMinusItem(intent: CartIntents, quintity: Int): CartViewState {
    var Cartarray = intent.viewState!!.CartItem!!
    var CartItem = Cartarray[intent.index!!].copy(quintity = quintity,
        total = intent.item!!.total!! - intent.item!!.price!!)
    Cartarray[intent.index!!] = CartItem
    var TotalSum = MakeTotalPluseOptions(Cartarray)
    return intent.viewState!!.copy(CartItem = Cartarray,
        SubTotal = TotalSum,
        AllTaxes = TotalSum * intent.viewState!!.ItemCartData!!.restaurantDetails?.first()?.taxes!! / 100)
}

fun updateAddItem(intent: CartIntents, quintity: Int): CartViewState {
    var Cartarray = intent.viewState!!.CartItem!!
    var CartItem = Cartarray[intent.index!!].copy(quintity = quintity,
        total = intent.item!!.total!! + intent.item!!.price!!)
    Cartarray[intent.index!!] = CartItem
    var TotalSum = MakeTotalPluseOptions(Cartarray)
    return intent.viewState!!.copy(CartItem = Cartarray,progress = false,
        SubTotal = TotalSum, AllTaxes = TotalSum
                * intent.viewState!!.ItemCartData!!.restaurantDetails?.first()?.taxes!! / 100)
}


fun RemoveItem(intent: CartIntents): CartViewState {
    val Cartarray = intent.viewState!!.CartItem!!
    Cartarray.removeAt(intent.index!!)
    val TotalSum = MakeTotalPluseOptions(Cartarray)
    return intent.viewState!!.copy(CartItem = Cartarray,progress = false ,
        SubTotal = TotalSum, AllTaxes = TotalSum

                * intent.viewState!!.ItemCartData!!.restaurantDetails?.first()?.taxes!! / 100)

}

fun MakeTotalPluseOptions(Items: ArrayList<MenuCategoriesItem>) =

    Items.sumOf {( it.total!!  * 10.0).roundToInt() / 10.0 }

