package com.codes_roots.imakeapp.presentaion.cartfragment.viewmodel

import android.util.Log
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.meem.physio.util.SingleLiveEvent
import com.codes_roots.imakeapp.di.IoDispatcher
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.presentaion.cartfragment.mvi.CartIntents
import com.codes_roots.imakeapp.presentaion.cartfragment.mvi.CartViewState
import com.codes_roots.imakeapp.presentaion.cartfragment.mvi.mapIntentToViewState
import com.codes_roots.imakeapp.presentaion.mainactivity.BaseViewModel
import com.codes_roots.imakeapp.repository.cart.CartDataRepo
import com.codes_roots.imakeapp.repository.cart.CartRemoteDataSource
import com.satafood.core.entities.login.LoginModel
import com.satafood.core.entities.ordermodel.OrderModel
import com.satafood.core.entities.paymentways.PaymentWaysItem
import com.satafood.core.entities.paymentways.VoucherModel
import com.satafood.core.entities.paymob.MakePaymentOrderIntegration
import com.satafood.core.entities.paymob.PayMobToken
import com.satafood.core.entities.paymob.Payment
import com.satafood.core.entities.paymob.Token
import com.satafood.core.entities.restaurantdetails.RestuarantsDetails
import io.reactivex.disposables.CompositeDisposable
import kotlinx.coroutines.*
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.flow.receiveAsFlow
import org.json.JSONException
import org.json.JSONObject
import java.io.IOException
import javax.inject.Inject


class CartViewModel @Inject constructor(
    private val DateRepoCompnay: CartDataRepo, private val Datasources: CartRemoteDataSource,
    @IoDispatcher private val ioDispatcher: CoroutineDispatcher

) : BaseViewModel<CartViewState>() {

    val intents: Channel<CartIntents> = Channel(Channel.UNLIMITED)

    protected val uiState: MutableStateFlow<CartViewState?> = MutableStateFlow(CartViewState())
    var makePaymentOrderIntegration: MutableLiveData<MakePaymentOrderIntegration> =
        MutableLiveData<MakePaymentOrderIntegration>()
    var paymentResponseLD : MutableLiveData<Payment>? = null
    var InActiveView = MutableLiveData<Int>()

    val state: MutableStateFlow<CartViewState?> get() = uiState

    @Inject
    lateinit var Pref: PreferenceHelper

    private var job: Job? = null

    var mCompositeDisposable = CompositeDisposable()

    var data: List<RestuarantsDetails>? = null

    var paymentJob: Job? = null

    val PaymentWaysLD = MutableLiveData<ArrayList<PaymentWaysItem>>()

    var OrderInfoLD: MutableLiveData<OrderModel>? = null

    var errorMessage = SingleLiveEvent<String>()
    var isPaymentError = false
    val loading = MutableLiveData<Boolean>()
    var authLD: MutableLiveData<LoginModel>? = null
    var VoucherLD: MutableLiveData<VoucherModel>? = null
    var PayMobTokenLD: MutableLiveData<Token>? = null

    init {
        OrderInfoLD = MutableLiveData()
        authLD = MutableLiveData()
        VoucherLD = MutableLiveData()
        PayMobTokenLD = MutableLiveData()
        paymentResponseLD = MutableLiveData()
        errorMessage = SingleLiveEvent()
        getIntent()
// لازم ابعت viewstate كامل واتحكم بكل option حسب المطلوب وارساله مرة اخري واستقباله هنا او في ال view
        //   uiState.value = CartViewState().copy(progress = true)
        // launchNextViewStateJob(intents.value)
    }

    fun getIntent() {
        job = viewModelScope.launch() {
            intents.receiveAsFlow().collect {
                uiState.value = mapIntentToViewState(it, DateRepoCompnay)
            }
        }
    }

    val coroutineExceptionHandler = CoroutineExceptionHandler { _, throwable ->
        throwable.printStackTrace()
    }

    fun getPaymentWays() {
        viewModelScope.launch(Dispatchers.IO) {
            try {
                val response = Datasources.getPaymentWays()
                if (response.isSuccessful) {
                    PaymentWaysLD.postValue(response.body())
                } else {
                    isPaymentError = true
                }
            } catch (e: IOException) {
                Log.i("CatchingCrash", "getPaymentWays: ${e.message}")
            }

        }

    }

    //addUsersAddress
    fun PaymentData(data:Payment) {
        job = CoroutineScope(ioDispatcher).launch {
            var token = PayMobToken()
            val response = Datasources.GetPaymentkey(data)
            withContext(Dispatchers.Main) {
                if (response.isSuccessful) {
                    paymentResponseLD?.postValue(response.body())
                } else {
                    try {
                        errorMessage.postValue(response.errorBody().toString())
                    } catch (e: JSONException) {
                        e.printStackTrace()
                    }
                }
            }
        }
    }
    //addUsersAddress
    fun checkVoucherRequest(voucherModel: VoucherModel?) {
        job = CoroutineScope(ioDispatcher).launch {
            val response = Datasources.checkVoucherRequest(voucherModel!!)
            withContext(Dispatchers.Main) {
                if (response.isSuccessful) {
                    VoucherLD?.postValue(response.body())
                } else {
                    if (response.code() == 400) {
                        // onError("Error : ${response.message()} ")
                        var jsonObject: JSONObject? = null
                        try {
                            jsonObject = JSONObject(response.errorBody()!!.string())
                            val userMessage: String = jsonObject.getString("message")
                            errorMessage.postValue(userMessage)

                            val internalMessage: String = jsonObject.getString("message")
//                            val errorCode: String = jsonObject.getString("errorCode")
                        } catch (e: JSONException) {
                            e.printStackTrace()
                        }
                    }
                }
            }
        }

    }
    //addUsersAddress
    fun getPaymobToken(auth_key : String?) {
        job = CoroutineScope(ioDispatcher).launch {
            var token = PayMobToken()
            token.api_key = auth_key
            val response = Datasources.getPaymobToken(token!!)
            withContext(Dispatchers.Main) {
                if (response.isSuccessful) {
                    PayMobTokenLD?.postValue(response.body())
                } else {
                        try {
                            errorMessage.postValue(response.errorBody().toString())
                        } catch (e: JSONException) {
                            e.printStackTrace()
                        }
                    }
                }
            }
        }



    //addUsersAddress
    fun addOrder(orderInfo: OrderModel?) {
        kotlin.runCatching {
            job = CoroutineScope(Dispatchers.IO).launch {
                val response = Datasources.AddOrder(orderInfo!!)
                withContext(Dispatchers.Main + coroutineExceptionHandler) {
                    if (response.isSuccessful) {

                        OrderInfoLD?.postValue(response.body())

                    } else {
                        if (response.code() == 405) {
                            var jsonObject: JSONObject? = null
                            try {
                                jsonObject = JSONObject(response.errorBody()!!.string())
                                val mobile: String = jsonObject.getString("mobile")
                                var email : String = jsonObject.getString("email")
//                                var token : String = jsonObject.getString("token")

                                Pref.userPhone = mobile
                                Pref.email = email
//                                Pref.token = token

                                InActiveView.postValue(1)
                            } catch (e: JSONException) {
                                e.printStackTrace()
                            }
                        }else {
                            onError("Error : ${response.message()} ")

                        }
//                        onError("Error : ${response.message()} ")
                    }
                }
            }
        }.getOrElse { throw it }
    }


    //authentication
    fun sendNotificationToDevice(loginModel: LoginModel?) {
        val coroutineExceptionHandler = CoroutineExceptionHandler { _, throwable ->
            throwable.printStackTrace()
        }
        job = CoroutineScope(Dispatchers.Default + coroutineExceptionHandler).launch {
            val response = Datasources.sendNotificationToDevice(loginModel!!)
            withContext(Dispatchers.Main + coroutineExceptionHandler) {
                if (response.isSuccessful) {
                    authLD?.postValue(response.body())

                } else {
                    onError("Error : ${response.message()} ")
                }
            }
        }

    }


    fun  MakePaymentOrderIntegration(loginModel:HashMap<String,Any>) {
        job =   CoroutineScope(Dispatchers.IO).launch {
            try {
                val response = Datasources.MakePaymentOrderIntegration(loginModel!!)
                if (response.isSuccessful) {
                    makePaymentOrderIntegration.postValue(response.body())
                } else {
                    isPaymentError = true
                }
            } catch (e: IOException) {
                Log.i("CatchingCrash", "getPaymentWays: ${e.message}")
            }

        }

    }


    private fun onError(message: String) {
        errorMessage.postValue(message)
        loading.value = false
    }

    override fun onCleared() {
        super.onCleared()
        mCompositeDisposable.dispose()
        mCompositeDisposable.clear()
        paymentJob?.cancel()
        job?.cancel()
    }
}