package com.codes_roots.imakeapp.presentaion.favouritefragment

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.view.isVisible
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentFactory
import androidx.fragment.app.viewModels
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.databinding.FavouriteFragmentBinding
import com.codes_roots.imakeapp.helper.BaseApplication
import com.codes_roots.imakeapp.helper.SUCCESS_MotionToast
import com.codes_roots.imakeapp.helper.WARN_MotionToast
import com.codes_roots.imakeapp.presentaion.ClickHandler
import com.codes_roots.imakeapp.presentaion.favouritefragment.adapter.Favourites_Adapter
import com.codes_roots.imakeapp.presentaion.favouritefragment.mvi.FavouriteViewModel
import com.codes_roots.imakeapp.presentaion.favouritefragment.mvi.MainIntent
import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.presentaion.mainactivity.MainActivity
import com.codes_roots.imakeapp.presentaion.register.RegisterActivity
import com.satafood.core.entities.favourite.FavouriteModelItem
import kotlinx.coroutines.flow.collect
import javax.inject.Inject


open class FavouriteFragment @Inject constructor(private val fragmentFactory : FragmentFactory) : Fragment() {

    @Inject
    lateinit var viewModelFactory: ViewModelProvider.Factory

    private val viewModel by viewModels<FavouriteViewModel> { viewModelFactory }
    var data:  ArrayList<FavouriteModelItem>?=null

    lateinit var FavouritesAdapter: Favourites_Adapter
    private lateinit var view: FavouriteFragmentBinding


    override fun onCreate(savedInstanceState: Bundle?) {
        childFragmentManager.fragmentFactory = fragmentFactory
        super.onCreate(savedInstanceState)
        if (savedInstanceState == null)  {
            BaseApplication.appComponent.inject(this)
        }

        if(( context as MainActivity).Pref.token.isNullOrEmpty()){
            WARN_MotionToast(requireActivity().getString(R.string.loginFirst),requireActivity())
            ClickHandler().switchToActivity(requireContext(), RegisterActivity())
        }

    }
    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {

        view = DataBindingUtil.inflate(inflater,
                R.layout.favourite_fragment, container, false)
        getAllData()
        favouritesRecycleView()
        return view.root
    }


    fun favouritesRecycleView() {
        FavouritesAdapter = Favourites_Adapter(viewModel, requireContext(),this)
        view.favoritesRecycle.apply {
            adapter = FavouritesAdapter;
            isNestedScrollingEnabled = false
            setHasFixedSize(true)
        }
    }


    fun getAllData() {
        lifecycleScope.launchWhenStarted {
            viewModel.deletestate?.collect {
                if (it == true) {

                    SUCCESS_MotionToast("تم المسح بنجاح",requireActivity())
                }
            }
            viewModel.state.collect {
                if (it != null) {

                    if (it.error != null) {
                        it.error?.let {
                            when (it) {
                                is UserError.InvalidId -> "Invalid id"
                                is UserError.NetworkError -> it.throwable.message
                                UserError.ServerError -> "Server error"
                                UserError.Unexpected -> "Unexpected error"
                                is UserError.UserNotFound -> "User not found"
                                UserError.ValidationFailed -> "Validation failed"
                                else -> {}
                            }
                        }

                    } else {

                        if (it.progress == true) {
                            viewModel.intents.send(MainIntent.Initialize(it,1))
                        } else {
                            view.progress.visibility = View.GONE

                            if (it.data?.isNotEmpty() == true){
                                data = it.data
                            ///newOffersAdapter

                            FavouritesAdapter.submitList(data)
                            }
                                else {
                                view.noOrderFoundLayout.isVisible = it.noFavouriteYet!!
                                view.animationView.isVisible = true
                                view.animationView.playAnimation()

                            }

                        }
                    }
                }

            }
        }
    }
}