package com.codes_roots.imakeapp.presentaion.filterfragment

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.databinding.DataBindingUtil
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.databinding.SortFragmentBinding
import com.codes_roots.imakeapp.helper.BaseApplication
import com.codes_roots.imakeapp.presentaion.ClickHandler
import com.codes_roots.imakeapp.presentaion.mainactivity.MainActivity
import com.codes_roots.imakeapp.presentaion.stores_fragment.mvi.RestaurantsListIntent
import com.codes_roots.imakeapp.presentaion.stores_fragment.viewmodel.RestaurantsListViewModel
import com.google.android.material.bottomsheet.BottomSheetDialogFragment
import com.codes_roots.imakeapp.helper.WARN_MotionToast
import javax.inject.Inject

class FilterFragment @Inject constructor(var viewmodel: RestaurantsListViewModel) :
    BottomSheetDialogFragment() {

    companion object {
        const val TAG = "TownBottomSheetDialogFragment"
    }

    lateinit var filterListAdapter: FilterListAdapter

    lateinit var view: SortFragmentBinding
    var SelectedSortOption = 0
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        BaseApplication.appComponent.inject(this)
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        view = DataBindingUtil.inflate(
            inflater,
            R.layout.sort_fragment, container, false
        )
        view.listener = ClickHandler()
        view.context = context as MainActivity
        view.viewModel = viewmodel

        var viewstate = viewmodel.state.value

        view.ButtonClick.setOnClickListener {
            if (viewstate?.BranchesUseCaseData!= null) {
                when (SelectedSortOption) {
                    0 -> {
                        viewmodel.intents.trySend(RestaurantsListIntent.FilterByTopRated(viewstate!!.copy(
                            filterText = getString(R.string.filterByRate))))
                    }
                    1 -> {
                        viewmodel.intents.trySend(RestaurantsListIntent.FilterFreeDeliveryVendors(
                            viewstate.copy(filterText = getString(R.string.free_delivery_vendor))))
                    }
                    2 -> {
                        viewmodel.intents.trySend(RestaurantsListIntent.FilterOpenedVendors(viewstate!!.copy(
                            filterText = getString(R.string.opend_restaurant))))
                    }
                    3 -> {
                        viewmodel.intents.trySend(RestaurantsListIntent.SortByDeliveryRating(viewstate))
                    }
                }
            }else{
                WARN_MotionToast(requireActivity().getString(R.string.noResturant),requireActivity())
            }
            this.dismiss()
        }

        view.dismiss.setOnClickListener {
            this.dismiss()
        }
        val data = arrayListOf(
            getString(R.string.filterByRate),
            getString(R.string.free_delivery_vendor),
            getString(R.string.opend_restaurant)
        )

        filterListAdapter = FilterListAdapter(viewmodel, this, data)

        view.sortRecycle.apply { adapter = filterListAdapter }


        return view.root
    }

}