package com.codes_roots.imakeapp.presentaion.gallery_fragment.viewmodel

import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.repository.gallery.GalleryDataSource
import com.codes_roots.imakeapp.repository.google.GoogleDataSource
import com.satafood.core.entities.branches_locations.BranchesLocation
import com.satafood.core.entities.gallery.BranchesGalleryModelItem
import com.satafood.core.entities.gallery.Gallery
import com.satafood.core.entities.google.Google
import com.satafood.core.entities.restaurantslist.BranchesModelListItem
import io.reactivex.disposables.CompositeDisposable
import kotlinx.coroutines.*
import javax.inject.Inject

class GalleryViewModel @Inject constructor(
    private val Datasources: GalleryDataSource,
) : ViewModel() {

    @Inject
    lateinit var Pref: PreferenceHelper
    private var job: Job? = null
    var mCompositeDisposable = CompositeDisposable()

    var galleryBranchesLD: MutableLiveData<ArrayList<BranchesGalleryModelItem>>? = null


    val errorMessage = MutableLiveData<String>()
    val loading = MutableLiveData<Boolean>()


    init {
        galleryBranchesLD = MutableLiveData()
    }

    val coroutineExceptionHandler = CoroutineExceptionHandler { _, throwable ->
        throwable.printStackTrace()
    }
//
    fun getBranchesLocation() {
        job = CoroutineScope(Dispatchers.IO + coroutineExceptionHandler).launch {
            val response = Datasources.getBranchesGallery()
            withContext(Dispatchers.Main + coroutineExceptionHandler) {
                if (response.isSuccessful) {
                    galleryBranchesLD!!.value = response.body()!!
                } else {
                    onError("Error : ${response.message()} ")
                }
            }
        }

    }



    private fun onError(message: String) {
        errorMessage.value = message
        loading.value = false
    }

    override fun onCleared() {
        super.onCleared()
        mCompositeDisposable.dispose()
        mCompositeDisposable.clear()
        job?.cancel()
    }
}