package com.codes_roots.imakeapp.presentaion.homefragment

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.meem.physio.util.SingleLiveEvent
import com.codes_roots.imakeapp.presentaion.mywallet.mvi.HomeIntent
import com.satafood.core.base.DataState
import com.codes_roots.imakeapp.repository.home.GetItemDetailsUseCase
import com.satafood.core.entities.restaurantdetails.MenuCategoriesItem
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch
import javax.inject.Inject

class HomeViewModel @Inject constructor(
    private val getItemDetailsUseCase: GetItemDetailsUseCase) : ViewModel() {

    val intentChannel = Channel<HomeIntent>(Channel.UNLIMITED)

    private val _itemDetails= SingleLiveEvent<DataState<List<MenuCategoriesItem>>>()


    val itemDetails = _itemDetails

    init {
        processIntent()

    }

    private fun processIntent() {
        viewModelScope.launch {
            intentChannel.consumeAsFlow().collect {
                when (it) {
                    is HomeIntent.GetItemDetails -> getItemDetails(it.itemId)
                }
            }
        }
    }

    private fun getItemDetails(itemId: Int) {
        viewModelScope.launch() {
            getItemDetailsUseCase(itemId)
                .buffer()
                .onEach { _itemDetails.postValue(it) }
                .launchIn(viewModelScope)
        }
    }

}