package com.codes_roots.imakeapp.presentaion.itemdetails

import android.app.Dialog
import android.graphics.Color
import android.graphics.PorterDuff
import android.os.Bundle
import android.view.*
import androidx.coordinatorlayout.widget.CoordinatorLayout
import androidx.databinding.DataBindingUtil
import androidx.databinding.ViewDataBinding
import com.google.android.material.bottomsheet.BottomSheetBehavior
import com.google.android.material.bottomsheet.BottomSheetDialog
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.helper.BaseApplication
import com.codes_roots.imakeapp.helper.Error_MotionToast
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.helper.SUCCESS_MotionToast
import com.codes_roots.imakeapp.presentaion.ClickHandler
import com.codes_roots.imakeapp.presentaion.mainactivity.MainActivity
import com.codes_roots.imakeapp.presentaion.vendordetailsfragment.viewmodel.RestaurantDetailsViewModel
import com.google.android.material.bottomsheet.BottomSheetDialogFragment
import com.codes_roots.imakeapp.databinding.ItemDetailsBottomFragmentBinding
import com.satafood.core.entities.restaurantdetails.MenuCategoriesItem
import kotlinx.android.synthetic.main.alert_dialog.*
import javax.inject.Inject

class ItemDetailsBottomSheet @Inject constructor(
    var viewmodel: RestaurantDetailsViewModel,
) : BottomSheetDialogFragment() {

    @Inject
    lateinit var Pref: PreferenceHelper

    var Total: Double = 0.0
    var cartSize = 0
    var FromHome = 0
    var ResturantId: Int = 0
    lateinit var bottomSheetBehavior: BottomSheetBehavior<View>
    lateinit var dialog: BottomSheetDialog
    lateinit var rootView: ViewDataBinding

    var Item: MenuCategoriesItem? = null

    companion object {
        const val TAG = "TownBottomSheetDialogFragment"
    }

    lateinit var optionListAdapter: Option_List_Adapter

    lateinit var view: ItemDetailsBottomFragmentBinding
    var data: MenuCategoriesItem? = null

    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
        dialog = super.onCreateDialog(savedInstanceState) as BottomSheetDialog
          return dialog
    }
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (savedInstanceState == null) {
            BaseApplication.appComponent.inject(this)
        }
        Item = arguments?.get("Item") as MenuCategoriesItem

        ResturantId = arguments?.getInt("ResturantId")!!

        viewmodel.OptionsArrayVariable?.value = 0.0
        viewmodel.OptionsArray.clear()
             FromHome = arguments?.getInt("FromHome")!!

    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?,
    ): View? {
        view = DataBindingUtil.inflate(
            inflater,
            R.layout.item_details_bottom_fragment, container, false
        )
        view.listener = ClickHandler()
        view.context = context as MainActivity
        view.data = Item
        if (FromHome == 1) {
            view.RestName.visibility = View.VISIBLE
        }else {
            view.RestName.visibility = View.GONE

        }
        view.ButtonClick.background.setColorFilter(
            Color.parseColor(Pref.appPrimaryColor),
            PorterDuff.Mode.SRC_ATOP
        )
        Total = if (Item!!.discount!=null){
            calculatePriceAfterDiscount()
        }else{
            Item!!.price?:0.0
        }


        view.TotalItem.text =
            Total.toString().plus(Pref.currency)

        view.CloseFragment.setOnClickListener { activity?.supportFragmentManager?.popBackStack(); }

        view.quintityPlus.setOnClickListener { increaseInteger() }
        view.quintityMinus.setOnClickListener { decreaseInteger() }

        if (Item!!.menu_options_topics.isNullOrEmpty()) {
            //  view.additionsScrollView.isVisible = false
        } else {
            optionListAdapter =
                Option_List_Adapter(viewmodel!!, requireContext(), Item!!.menu_options_topics!!)
            view.optionRecycle.apply { adapter = optionListAdapter }
        }

        view.ButtonClick.setOnClickListener {
            cartSize = Pref.retriveCartItemsValue().size
            when (Pref.restaurantId) {
                0 -> {
                    addItemToCart()
                    viewmodel.quantity!!.value = (++cartSize)

                }
                ResturantId -> {
                    addItemToCart()
                    viewmodel.quantity!!.value = (++cartSize)
                }
                else -> {
                    val pDialog = Dialog(requireContext())
                    pDialog.getWindow()?.requestFeature(Window.FEATURE_NO_TITLE);
                    pDialog.setContentView(
                        getLayoutInflater().inflate(
                            R.layout.alert_dialog,
                            null
                        )
                    );
                    pDialog.done.setOnClickListener {
                        Pref.clearCart()
                        pDialog.hide()
                        Pref.restaurantId = 0
                        viewmodel.quantity!!.value = 0

                    }
                    pDialog.cancel.setOnClickListener {
                        pDialog.hide()
                    }
                    pDialog.show();
                }
            }

        }

        view.CloseFragment.setOnClickListener {
            dismiss()
        }

        return view.root
    }
    private fun calculatePriceAfterDiscount(): Double {
        val priceBeforeDiscount = Item!!.price!!
        val discountPercentage = Item!!.discount!!
        val discountValue = priceBeforeDiscount * discountPercentage / 100

        return priceBeforeDiscount - discountValue
    }
    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        bottomSheetBehavior = BottomSheetBehavior.from(view.parent as View)

        bottomSheetBehavior.state = BottomSheetBehavior.STATE_EXPANDED
//
        val layout : CoordinatorLayout? = dialog.findViewById(R.id.bottomSheetLayout)

        assert(layout != null)
        layout!!.minimumHeight = requireActivity().getWindowManager().getDefaultDisplay().getHeight();

    }

    private fun addItemToCart() {
        Pref.addItemtoCart(
            Item!!.copy(
                quintity = view.quintityValue.text.toString().toInt(),
                notes = view.note.text.toString(),
                total = Total * view.quintityValue.text.toString().toInt(),
                price = Total,
                menu_options = viewmodel.OptionsArray,
                delivery_cost = Item!!.delivery_cost,
                )
        )
        Pref.restaurantId = ResturantId
        // viewmodel.OptionsArray.clear()
        SUCCESS_MotionToast(getString(R.string.success), requireActivity())
        //  viewmodel.OptionsArrayVariable?.value = 0.0

        dismiss()
                if (FromHome == 1) {
                    (context as MainActivity).let { ClickHandler().switchCartFragment(it) }
        }
    }

    private fun increaseInteger() {
        display(view.quintityValue.text.toString().toInt() + 1)
        view.TotalItem.text =
            ((Total * view.quintityValue.text.toString()
                .toInt()).toString()).plus(Pref.currency)

    }

    private fun decreaseInteger() {
        if (view.quintityValue.text.toString().toInt() > 1) {
            display(view.quintityValue.text.toString().toInt() - 1)
            view.TotalItem.text =
                ((Total * view.quintityValue.text.toString()
                    .toInt()).toString()).plus(Pref.currency)
        } else {
            Error_MotionToast(getString(R.string.minimum1), requireActivity())
        }
    }

    private fun display(number: Int) {
        view.quintityValue.setText("$number")
    }

}