package com.codes_roots.imakeapp.presentaion.mainactivity


import android.app.AlertDialog
import android.content.DialogInterface
import android.content.Intent
import android.os.Build
import android.os.Bundle
import android.util.Log
import com.codes_roots.imakeapp.presentaion.mainactivity.mvi.MainIntent
import androidx.activity.viewModels
import androidx.annotation.RequiresApi
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.app.AppCompatDelegate
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentFactory
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import cn.pedant.SweetAlert.SweetAlertDialog
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.databinding.ActivityMainBinding
import com.codes_roots.imakeapp.helper.BaseApplication
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.network_connection.networkService
import com.codes_roots.imakeapp.presentaion.Permissions
import com.codes_roots.imakeapp.presentaion.ThemeColors
import com.codes_roots.imakeapp.presentaion.homefragment.HomeFragment
import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.MainViewModel
import com.satafood.core.base.DataState
import com.satafood.core.entities.homepage.CitiesModelItem
import dagger.android.AndroidInjection
import dagger.android.AndroidInjector
import dagger.android.DispatchingAndroidInjector
import dagger.android.HasAndroidInjector
import io.socket.client.IO
import io.socket.client.Socket
import kotlinx.coroutines.launch
import javax.inject.Inject


class MainActivity : AppCompatActivity(), HasAndroidInjector {

    var name: CitiesModelItem? = null
    var mSocket: Socket? = null


    @Inject
    lateinit var viewModelFactory: ViewModelProvider.Factory


    @Inject
    lateinit var fragmentFactory: FragmentFactory

    @Inject
    lateinit var Pref: PreferenceHelper

    @Inject
    lateinit var socket: Socket

    private var curremtlang ///true if arabic
            = false

    @Inject
    lateinit var sweetAlertDialog: SweetAlertDialog

    val paymentKey = ""

    lateinit var fragment: Fragment

    private val viewModel by viewModels<MainViewModel> { viewModelFactory }

    private val mainViewModel by viewModels<com.codes_roots.imakeapp.presentaion.mainactivity.MainViewModel> { viewModelFactory }

    private lateinit var binding: ActivityMainBinding


    @RequiresApi(Build.VERSION_CODES.M)
    override fun onCreate(savedInstanceState: Bundle?) {
        AndroidInjection.inject(this)
        supportFragmentManager.fragmentFactory = fragmentFactory
        super.onCreate(savedInstanceState)
         ThemeColors(this)

        setContentView(R.layout.activity_main)

        try {
            AppCompatDelegate.setDefaultNightMode(AppCompatDelegate.MODE_NIGHT_NO);
            lifecycleScope.launch {
                mainViewModel.intentChannel.send(MainIntent.SubscribeTopicsIntent(Pref.googleID ?: ""))
            }
            //      Toast.makeText(this, "Service Destroyed", Toast.LENGTH_LONG).show()
            startService(Intent(applicationContext, networkService::class.java))

        } catch (e: Exception) {
        }

        if (savedInstanceState == null) {
            supportFragmentManager.beginTransaction()
                .setCustomAnimations(0, 0, 0, 0)
                .replace(R.id.main_frame, HomeFragment(fragmentFactory),"HomeFragment")
                .commit()
        }

        val app: BaseApplication = application as BaseApplication
        mSocket = app.getMSocket()
          //connecting socket
        mSocket?.connect()
        // mSocket?.emit("makeNewOrder",122)
        val options = IO.Options()
        options.reconnection = true //reconnection
        options.forceNew = true
        if (Permissions().CheckPermission(this)) {
        } else {
            Permissions().RequestPermission(this)
        }
        Menus(viewModel, fragmentFactory).bottomMenu(this)

        observeLiveData()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        for (fragment in supportFragmentManager.fragments) {
            fragment.onActivityResult(requestCode, resultCode, data)
        }
    }

    private fun observeLiveData() {
        mainViewModel.subscribeTopicsResponse.observe(this) { dataState ->
            when (dataState.status) {
                DataState.Status.SUCCESS -> {
                    Log.d("TAG", "observeLiveData: success")
               //     binding.layoutProgress.visibility = View.INVISIBLE
                }
                DataState.Status.ERROR -> {
               //     binding.layoutProgress.visibility = View.INVISIBLE
                    dataState.error?.throwableMessage?.let {
                        Log.d("TAG", "observeLiveData: $it")
                    }
                }
                //      DataState.Status.LOADING -> binding.layoutProgress.visibility = View.VISIBLE
                else -> {}
            }
        }
    }
    override fun onBackPressed() {
        if (supportFragmentManager.getBackStackEntryCount() > 0) {
            // If there are back-stack entries, leave the FragmentActivity
            // implementation take care of them.
            val fragments = supportFragmentManager.findFragmentByTag("CompleteOrder")
            if (fragments != null ){
                supportFragmentManager.beginTransaction().remove(fragments).commit()
                supportFragmentManager.popBackStack()

            }else {
                supportFragmentManager.popBackStack()

            }

        } else {
            // Otherwise, ask user if he wants to leave :)
            AlertDialog.Builder(this)
                .setTitle("Really Exit?")
                .setMessage("Are you sure you want to exit?")
                .setNegativeButton(android.R.string.no, null)
                .setPositiveButton(android.R.string.yes,
                    DialogInterface.OnClickListener { arg0, arg1 ->
                        moveTaskToBack(true)
                    })
                .create().show()
        }
    }

    @Inject
    lateinit var androidInjector: DispatchingAndroidInjector<Any>
    override fun androidInjector(): AndroidInjector<Any> {
        return androidInjector
    }
}
