package com.codes_roots.imakeapp.presentaion.mainactivity

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.codes_roots.imakeapp.presentaion.mainactivity.mvi.MainIntent

import com.satafood.core.base.DataState
import com.satafood.core.mapper.main.SubscribeTopicUseCase
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch
import javax.inject.Inject

class MainViewModel @Inject constructor(
    private val subscribeTopicUseCase: SubscribeTopicUseCase
    ): ViewModel() {

    val intentChannel = Channel<MainIntent>(Channel.UNLIMITED)

    private val _subscribeTopicsResponse = MutableLiveData<DataState<Unit>>()

    val subscribeTopicsResponse: LiveData<DataState<Unit>> = _subscribeTopicsResponse


    init {
        processIntent()

    }

    private fun processIntent() {
        viewModelScope.launch {
            intentChannel.consumeAsFlow().collect {
                when (it) {
                    is MainIntent.SubscribeTopicsIntent -> subscribeTopics(it.topics)
                }
            }
        }
    }


    private fun subscribeTopics(topics: String) {
        _subscribeTopicsResponse.postValue(DataState.loading())
        viewModelScope.launch() {
            subscribeTopicUseCase(topics) {
                _subscribeTopicsResponse.postValue(it)
            }
        }
    }
}