package com.codes_roots.imakeapp.presentaion.map_activity



import android.annotation.SuppressLint
import android.app.Dialog
import android.content.Context
import android.content.DialogInterface
import android.content.Intent
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.location.Address
import android.location.Geocoder
import android.location.Location
import android.location.LocationManager
import android.net.Uri
import android.os.Bundle
import android.provider.Settings
import android.util.Log
import android.view.Gravity
import android.view.ViewGroup
import android.view.Window
import android.view.WindowManager
import android.widget.Toast
import androidx.activity.viewModels
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.widget.AppCompatButton
import androidx.appcompat.widget.AppCompatImageView
import androidx.core.view.isVisible
import androidx.lifecycle.ViewModelProvider
import com.airbnb.lottie.LottieAnimationView
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.helper.*
import com.codes_roots.imakeapp.presentaion.ClickHandler
import com.codes_roots.imakeapp.presentaion.filterfragment.FilterFragment.Companion.TAG
import com.codes_roots.imakeapp.presentaion.mainactivity.MainActivity
import com.codes_roots.imakeapp.presentaion.map_activity.viewmodel.MapViewModel
import com.codes_roots.imakeapp.presentaion.myaddress.cities.CitiesActivity
import com.google.android.gms.location.FusedLocationProviderClient
import com.google.android.gms.location.LocationServices
import com.google.android.gms.maps.CameraUpdateFactory
import com.google.android.gms.maps.GoogleMap
import com.google.android.gms.maps.OnMapReadyCallback
import com.google.android.gms.maps.SupportMapFragment
import com.google.android.gms.maps.model.BitmapDescriptorFactory
import com.google.android.gms.maps.model.GroundOverlayOptions
import com.google.android.gms.maps.model.LatLng
import com.google.android.gms.maps.model.MarkerOptions
import com.google.android.gms.tasks.OnSuccessListener
import dagger.android.AndroidInjection
import dagger.android.AndroidInjector
import dagger.android.DispatchingAndroidInjector
import dagger.android.HasAndroidInjector
import kotlinx.android.synthetic.main.map_activity.*
import me.samlss.lighter.Lighter
import me.samlss.lighter.parameter.Direction
import me.samlss.lighter.parameter.LighterParameter
import me.samlss.lighter.parameter.MarginOffset
import me.samlss.lighter.shape.RectShape
import java.util.*
import javax.inject.Inject


class MapActivity : AppCompatActivity(), HasAndroidInjector, OnMapReadyCallback {


    private var permissionDenied: Boolean? = null
    //private var place_ids: String? = null

    @Inject
    lateinit var Pref: PreferenceHelper
    internal var mFusedLocationClient: FusedLocationProviderClient? = null
    private lateinit var map: GoogleMap

    var latitude: Double? = 30.033333
    var longitude: Double? = 31.233334
    val overlaySize = 100f
    var address = ""
    var streetName: String? = null

    var addresses: List<Address>? = null
    var intent1: Intent? = null

    @Inject
    lateinit var viewModelFactory: ViewModelProvider.Factory


    val viewModel by viewModels<MapViewModel> { viewModelFactory }

    public override fun onCreate(icicle: Bundle?) {
        AndroidInjection.inject(this)

        super.onCreate(icicle)
        BaseApplication.appComponent.inject(this)

        setContentView(R.layout.map_activity)
        mFusedLocationClient = LocationServices.getFusedLocationProviderClient(this)

        // Get the SupportMapFragment and request notification when the map is ready to be used.
        val mapFragment = supportFragmentManager.findFragmentById(R.id.map) as? SupportMapFragment
        mapFragment?.getMapAsync(this)


        ResourceUtil().changeLang(Pref.lang!!, this)

        this.window.setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_ADJUST_RESIZE or WindowManager.LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN)

        Lighter.with(this)
            .addHighlight(LighterParameter.Builder()
                .setHighlightedViewId(R.id.searchBar)
                .setTipLayoutId(R.layout.tip1)
                .setLighterShape(RectShape(5f, 5f, 30f))
                .setTipViewRelativeDirection(Direction.BOTTOM)
                .setTipViewRelativeOffset(MarginOffset(150, 0, 30, 0))
                .build())
                .show()


        searchBar.setOnClickListener {
            val i = Intent(this, CitiesActivity::class.java)
            (this).startActivityForResult(i, 1000);
        }

        appCompatImageView6.setOnClickListener {
            ClickHandler().switchToActivity(this, MainActivity())
        }

        goToMainActivity.isEnabled = Pref.areaId != 0

        goToMainActivity.setOnClickListener {
            ClickHandler().switchToActivity(this, MainActivity())

        }

    }


    @Inject
    lateinit var androidInjector: DispatchingAndroidInjector<Any>
    override fun androidInjector(): AndroidInjector<Any> {
        return androidInjector
    }


    override fun onMapReady(googleMap: GoogleMap) {

        map = googleMap

        map.clear();

        MapHelper().setPoiClick(map)

        // MapHelper().setMapStyle(map, this)

        map.setOnCameraMoveStartedListener {
            delivery_layout.isVisible = false
        }
        val homeLatLng = LatLng(latitude!!, longitude!!)
        val zoomLevel = 15f

        map.moveCamera(CameraUpdateFactory.newLatLngZoom(homeLatLng, zoomLevel))

         viewModel.getBranchesLocation()
        viewModel.branchesLocationLD.observe(this) {
            if (it.has_takeway == 1){
                it.branches.map { branch ->
                    val homeLatLng = LatLng(branch.latitude!!, branch.longitude!!)
                    val name = branch.name

                    map.addMarker(MarkerOptions().position(homeLatLng).title(
                        name))!!.showInfoWindow()
                }
            }else{
                Toast.makeText(
                    this, this.getString(R.string.notakeaway),
                    Toast.LENGTH_LONG
                ).show()
            }

        }

        statusCheck()

        if (MapHelper().CheckPermission(this)) {
            if (MapHelper().isLocationEnabled(this)) {
                getMyLocation(this)

            } else {
                delivery_layout.isEnabled = false
                delivery_layout.isVisible = true
                MapHelper().RequestPermission(this)
                Toast.makeText(
                    this,
                    "Please Turn on Your device Location",
                    Toast.LENGTH_LONG
                ).show()
            }
        } else {

            MapHelper().RequestPermission(this)

        }

    }
    private fun getCompleteAddressString(LATITUDE: Double, LONGITUDE: Double): String {
        var strAdd = ""
        val geocoder = Geocoder(this, Locale.getDefault())
        try {
            val addresses = geocoder.getFromLocation(LATITUDE, LONGITUDE, 1)
            if (addresses != null) {
                val returnedAddress = addresses[0]
                val strReturnedAddress = StringBuilder("")
                for (i in 0..returnedAddress.maxAddressLineIndex) {
                    strReturnedAddress.append(returnedAddress.getAddressLine(i)).append("\n")
                }
                strAdd = strReturnedAddress.toString()
                deliveryLocation.text = strAdd
                Log.w("My Current loction address", strReturnedAddress.toString())
            } else {
                Log.w("My Current loction address", "No Address returned!")
            }
        } catch (e: Exception) {
            e.printStackTrace()
            Log.w("My Current loction address", "Canont get Address!")
        }
        return strAdd
    }
    private fun requestLocationPermissionManual() {
        val intent = Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS)
        val uri: Uri = Uri.fromParts("package", packageName, null)
        intent.data = uri
        startActivity(intent)
    }

    private fun requestLocationPermissionDialog() {
        try {
            val dialog = Dialog(this)
            dialog.requestWindowFeature(Window.FEATURE_NO_TITLE)
            dialog.setContentView(R.layout.location_setting_dialog)
            dialog.findViewById<LottieAnimationView>(R.id.animation_view).playAnimation()
            dialog.findViewById<AppCompatButton>(R.id.goToSettings).setOnClickListener {
                requestLocationPermissionManual()
            }
            dialog.findViewById<AppCompatImageView>(R.id.closeBtn).setOnClickListener {
                dialog.dismiss()
            }
            dialog.show()
            dialog.window?.setLayout(ViewGroup.LayoutParams.MATCH_PARENT,
                ViewGroup.LayoutParams.WRAP_CONTENT)
            dialog.window?.setBackgroundDrawable(object : ColorDrawable(Color.TRANSPARENT) {})
            dialog.window?.attributes?.windowAnimations = R.style.DialogAnimation
            dialog.window?.setGravity(Gravity.CENTER)
        } catch (e: Exception) {

        }
    }
    @SuppressLint("MissingPermission")
    fun getMyLocation(context: Context) {
        map.isMyLocationEnabled = true
        mFusedLocationClient!!.lastLocation.addOnCompleteListener { task ->
            val location: Location? = task.result
            if (location == null) {
                MapHelper().NewLocationData(context)
            } else {
                latitude = location.latitude
                longitude = location.longitude
               val homeLatLng = LatLng(latitude!!, longitude!!)
                val zoomLevel = 12f
               map.moveCamera(CameraUpdateFactory.newLatLngZoom(homeLatLng, zoomLevel))
              //  map.addMarker(MarkerOptions().position(homeLatLng))
                //   setMapLongClick(map)
                map.mapType = GoogleMap.MAP_TYPE_TERRAIN
                val googleOverlay = GroundOverlayOptions()
                    .image(BitmapDescriptorFactory.fromResource(R.drawable.android))
                    .position(homeLatLng, overlaySize)
                map.addGroundOverlay(googleOverlay)
                goToAddress(latitude!!, longitude!!)
            }
            Pref.currentLatitude = location?.latitude.toString()
            Pref.currentLongitude = location?.longitude.toString()
        }
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<String>,
        grantResults: IntArray,
    ) {
        Log.i(TAG, "onRequestPermissionsResult: " + requestCode)
        if (requestCode != MY_PERMISSIONS_REQUEST_LOCATION) {
            super.onRequestPermissionsResult(
                requestCode,
                permissions,
                grantResults
            )
            return
        }
        if (MapHelper().CheckPermission(this)) {
            if (MapHelper().isLocationEnabled(this)) {
                // Enable the my location layer if the permission has been granted.
                getMyLocation(context = this)
            } else {
                // Permission was denied. Display an error message
                // Display the missing permission error dialog when the fragments resume.
                MapHelper().RequestPermission(this)

            }
        }
    }

    @SuppressLint("MissingPermission")
    private fun getLocationPermission() {
        if (MapHelper().CheckPermission(this)) {
            mFusedLocationClient?.lastLocation?.addOnSuccessListener(this,
                OnSuccessListener<Location?> { location ->
                    // Got last known location. In some rare situations this can be null.
                    if (location != null) {
                        latitude = location.latitude
                        longitude = location.longitude

                    }
                })
        } else {
            MapHelper().RequestPermission(this)
        }
    }

    private fun goToAddress(mlatitude: Double, mLogitude: Double) {
        //  map.animateCamera(CameraUpdateFactory.newLatLngZoom(LatLng(mlatitude, mLogitude), 16.0f))
        map.setOnCameraIdleListener(GoogleMap.OnCameraIdleListener {
            latitude = map.cameraPosition.target.latitude
            longitude = map.cameraPosition.target.longitude
            getCompleteAddressString(latitude!!,longitude!!)
            delivery_layout.isVisible = true
        })
    }

    fun gpsStatus() {
        intent1 = Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
        startActivity(intent1);
    }

    fun statusCheck() {
        val manager: LocationManager = getSystemService(LOCATION_SERVICE) as LocationManager
        if (!manager.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
            buildAlertMessageNoGps()
        }
    }

    val positiveButtonClick = { dialog: DialogInterface, which: Int ->
        gpsStatus()
    }
    val negativeButtonClick = { dialog: DialogInterface, which: Int ->
        dialog.cancel()
    }

    private fun buildAlertMessageNoGps() {
        val builder: AlertDialog.Builder = AlertDialog.Builder(this)
        builder.setMessage("Your GPS seems to be disabled, do you want to enable it?")
            .setCancelable(false)
        builder.setPositiveButton(android.R.string.yes, positiveButtonClick)
        builder.setNegativeButton(android.R.string.no, negativeButtonClick)
        val alert: AlertDialog = builder.create()
        alert.show()
    }


    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == 1000) {
            if (data != null) {

                searchBar.setText(Pref.CityTileName!!)
                getCompleteAddressString(latitude!!,longitude!!)
                goToMainActivity.isEnabled = true
                Pref.userLocation = Pref.CityTileName
            } else
                WARN_MotionToast(this.getString(R.string.chooseArea), this)
        }
    }


    companion object {
        private const val MY_PERMISSIONS_REQUEST_LOCATION = 1010
    }

    override fun onResume() {
        super.onResume()

    }


}
