package com.codes_roots.imakeapp.presentaion.map_activity.viewmodel

import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.repository.google.GoogleDataSource
import com.satafood.core.entities.branches_locations.BranchesLocation
import com.satafood.core.entities.google.Google
import com.satafood.core.entities.restaurantslist.BranchesModelListItem
import io.reactivex.disposables.CompositeDisposable
import kotlinx.coroutines.*
import javax.inject.Inject

class MapViewModel @Inject constructor(
    private val Datasources: GoogleDataSource,
) : ViewModel() {

    @Inject
    lateinit var Pref: PreferenceHelper
    private var job: Job? = null
    var mCompositeDisposable = CompositeDisposable()

    var BranchesLD: MutableLiveData<ArrayList<BranchesModelListItem>>? = null
    var branchesByAreasForGoogleMaps: MutableLiveData<BranchesModelListItem>? = null
    var PlaceID = MutableLiveData<Google>()
    var branchesLocationLD = MutableLiveData<BranchesLocation>()

    val errorMessage = MutableLiveData<String>()
    val loading = MutableLiveData<Boolean>()


    init {
        BranchesLD = MutableLiveData()
        branchesByAreasForGoogleMaps = MutableLiveData()
    }

    val coroutineExceptionHandler = CoroutineExceptionHandler { _, throwable ->
        throwable.printStackTrace()
    }
//
    fun getBranchesLocation() {

        job = CoroutineScope(Dispatchers.IO + coroutineExceptionHandler).launch {
            val response = Datasources.getBranchesMap()
            withContext(Dispatchers.Main + coroutineExceptionHandler) {
                if (response.isSuccessful) {

                    branchesLocationLD.value = response.body()!!

                } else {
                    onError("Error : ${response.message()} ")

                }
            }
        }

    }

//    fun getPlaceIdByGeoCode(latLng: String, language: String, type: String) {
//        kotlin.runCatching {
//            job = CoroutineScope(Dispatchers.IO + coroutineExceptionHandler).launch {
//                val response = Datasources.getPlaceIdByGeoCode(latLng, language, type)
//                withContext(Dispatchers.Main + coroutineExceptionHandler) {
//                    if (response.isSuccessful) {
//
//                        PlaceID.value = response.body()!!
//                    } else {
//                        onError("Error : ${response.message()} ")
//
//                    }
//                }
//            }
//        }.getOrElse { throw  it }
//    }

//
//    fun getRestaurantList(googleModel: GoogleModel) {
//        job = CoroutineScope(Dispatchers.IO + coroutineExceptionHandler).launch {
//            val response = Datasources.getBranchesByAreasForGoogleMaps(googleModel)
//            withContext(Dispatchers.Main + coroutineExceptionHandler) {
//                if (response.isSuccessful) {
//                    branchesByAreasForGoogleMaps?.postValue(response.body())
//
//                } else {
//                    onError("Error : ${response.message()} ")
//                }
//            }
//        }
//
//    }


    private fun onError(message: String) {
        errorMessage.value = message
        loading.value = false
    }

    override fun onCleared() {
        super.onCleared()
        mCompositeDisposable.dispose()
        mCompositeDisposable.clear()
        job?.cancel()
    }
}