package com.codes_roots.imakeapp.presentaion.menufragment

import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.View.GONE
import android.view.View.VISIBLE
import android.view.ViewGroup
import androidx.core.view.isVisible
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.create_vendor_driver_account.create_driver_account_fragment.CreateDriverAccountFragment
import com.codes_roots.imakeapp.create_vendor_driver_account.create_vendor_account_fragment.CreateVendorAccountFragment
import com.codes_roots.imakeapp.databinding.MenuFragmentBinding
import com.codes_roots.imakeapp.helper.BaseApplication
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.helper.ResourceUtil
import com.codes_roots.imakeapp.presentaion.ClickHandler
import com.codes_roots.imakeapp.presentaion.choose_language.LanguageActivity
import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.presentaion.mainactivity.MainActivity
import com.codes_roots.imakeapp.presentaion.menufragment.adapter.SocialAdapter
import com.codes_roots.imakeapp.presentaion.menufragment.adapter.StaticPagesAdapter
import com.codes_roots.imakeapp.presentaion.menufragment.mvi.MainIntent
import com.codes_roots.imakeapp.presentaion.menufragment.viewmodel.StaticPagesViewModel
import com.codes_roots.imakeapp.presentaion.mypoints.PointsFragment
import kotlinx.android.synthetic.main.map_activity.*
import kotlinx.coroutines.flow.collect
import javax.inject.Inject


open class MenuFragment @Inject constructor() : Fragment() {

    private var curremtlang ///true if arabic
            = false

    lateinit var staticPagesAdapter: StaticPagesAdapter
    lateinit var socialAdapter: SocialAdapter

    @Inject
    lateinit var Pref: PreferenceHelper

    @Inject
    lateinit var viewModelFactory: ViewModelProvider.Factory

    val viewModel by viewModels<StaticPagesViewModel> { viewModelFactory }
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (savedInstanceState == null) {
            BaseApplication.appComponent.inject(this)

        }
    }

    private lateinit var view: MenuFragmentBinding

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?,
    ): View? {


        view = DataBindingUtil.inflate(
            inflater,
            R.layout.menu_fragment, container, false
        )


        ResourceUtil().changeLang(Pref.lang.toString(), requireContext())

        view.click = ClickHandler()
        view.context = context as MainActivity
        view.pref = (context as MainActivity).Pref

        view.driverFragment = CreateDriverAccountFragment()
        view.pointFragment = PointsFragment()

        if (PreferenceHelper(requireContext()).UserId > 0)
            view.loginLayout.visibility = View.GONE
        else {
            view.logoutLayout.visibility = View.GONE;

        }


        if (ResourceUtil().getCurrentLanguage(requireContext())!!.contains("ar"))
            view.langChosen.text = "عربي"
        else
            view.langChosen.text = "English"

        view.langLayout.setOnClickListener {
            changeAppLanguage()
        }


        staticPagesRecycleView()
        socialRecycleView()
        getAllData()

        return view.root
    }

    private fun changeAppLanguage() {
        val i = Intent(activity, LanguageActivity::class.java)
        startActivity(i)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == 10000) {
            if (PreferenceHelper(requireContext()).UserId > 0) {
                view.loginLayout.visibility = GONE
                view.logoutLayout.visibility = VISIBLE
                view.myname.text = Pref.userName
            } else {
                view.logoutLayout.visibility = GONE;
                view.loginLayout.visibility = VISIBLE
                view.myname.text = requireContext().getString(R.string.hello)
            }
        }
    }

    fun getAllData() {
        lifecycleScope.launchWhenStarted {
            viewModel.state.collect {
                if (it != null) {
                    if (it.error != null) {
                        it.error?.let { userError ->
                            when (userError) {
                                is UserError.InvalidId -> "Invalid id"
                                is UserError.NetworkError -> userError.throwable.message
                                UserError.ServerError -> "Server error"
                                UserError.Unexpected -> "Unexpected error"
                                is UserError.UserNotFound -> "User not found"
                                UserError.ValidationFailed -> "Validation failed"
                                else -> {}
                            }
                        }
                        viewModel.intents.trySend(MainIntent.ErrorDisplayed(it))
                    } else {
                        if (it.progress == true) {
                            viewModel.intents.send(MainIntent.Initialize(it))
                            viewModel.intents.send(MainIntent.InitializeSocialData(it))
                        } else {
                            view.progress.isVisible = it.progress!!
                            if (it.data?.isNotEmpty() == true){
                                staticPagesAdapter.submitList(it.data!!)
                            }
                            if (it.socialData?.isNotEmpty() == true){
                                socialAdapter.submitList(it.socialData!!)
                            }
                        }
                    }
                }

            }
        }
    }

    private fun staticPagesRecycleView() {
        staticPagesAdapter = StaticPagesAdapter(requireContext())
        view.pagesRecycle.apply {
            layoutManager = LinearLayoutManager(context)
            adapter = staticPagesAdapter;
            isNestedScrollingEnabled = true

        }
    } 
    
    fun socialRecycleView() {
        socialAdapter = SocialAdapter(requireContext())
        view.socialRecycle.apply {
            adapter = socialAdapter;
            isNestedScrollingEnabled = true
            setHasFixedSize(true)
        }
    }
}