package com.codes_roots.imakeapp.presentaion.menufragment.mvi


import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.repository.homepage.DataRepo
import com.satafood.core.entities.social_media.SocialMediaModelItem
import com.satafood.core.entities.static_pages.StaticPagesItem
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first


/**
 * this is the model function in MVI, it's responsibility is to convert intents into views states
 */
suspend fun mapIntentToViewState(intent: MainIntent, Datarepo: DataRepo,
    loadMainData: suspend () -> Flow<Result<ArrayList<StaticPagesItem>>> = { Datarepo.getStaticPages() },
    loadSocialData: suspend () -> Flow<Result<ArrayList<SocialMediaModelItem>>> = { Datarepo.getSocialIcons() },
) = when (intent) {
    is MainIntent.Initialize -> proceedWithInitialize(loadMainData,intent)
    is MainIntent.InitializeSocialData -> proceedWithInitializeSocial(loadSocialData,intent)
    is MainIntent.ErrorDisplayed -> intent.viewState.copy(error = null)
}


private suspend fun proceedWithInitialize(loadCart: suspend () -> Flow<Result<ArrayList<StaticPagesItem>>>, intent:MainIntent): MainViewState {
    val response =   loadCart()
    val data = response.first()
    return runCatching {
        intent.viewState!!.copy(data = (data.getOrThrow()), error = null, progress = false)
    }
        .getOrElse {
            intent.viewState!!.copy(error = UserError.NetworkError(it))
        }
}
private suspend fun proceedWithInitializeSocial(loadCart: suspend () -> Flow<Result<ArrayList<SocialMediaModelItem>>>, intent:MainIntent): MainViewState {
    val response =   loadCart()
    val data = response.first()
    return runCatching {
        intent.viewState!!.copy(socialData = (data.getOrThrow()), error = null, progress = false)
    }
        .getOrElse {
            intent.viewState!!.copy(error = UserError.NetworkError(it))
        }
}




