package com.codes_roots.imakeapp.presentaion.myaddress.cities

import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.view.WindowManager
import androidx.activity.viewModels
import androidx.appcompat.app.AppCompatActivity
import androidx.core.view.isVisible
import androidx.core.widget.doOnTextChanged
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.databinding.CitiesActivityBinding
import com.codes_roots.imakeapp.helper.BaseApplication
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.presentaion.myaddress.cities.mvi.MainIntent
import com.codes_roots.imakeapp.presentaion.myaddress.viewmodel.UserAddressViewModel
import dagger.android.AndroidInjector
import dagger.android.DispatchingAndroidInjector
import dagger.android.HasAndroidInjector
import kotlinx.android.synthetic.main.cities_activity.*
import kotlinx.android.synthetic.main.main_fragment.*
import kotlinx.coroutines.flow.collect
import javax.inject.Inject


open class CitiesActivity : AppCompatActivity(), HasAndroidInjector {

    @Inject
    lateinit var viewModelFactory: ViewModelProvider.Factory

    @Inject
    lateinit var Pref: PreferenceHelper

    private val viewModel by viewModels<UserAddressViewModel> { viewModelFactory }

    var citiesAdapter: Cities_Adapter? = null


    lateinit var view: CitiesActivityBinding

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        BaseApplication.appComponent.inject(this)
        view = DataBindingUtil.setContentView(this, R.layout.cities_activity);

        this.getWindow()?.setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_ADJUST_NOTHING);


       viewModel.intents.trySend(MainIntent.Initialize(viewModel.state.value!!))
       //startService(Intent(baseContext, networkService::class.java))

        //  view.searchBar.setError("assad")
        view.searchBar.doOnTextChanged { text, start, before, count ->
            viewModel.intents.trySend(MainIntent.SearchByName(viewModel.state.value!!,
                text.toString()))
        }

        backBtn.setOnClickListener {
            onBackPressed()
        }


        getAllData()
        categoryRecycleView()


    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)

        Log.d("result", resultCode.toString())

    }

    fun categoryRecycleView() {
        citiesAdapter =
            Cities_Adapter(viewModel, viewModel.intents, this@CitiesActivity)
        view.citiesRecycle.apply {
            adapter = citiesAdapter;
            setHasFixedSize(true)
        }
    }


    fun getAllData() {
        lifecycleScope.launchWhenStarted {
            viewModel.state.collect { viewState ->
                try {
                if (viewState != null) {

                        if (viewState?.error != null) {
                            viewState.error?.let {
                                when (it) {
                                    is UserError.InvalidId -> "Invalid id"
                                    is UserError.NetworkError -> "Server error"
                                    is UserError.ServerError -> "Server error"
                                    is UserError.Unexpected -> "Unexpected error"
                                    is UserError.UserNotFound -> "User not found"
                                    is UserError.ValidationFailed -> "Validation failed"
                                    else -> {}
                                }
                            }
                            viewModel.intents.send(MainIntent.ErrorDisplayed(viewState))
                        } else {

                            view.progress.isVisible = viewState.progress!!

                            citiesAdapter?.submitList(viewState.filteredData)
                        }
                }
                } catch (e: Exception) {

                }
            }
        }
    }


    @Inject
    lateinit var androidInjector: DispatchingAndroidInjector<Any>
    override fun androidInjector(): AndroidInjector<Any> {
        return androidInjector
    }


}