package com.codes_roots.imakeapp.presentaion.myaddress.cities.mvi


import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.repository.homepage.DataRepo
import com.satafood.core.entities.homepage.CitiesModelItem
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first


/**
 * this is the model function in MVI, it's responsibility is to convert intents into views states
 */
suspend fun mapIntentToViewState(
    intent: MainIntent,
    Datarepo:DataRepo,
    loadMainData: suspend () -> Flow<Result<ArrayList<CitiesModelItem>?>> = { Datarepo.GetCities() },
    loadTownData: suspend () -> Flow<Result<ArrayList<CitiesModelItem>>> = { Datarepo.getAreasData(intent.cityId!!.toInt()) },
    LoadAreas: suspend () -> Flow<Result<ArrayList<CitiesModelItem>?>> = { Datarepo.getTownData(intent.cityId!!.toInt()) },
    ) = when (intent) {
    is MainIntent.Initialize -> proceedWithInitialize(loadMainData,intent)
    is MainIntent.ErrorDisplayed -> intent.viewState.copy(error = null)
    is MainIntent.SearchByName -> SearchByCityName(intent,intent.cityName!!)
    is MainIntent.InitializeCities -> getTowns(intent.viewState!!.townsdata!!,intent)
    is MainIntent.GetAreas -> getTowns(intent.viewState!!.areadata!!,intent)
}


private suspend fun proceedWithInitialize(citiesData: suspend () -> Flow<Result<ArrayList<CitiesModelItem>?>>, intent:MainIntent): CitiesViewState {
    val response =   citiesData()
    val data = response.first()

    return runCatching {
        intent.viewState!!.copy(citiesdata = data.getOrThrow()
            ,filteredData = data.getOrThrow()
            , error = null
            , progress = false)
    }
        .getOrElse {
            intent.viewState!!.copy(error = UserError.NetworkError(it))
        }
}

private  fun getTowns(townData:  ArrayList<CitiesModelItem>, intent:MainIntent): CitiesViewState {

    return runCatching {
        intent.viewState!!.copy(
             citiesdata = townData
            , filteredData = townData
            , error = null
            , progress = false)
    }
        .getOrElse {
            intent.viewState!!.copy(error = UserError.NetworkError(it))
        }
}




private  fun SearchByCityName(intent: MainIntent, CitytName:String): CitiesViewState
{
   runCatching {
    val filterData = SearchCityBynamee(CitytName,intent.viewState?.citiesdata)
    return intent.viewState!!.copy(filteredData = filterData as ArrayList, progress = false)
   }.getOrElse { throw  it }
}


fun SearchCityBynamee(cityName: String, brancharray: ArrayList<CitiesModelItem>?) =
    runCatching {
    brancharray?.filter { data ->
        data.name!!.contains(cityName) }  }.getOrElse { throw  it }
