package com.codes_roots.imakeapp.presentaion.myaddress.viewmodel

import android.util.Log
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.meem.physio.util.SingleLiveEvent
import com.codes_roots.imakeapp.di.IoDispatcher
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.presentaion.mainactivity.BaseViewModel
import com.codes_roots.imakeapp.presentaion.myaddress.cities.mvi.CitiesViewState
import com.codes_roots.imakeapp.presentaion.myaddress.cities.mvi.MainIntent
import com.codes_roots.imakeapp.presentaion.myaddress.cities.mvi.mapIntentToViewState
import com.codes_roots.imakeapp.repository.homepage.DataRepo
import com.codes_roots.imakeapp.repository.retaurantdetails.RemoteDataSource
import com.satafood.core.entities.addresses.UserAddressesItem
import com.satafood.core.entities.login.User
import com.satafood.core.entities.profile.ProfileModel
import kotlinx.coroutines.*
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.flow.*
import okhttp3.MultipartBody
import java.io.IOException
import javax.inject.Inject

class UserAddressViewModel @Inject constructor(private val DateRepoCompnay: DataRepo,
                                               private val Datasources: RemoteDataSource,
                                               @IoDispatcher private val ioDispatcher: CoroutineDispatcher
)
    : BaseViewModel<CitiesViewState>() {
    val intents : Channel<MainIntent> = Channel(Channel.UNLIMITED)
    protected val uiState : MutableStateFlow<CitiesViewState?> = MutableStateFlow(CitiesViewState())

    val state: MutableStateFlow<CitiesViewState?> get() = uiState
    @Inject

    lateinit var Pref: PreferenceHelper
    private var job: Job? = null

    var MyAddressesLD = SingleLiveEvent <ArrayList<UserAddressesItem>>()

    var checkAddressesLD = SingleLiveEvent <ArrayList<UserAddressesItem>>()

    var AddAddressesLD: SingleLiveEvent<UserAddressesItem>? = null

    var userLD: SingleLiveEvent<ProfileModel>? = null
    var editUserLD: SingleLiveEvent<User>? = null

    var isAddressError = false



    var errorMessage = SingleLiveEvent<String>()
    val loading = MutableLiveData<Boolean>()


    init {


        userLD = SingleLiveEvent()
        editUserLD = SingleLiveEvent()
        checkAddressesLD = SingleLiveEvent()
        AddAddressesLD = SingleLiveEvent()
        MyAddressesLD = SingleLiveEvent()


        getIntent()

// لازم ابعت viewstate كامل واتحكم بكل option حسب المطلوب وارساله مرة اخري واستقباله هنا او في ال view



        // launchNextViewStateJob(intents.value)
    }
    val coroutineExceptionHandler = CoroutineExceptionHandler{_, throwable ->
        throwable.printStackTrace()
    }
    fun getIntent(){
        uiState.value = CitiesViewState().copy(progress = true)

        job = viewModelScope.launch() {

            intents.receiveAsFlow().collect {

                uiState.value = mapIntentToViewState(it,DateRepoCompnay)

            }

        }

    }


    fun getUserAddresses() {
        job = CoroutineScope(Dispatchers.IO).launch {
            try {
                val response = Datasources.getUserAddresses()
                withContext(Dispatchers.Main + coroutineExceptionHandler) {
                    if (response.isSuccessful) {
                        MyAddressesLD.postValue(response.body())

                    } else {
                        isAddressError = true
                        onError("Error : ${response.message()} ")
                    }
                }
            } catch (e: IOException) {
                Log.i("CatchingCrash", "getUserAddresses: ${e.message}")
            }

        }
    }

    fun checkRestaurantAddress(branchId:Int) {
       job = CoroutineScope(Dispatchers.IO).launch {
           try {
            val response = Datasources.checkRestaurantAddress(branchId)
            withContext(Dispatchers.Main+coroutineExceptionHandler) {
                if (response.isSuccessful) {
                    checkAddressesLD.postValue(response.body())

                } else {
                    onError("Error : ${response.message()} ")
                }
            }

      }catch (e:Exception){
           }
       }

    }

    //addUsersAddress
    fun addUsersAddress(addressInfo: UserAddressesItem?) {
     try {

        job = CoroutineScope(Dispatchers.IO).launch {
            val response = Datasources.addUsersAddress(addressInfo!!)
            withContext(Dispatchers.Main+coroutineExceptionHandler) {
                if (response.isSuccessful) {
                    AddAddressesLD?.postValue(response.body())

                } else {
                    onError("Error : ${response.message()} ")
                }
            }
        }

     }catch (e:Exception){

     }
    }

    //addUsersAddress
    fun getUsersData(userId: Int?) {
        runCatching {

            job = CoroutineScope(Dispatchers.Default + coroutineExceptionHandler).launch {
            try {
            val response = Datasources.getUsersData(userId!!)
            withContext(Dispatchers.Main+coroutineExceptionHandler) {
                if (response.isSuccessful) {
                    userLD?.postValue(response.body())

                } else {
                    onError("Error : ${response.message()} ")
                    isAddressError = true
                }
            }
            }catch (e:Exception){
                Log.i("CatchingCrash", "getUsersData: ${e.message}")
            }
        }

        }.getOrElse { throw  it }
    }
    //addUsersAddress
    fun editUserData(file: MultipartBody.Part?, user: User){
            job = CoroutineScope(Dispatchers.IO).launch {
                runCatching {
                val response = Datasources.editUserData(file, user)
                withContext(Dispatchers.Main+coroutineExceptionHandler) {
                    if (response.isSuccessful) {
                        editUserLD?.postValue(response.body())
                    } else {
                        onError("Error : ${response.message()} ")
                    }
                }

                }.getOrElse { throw  it }
            }

    }

    //addUsersAddress
    fun editUsersDataWithOutPhoto(user: User){
        job = CoroutineScope(Dispatchers.IO).launch {
            runCatching {
                val response = Datasources.editUsersDataWithOutPhoto(user)
                withContext(Dispatchers.Main+coroutineExceptionHandler) {
                    if (response.isSuccessful) {
                        editUserLD?.postValue(response.body())
                    } else {
                        onError("Error : ${response.message()} ")
                    }
                }

            }.getOrElse { throw  it }
        }

    }

    //deleteUsersAddress
    fun deleteUsersAddress(addressId: Int?) {
        job = CoroutineScope(Dispatchers.IO).launch {
            runCatching {
            val response = Datasources.deleteUsersAddress(addressId!!)
            withContext(Dispatchers.Main+coroutineExceptionHandler) {
                if (response.isSuccessful) {

                } else {
                    onError("Error : ${response.message()} ")
                }
            }    }.getOrElse { throw  it }
        }

    }    //editUsersAddress
    fun editUsersAddress(addressId: Int,addressInfo: UserAddressesItem?) {
        job = CoroutineScope(Dispatchers.IO).launch {
            runCatching {      val response = Datasources.editUsersAddress(addressId,addressInfo!!)
            withContext(Dispatchers.Main+coroutineExceptionHandler) {
                if (response.isSuccessful) {
                    AddAddressesLD?.postValue(response.body())

                } else {
                    onError("Error : ${response.message()} ")
                }
            }
            }.getOrElse { throw  it }
        }

    }

    //addUsersAddress
    private fun onError(message: String) {
        errorMessage.value = message
        loading.value = false
    }

    override fun onCleared() {
        super.onCleared()
        job?.cancel()
    }
}