package com.codes_roots.imakeapp.presentaion.myorderDetails

import android.app.Dialog
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.media.MediaPlayer
import android.os.Bundle
import android.os.CountDownTimer
import android.os.Handler
import android.os.Looper
import android.view.*
import androidx.appcompat.widget.AppCompatButton
import androidx.appcompat.widget.AppCompatImageView
import androidx.core.view.isVisible
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentFactory
import androidx.fragment.app.viewModels
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProvider
import com.airbnb.lottie.LottieAnimationView
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.databinding.MyOrdersDetailsFragment2Binding
import com.codes_roots.imakeapp.databinding.MyOrdersDetailsFragmentBinding
import com.codes_roots.imakeapp.helper.BaseApplication
import com.codes_roots.imakeapp.helper.Error_MotionToast
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.helper.SUCCESS_MotionToast
import com.codes_roots.imakeapp.presentaion.ClickHandler
import com.codes_roots.imakeapp.presentaion.mainactivity.MainActivity
import com.codes_roots.imakeapp.presentaion.myorderDetails.adapter.OrdersDetailsAdapter
import com.codes_roots.imakeapp.presentaion.myorders.mvi.MyOrderViewModel
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import com.satafood.core.entities.myorders.MyOrderModelItem
import com.satafood.core.entities.restaurantdetails.MenuOption
import java.time.Duration
import java.time.LocalDateTime
import java.time.ZonedDateTime
import javax.inject.Inject

class MyOrdersDetailsFragment @Inject constructor
    (var Item: MyOrderModelItem, var fragmentFactory: FragmentFactory) : Fragment() {

    private lateinit var timer: CountDownTimer
    private var minutesRemaining: Long = 0
    private var secondsRemaining: Long = 0
    @Inject
    lateinit var viewModelFactory: ViewModelProvider.Factory

    val viewModel by viewModels<MyOrderViewModel> { viewModelFactory }


    @Inject
    lateinit var Pref: PreferenceHelper

    lateinit var myOrdersAdapter: OrdersDetailsAdapter

    override fun onCreate(savedInstanceState: Bundle?) {
        childFragmentManager.fragmentFactory = fragmentFactory
        super.onCreate(savedInstanceState)
        BaseApplication.appComponent.inject(this)

    }

    private lateinit var view: MyOrdersDetailsFragment2Binding


    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?,
    ): View {

        view = DataBindingUtil.inflate(
            inflater, R.layout.my_orders_details_fragment2, container, false
        )

        view.name.isSelected = true
        view.data = Item
        view.listener = ClickHandler()
        view.context = context as MainActivity
        view.backArrow.setOnClickListener { requireActivity().onBackPressed() }

//        view.shimmerViewContainer.startShimmerAnimation()

        val now = LocalDateTime.now()
        val dt1: ZonedDateTime = ZonedDateTime.parse(now.toString()+"Z")
        val dt2: ZonedDateTime = ZonedDateTime.parse(Item.getDateDif().toString())

        val dt3 = Duration.between(dt1, dt2).toMinutes().toLong()
        if (Item.delivery_time != null) {
            if (Item.order_status_id!! < 4) {
                minutesRemaining = dt3 + Item.delivery_time!!.toLong()
                secondsRemaining = minutesRemaining * 60
                startTimer()
            }
        }
        (context as MainActivity).mSocket?.emit("CreateDeliveryRoom", Item.OrderId.toString())
        (context as MainActivity).mSocket?.on("ChangeStatus") {
            val handler = Handler(Looper.getMainLooper())
            handler.post {
                val data: MyOrderModelItem? = null
                val gson = Gson()
                val json = it.first().toString()
                val type = object : TypeToken<MyOrderModelItem>() {}.type
                val newitem = gson.fromJson<MyOrderModelItem>(json, type)
                when (newitem?.status) {
                    "1" -> {
                        preparingStatus()
                    }
                    "2" -> {
                        preparedStatus()
                    }
                    "3" -> {
                        statusDelivering()
                    }
                    "4" -> {
                        statusDelivered()
                    }
                    "5" -> {
                        canceledStatus()
                    }
                    else -> {

                    }

                }
                try {
                    val mp = MediaPlayer.create(requireContext(), R.raw.message)
                    mp.start()
                } catch (e: Exception) {
                }
            }

        }

        view.reOrder.setOnClickListener {
            reOrder()
        }

        viewModel.reOrderLD?.observe(viewLifecycleOwner, Observer {
            if (it.toDouble() == Item.sub_total) {
                if (Pref.retriveCartItemsValue().isNullOrEmpty()) {
                    addItemToCart()
                    ClickHandler().switchCartToReorder(requireContext(), it.toDouble())

                } else {
                    Pref.clearCart()
                }
            }else{
                reorderFailed()
            }
        })

        view.rateProduct.isVisible = Item.order_status_id!! == 4

        myOrderRecycleView()
        return view.root
    }

    private fun reorderFailed(){
        try {
            val dialog = Dialog(requireContext())
            dialog.requestWindowFeature(Window.FEATURE_NO_TITLE)
            dialog.setContentView(R.layout.reorder_warning)
            dialog.findViewById<LottieAnimationView>(R.id.animation_view).playAnimation()
            dialog.findViewById<AppCompatButton>(R.id.seeMenu).setOnClickListener {
                ClickHandler().switchRestaurantsDetailsFragmentFromBestSeller(Item.restaurantsId!!.toLong(),Pref.areaId,requireContext())

            }
            dialog.findViewById<AppCompatImageView>(R.id.closeBtn).setOnClickListener {
                dialog.dismiss()
            }
            dialog.show()
            dialog.window?.setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT)
            dialog.window?.setBackgroundDrawable(object : ColorDrawable(Color.TRANSPARENT){})
            dialog.window?.attributes?.windowAnimations = R.style.DialogAnimation
            dialog.window?.setGravity(Gravity.BOTTOM)
        }catch (e:Exception){}
    }
    private fun myOrderRecycleView() {
        myOrdersAdapter = OrdersDetailsAdapter(requireContext())
        view.orderItemsRv.apply {
            adapter = myOrdersAdapter
            myOrdersAdapter.submitList(Item.order_details)
            isNestedScrollingEnabled = false
            setHasFixedSize(true)
            view.progress.visibility = View.GONE
        }
    }

    private fun reOrder() {
        val data = Item
        viewModel.preOrder(data)
    }

    private fun addItemToCart() {
        val data = Item.order_details
        var menu_options: List<MenuOption>? = null
        for (item in data!!) {
            var total = 0.0
            menu_options
            total = if (!item.order_details_options?.isNullOrEmpty()!!) {
                item.order_details_options?.first()?.menu_options?.price!!
            } else {
                item.menu_categories_items?.price!!
            }
            Pref.addItemtoCart(
                item.menu_categories_items?.copy(
                    quintity = item.amount!!,
                    notes = item.notes,
                    total = total * item.amount!!,
                    price = total,
                    id = item.menu_categories_itemId,
                    menu_options = item.order_details_options!!.map { it.menu_options!! },
                    delivery_cost = item.menu_categories_items!!.delivery_cost,
                )!!
            )
        }
        Pref.restaurantId = Item.restaurantsId!!
        // viewmodel.OptionsArray.clear()
        SUCCESS_MotionToast(getString(R.string.success), requireActivity())
        //  viewmodel.OptionsArrayVariable?.value = 0.0
        activity?.supportFragmentManager?.popBackStack()
    }


    private fun preparingStatus() {
        try {
            view.line.setBackgroundColor((context as MainActivity).resources.getColor(R.color.orange))
            view.status.text = (context as MainActivity).getString(R.string.statues1)
            view.stateOneImage.isVisible = true
            view.stateOneImage.setImageResource(R.drawable.state3)
        } catch (e: Exception) {
        }
    }

    private fun preparedStatus() {
        try {
            view.statusLine2.setBackgroundColor((context as MainActivity).resources.getColor(R.color.orange))
            view.status.text = (context as MainActivity).getString(R.string.statues2)
            view.stateTwoImage.isVisible = true
            view.stateTwoImage.setImageResource(R.drawable.state3)

        } catch (e: Exception) {
        }
    }

    private fun statusDelivering() {
        try {
            view.stateThreeImage.isVisible = true
            view.stateThreeImage2.isVisible = true
            view.stateThreeImage.setImageResource(R.drawable.state3)
            view.status.text = (context as MainActivity).getString(R.string.statues3)
            view.statusLine3.setBackgroundColor((context as MainActivity).resources.getColor(R.color.orange))
        } catch (e: Exception) {
        }
    }

    private fun statusDelivered() {
        try {
            view.status.text = (context as MainActivity).getString(R.string.delivered)
            view.stateFourImage.setImageResource(R.drawable.state4)
            view.stateFourImage.isVisible = true
            view.countDownLayout.isVisible = false

        } catch (e: Exception) {
        }
    }

    private fun canceledStatus() {
        Error_MotionToast(
            (context as MainActivity).getString(R.string.orderCanceled),
            (context as MainActivity)
        )
    }

    private fun startTimer(){
        view.countDownLayout.isVisible = true

        timer = object : CountDownTimer(secondsRemaining * 1000, 1000) {
            override fun onFinish() {
                view.countDownLayout.isVisible = false
            }
            override fun onTick(millisUntilFinished: Long) {
                secondsRemaining = millisUntilFinished / 1000
                updateCountdownUI()
            }
        }.start()
    }

    private fun updateCountdownUI() {
        val minutesUntilFinished = secondsRemaining / 60
        val secondsInMinuteUntilFinished = secondsRemaining - minutesUntilFinished * 60
        val secondsStr = secondsInMinuteUntilFinished.toString()
        view.remainingTime.text = "$minutesUntilFinished:${if (secondsStr.length == 2) secondsStr else "0" + secondsStr}"
    }


}

