package com.codes_roots.imakeapp.presentaion.myorders

import android.graphics.Color
import android.graphics.PorterDuff
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.core.view.isVisible
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.databinding.MyOrdersFragmentBinding
import com.codes_roots.imakeapp.helper.BaseApplication
import com.codes_roots.imakeapp.helper.OrdersStatus
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.presentaion.myorders.adapter.MyOrdersAdapter
import com.codes_roots.imakeapp.presentaion.myorders.mvi.MainIntent
import com.codes_roots.imakeapp.presentaion.myorders.mvi.MyOrderViewModel
import com.codes_roots.imakeapp.presentaion.vendordetailsfragment.viewmodel.RestaurantDetailsViewModel
import com.satafood.core.entities.myorders.MyOrderModelItem
import com.satafood.core.entities.ordermodel.OrderModel
import kotlinx.coroutines.flow.collect
import java.util.ArrayList
import javax.inject.Inject

class MyOrdersFragment : Fragment() {

    @Inject
    lateinit var viewModelFactory: ViewModelProvider.Factory

    val viewModel by viewModels<MyOrderViewModel> { viewModelFactory }
    private val viewModelRestaurants by viewModels<RestaurantDetailsViewModel> { viewModelFactory }

    @Inject
    lateinit var Pref: PreferenceHelper

    lateinit var myOrdersAdapter: MyOrdersAdapter

    var data: ArrayList<MyOrderModelItem>? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        BaseApplication.appComponent.inject(this)

    }

    private lateinit var view: MyOrdersFragmentBinding


    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?,
    ): View {

        view = DataBindingUtil.inflate(inflater, R.layout.my_orders_fragment, container, false)

        myOrderRecycleView()

        view.backBtn.setOnClickListener { requireActivity().onBackPressed() }
        kotlin.runCatching {
            view.head.background.setColorFilter(
                Color.parseColor(Pref.appPrimaryColor),
                PorterDuff.Mode.SRC_ATOP
            )
//            view.pending.setOnClickListener {
//                noColoredView()
//                coloredView(view.pending)
//                viewModel.intents.trySend(MainIntent.FilterData(viewModel.state.value!!.copy(progress = true), OrderModel(order_status_id = OrdersStatus.PENDING.status) ))
//
//            }

            view.current.setOnClickListener {
                noColoredView()
                coloredView(view.current)
                viewModel.intents.trySend(MainIntent.FilterData(viewModel.state.value!!.copy(progress = true,noOrderYet = false), OrderModel(order_status_id = OrdersStatus.CURRENT.status) ))

            }

//            view.ready.setOnClickListener {
//                noColoredView()
//                coloredView(view.ready)
//                viewModel.intents.trySend(MainIntent.FilterData(viewModel.state.value!!.copy(progress = true), OrderModel(order_status_id =  OrdersStatus.PREPARED.status)))
//            }

//            view.delivering.setOnClickListener {
//                noColoredView()
//                coloredView(view.delivering)
//                viewModel.intents.trySend(MainIntent.FilterData(viewModel.state.value!!.copy(progress = true), OrderModel(order_status_id =   OrdersStatus.DELIVERING.status)))
//            }

            view.previous.setOnClickListener {
                noColoredView()
                coloredView(view.previous)
                viewModel.intents.trySend(MainIntent.FilterData(viewModel.state.value!!.copy(progress = true,noOrderYet = false), OrderModel(order_status_id = OrdersStatus.PREVIOUS.status) ))

            }

//            view.canceledOrders.setOnClickListener {
//                noColoredView()
//                coloredView(view.canceledOrders)
//                viewModel.intents.trySend(MainIntent.FilterData(viewModel.state.value!!.copy(progress = true), OrderModel(order_status_id =   OrdersStatus.CLIENT_CANCEL.status)))
//            }

        }.getOrElse { throw it }

        viewModel.intents.trySend(MainIntent.Initialize(viewModel.state.value!!.copy(progress = true,noOrderYet = false)))

        getAllData()
        return view.root
    }

    fun myOrderRecycleView() {
        myOrdersAdapter = MyOrdersAdapter(viewModel.intents, requireContext(),viewModel,this)
        view.ordersRv.apply {
            adapter = myOrdersAdapter;
            setNestedScrollingEnabled(false)
            setHasFixedSize(true)
        }
    }

    private fun getAllData() {
        lifecycleScope.launchWhenStarted {
            viewModel.state.collect {
                if (it != null) {
                    if (it.error != null) {
                        it.error?.let {
                            when (it) {
                                is UserError.InvalidId -> "Invalid id"
                                is UserError.NetworkError -> it.throwable.message
                                UserError.ServerError -> "Server error"
                                UserError.Unexpected -> "Unexpected error"
                                is UserError.UserNotFound -> "User not found"
                                UserError.ValidationFailed -> "Validation failed"
                                else -> {

                                }
                            }
                        }
                    } else {
                        view.progress.isVisible = it.progress!!
                        if (it.noOrderYet!!)
                        {
                    view.productNotFound.isVisible = it.noOrderYet!!
                    view.animationView.playAnimation()
                }

                        data = it.data
                        myOrdersAdapter.submitList(it.data)
                    }
                }
            }
        }
    }

    private fun coloredView(view: TextView) {
        view.setBackgroundDrawable(ContextCompat
            .getDrawable(requireContext(), R.drawable.gradient_round_bg))
        view.setTextColor(requireContext().resources.getColor(R.color.white))
    }
    private fun whiteView(view: TextView) {
        view.setBackgroundDrawable(ContextCompat.getDrawable(requireContext(), R.drawable.orders_tv_bg))
        view.setTextColor(requireContext().resources.getColor(R.color.gray_font))
    }

    private fun noColoredView() {
        view.progress.isVisible = true
//        whiteView(view.ready)
        whiteView(view.current)
        whiteView(view.previous)
//        whiteView(view.canceledOrders)
//        whiteView(view.delivering)
//        whiteView(view.pending)
    }
}


