package com.codes_roots.imakeapp.presentaion.myorders.adapter

import android.annotation.SuppressLint
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.CountDownTimer
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.view.isVisible
import androidx.databinding.DataBindingUtil
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.databinding.MyorderItemAdapterBinding
import com.codes_roots.imakeapp.helper.OrdersStatus
import com.codes_roots.imakeapp.helper.SUCCESS_MotionToast
import com.codes_roots.imakeapp.helper.getCurrentDateFormatted
import com.codes_roots.imakeapp.presentaion.ClickHandler
import com.codes_roots.imakeapp.presentaion.mainactivity.MainActivity
import com.codes_roots.imakeapp.presentaion.myorders.MyOrdersFragment
import com.codes_roots.imakeapp.presentaion.myorders.mvi.MainIntent
import com.codes_roots.imakeapp.presentaion.myorders.mvi.MainViewState
import com.codes_roots.imakeapp.presentaion.myorders.mvi.MyOrderViewModel
import com.satafood.core.entities.login.LoginModel
import com.satafood.core.entities.myorders.MyOrderModelItem
import com.satafood.core.entities.myorders.OrderStatus
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.flow.MutableStateFlow
import java.text.SimpleDateFormat
import java.util.*

class MyOrdersAdapter(
    var Intent: Channel<MainIntent>,
    var context: Context,
    var viewmodel: MyOrderViewModel,
    var frag: MyOrdersFragment
) :
    ListAdapter<MyOrderModelItem, MenuViewHolder>(MenuDiffCallback()) {
    lateinit var viewModel: MutableStateFlow<MainViewState?>
    val scope = CoroutineScope(Dispatchers.Main)
    val calendar = Calendar.getInstance()
    val timeFormat = SimpleDateFormat("mm")

    override fun onBindViewHolder(holder: MenuViewHolder, position: Int) {
        holder.bind(context, viewmodel, currentList[position])

        holder.binding.gotoRestaurant.setOnClickListener {

            ClickHandler().goToBranchLocation((context as MainActivity),
                currentList[position].latitude!!,currentList[position].longitude!!)
        }


        val orderCreation = currentList[position].getCreationDate()
        val currentTimeFormat = timeFormat.format(calendar.time).toInt()
        val orderTimeFormat = currentList[position].created?.let {
            currentList[position].getCreationMinute(it)
        }!!.toInt()
        val getCurrentDateTime = getCurrentDateFormatted()
        val countDownTimerNum = (currentTimeFormat - orderTimeFormat).toLong()
        val orderId = currentList[position].id
        val orderStatus = currentList[position].order_status_id

        if (orderStatus == 7) {
            holder.binding.canceled.canceledOrder.visibility = View.VISIBLE
            holder.binding.statusLayout.visibility = View.INVISIBLE
        }


        if (getCurrentDateTime.equals(orderCreation)) {
            if (countDownTimerNum < 5 && orderStatus == 0 || countDownTimerNum < 5 && orderStatus == 1) {
                holder.binding.cancelActionLayout.isVisible = true

                holder.binding.cancelActionLayout.setOnClickListener {

                    try {

                        holder.binding.cancelActionLayout.isVisible = false
                        viewmodel.changeOrderStatus(
                            currentList[position].id!!,
                            OrderStatus(order_status_id = OrdersStatus.CLIENT_CANCEL.status)
                        )
                        val gson = Gson()
                        val type = object : TypeToken<OrderStatus?>() {}.type
                        val data = OrderStatus(roomID = currentList[position].code, cancel="1")
                        val newdata = gson.toJson(data, type)
                        (context as MainActivity).mSocket?.emit("CancelOrderFromClient", newdata)
                        holder.sendNotificationToDevice(
                            currentList[position].BranchUserId!!,
                            "الغاء",
                            " تم الغاء الطلب رقم $orderId"
                        )
//                        frag.data!!.removeAt(position)
//                        notifyItemRemoved(position)
                        val intent = Intent(context, MainActivity::class.java)

                        context.startActivity(intent)
                        SUCCESS_MotionToast("تم الغاء الاوردر", context as MainActivity)
                    } catch (e: Exception) {
                    }

                }
                object : CountDownTimer((countDownTimerNum - 5) * -60000, 1000) {
                    override fun onTick(millisUntilFinished: Long) {
                        holder.binding.remainingTime.text = (millisUntilFinished / 1000).toString()
                    }

                    override fun onFinish() {
                        holder.binding.cancelActionLayout.isVisible = false
                    }
                }.start()

            } else {
                holder.binding.cancelActionLayout.visibility = View.GONE
            }
        } else {

            Log.d("Return", "getMyTime older than getCurrentDateTime ")


        }


    }


    override fun onCreateViewHolder(p0: ViewGroup, p1: Int): MenuViewHolder {
        val binding: MyorderItemAdapterBinding = DataBindingUtil.inflate(
            LayoutInflater.from(p0.context),
            R.layout.myorder_item_adapter, p0, false
        )

        return MenuViewHolder(binding)
    }
}

private class MenuDiffCallback : DiffUtil.ItemCallback<MyOrderModelItem>() {

    override fun areItemsTheSame(oldItem: MyOrderModelItem, newItem: MyOrderModelItem) =
        oldItem.id == newItem.id

    @SuppressLint("DiffUtilEquals")
    override fun areContentsTheSame(oldItem: MyOrderModelItem, newItem: MyOrderModelItem) =
        oldItem == newItem
}

class MenuViewHolder(
    val binding: MyorderItemAdapterBinding,
) : RecyclerView.ViewHolder(binding.root) {
    fun bind(context: Context?, viewModel: MyOrderViewModel, data: MyOrderModelItem) {
        binding.listener = ClickHandler()
        binding.data = data
        binding.context = context as MainActivity?
        binding.viewmodel = viewModel
    }

    fun sendNotificationToDevice(user_id: Int, title: String, message: String) {
        val notificationInfo = LoginModel(
            title = title, user_id = user_id, message = message
        )
        binding.viewmodel?.sendNotificationToDevice(notificationInfo)

    }

}
