package com.codes_roots.imakeapp.presentaion.myorders.mvi


import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.repository.homepage.DataRepo
import com.satafood.core.entities.myorders.MyOrderModelItem
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first


/**
 * this is the model function in MVI, it's responsibility is to convert intents into views states
 */
suspend fun mapIntentToViewState(
    intent: MainIntent,
    Datarepo: DataRepo,
    loadMainData: suspend () -> Flow<Result<ArrayList<MyOrderModelItem>>> = { Datarepo.getUserOrders },
    loadFilteredData: suspend () -> Flow<Result<ArrayList<MyOrderModelItem>>> = { Datarepo.getFilteredUserOrders(intent.stateId!!) },
) = when (intent) {
    is MainIntent.Initialize -> proceedWithInitialize(loadMainData, intent)
    is MainIntent.ErrorDisplayed -> intent.viewState.copy(error = null)
    is MainIntent.FilterData -> proceedWithFilterData(loadFilteredData,intent)
}

//            filterData=filterOrder(OrdersStatus.PENDING.status,data.getOrThrow() ) as ArrayList<MyOrderModelItem>,
private suspend fun proceedWithInitialize(
    loadCart: suspend () -> Flow<Result<ArrayList<MyOrderModelItem>>>,
    intent: MainIntent,
): MainViewState {
    val response = loadCart()
    val data = response.first()
    return runCatching {
        if (data.getOrThrow().isEmpty()){intent.viewState!!.copy(data = (data.getOrThrow()),
            noOrderYet = true,
            error = null,
            progress = false)
        }else{
            intent.viewState!!.copy(data = (data.getOrThrow()),
                noOrderYet = false,
                error = null,
                progress = false)
            }
    } .getOrElse {
            intent.viewState!!.copy(error = UserError.NetworkError(it))
        }

}
private suspend fun proceedWithFilterData(
    loadCart: suspend () -> Flow<Result<ArrayList<MyOrderModelItem>>>,
    intent: MainIntent,
): MainViewState {
    val response = loadCart()
    val data = response.first()
    return runCatching {
        intent.viewState!!.copy(data = (data.getOrThrow()),
            noOrderYet = false,
            error = null,
            progress = false)
    }
        .getOrElse {
            intent.viewState!!.copy(error = UserError.NetworkError(it))
        }

}


private fun filterDataByCatId(intent: MainIntent, order_status_id: Int): MainViewState {
    val filterDataArray = filterOrder(order_status_id, intent.viewState?.data!!)
    return intent.viewState!!.copy(filterData = filterDataArray as ArrayList<MyOrderModelItem>)
}

fun filterOrder(order_status_id: Int, productArray: ArrayList<MyOrderModelItem>?) =
    productArray!!.filter { data -> data.order_status_id == order_status_id }





