package com.codes_roots.imakeapp.presentaion.myorders.mvi

import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.codes_roots.imakeapp.presentaion.mainactivity.BaseViewModel
import com.codes_roots.imakeapp.repository.homepage.DataRepo
import com.codes_roots.imakeapp.repository.retaurantdetails.RemoteDataSource
import com.satafood.core.entities.login.LoginModel
import com.satafood.core.entities.myorders.MyOrderModelItem
import com.satafood.core.entities.myorders.OrderStatus
import kotlinx.coroutines.*
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.flow.receiveAsFlow
import org.json.JSONException
import org.json.JSONObject
import javax.inject.Inject


class MyOrderViewModel @Inject constructor(
    private val DateRepoCompnay: DataRepo,
    private val Datasources: RemoteDataSource,
) : BaseViewModel<MainViewState>() {

    val intents: Channel<MainIntent> = Channel<MainIntent>(Channel.UNLIMITED)

    protected val uiState: MutableStateFlow<MainViewState?> = MutableStateFlow(MainViewState())

    val state: MutableStateFlow<MainViewState?> get() = uiState

    private var job: Job? = null

    var OrderStateLD: MutableLiveData<OrderStatus>? = null
    var reOrderLD: MutableLiveData<Double>? = null
    var authLD: MutableLiveData<LoginModel>? = null

    private val viewStateMapper: suspend (MainIntent)

    -> MainViewState = { mapIntentToViewState(it, DateRepoCompnay) }

    val coroutineExceptionHandler = CoroutineExceptionHandler { _, throwable ->
        throwable.printStackTrace()
    }

    init {
        OrderStateLD = MutableLiveData()
        authLD = MutableLiveData()
        reOrderLD= MutableLiveData()
        getIntent()

        // لازم ابعت viewstate كامل واتحكم بكل option حسب المطلوب وارساله مرة اخري واستقباله هنا او في ال view

        uiState.value = MainViewState().copy(progress = true)

        // launchNextViewStateJob(intents.value)
    }

    fun getIntent() {
        job = viewModelScope.launch() {
            intents.receiveAsFlow().collect {
                uiState.value = (viewStateMapper(it))
            }
        }
    }
    fun preOrder(orderInfo: MyOrderModelItem?) {
        kotlin.runCatching {
            job = CoroutineScope(Dispatchers.IO).launch {
                val response = Datasources.preOrder(orderInfo!!)
                withContext(Dispatchers.Main + coroutineExceptionHandler) {
                    if (response.isSuccessful) {
                        reOrderLD?.postValue(response.body())
                    } else {
                        if (response.code() == 405) {
                            var jsonObject: JSONObject? = null
                            try {
                                jsonObject = JSONObject(response.errorBody()!!.string())

                            } catch (e: JSONException) {
                                e.printStackTrace()
                            }
                        } else {
                            onError("Error : ${response.message()} ")
                        }

                    }
                }
            }
        }.getOrElse { throw it }
    }


    fun sendNotificationToDevice(loginModel: LoginModel?) {
        val coroutineExceptionHandler = CoroutineExceptionHandler { _, throwable ->
            throwable.printStackTrace()
        }
        job = CoroutineScope(Dispatchers.Default + coroutineExceptionHandler).launch {
            val response = Datasources.sendNotificationToDevice(loginModel!!)
            withContext(Dispatchers.Main + coroutineExceptionHandler) {
                if (response.isSuccessful) {
                    authLD?.postValue(response.body())

                } else {
                    onError("Error : ${response.message()} ")
                }
            }
        }
    }

    fun changeOrderStatus(Id: Int, data: OrderStatus) {
        job = CoroutineScope(Dispatchers.IO + coroutineExceptionHandler).launch {
            val response = DateRepoCompnay.changeOrderStatus(Id, data)
            withContext(Dispatchers.Main + coroutineExceptionHandler) {
                (response.collect {
                    runCatching {
                        OrderStateLD?.value = it.getOrNull()!!
                    }.getOrElse {
                        onError("Error : ${it.message} ")
                    }
                })
            }
        }
    }

    override fun onCleared() {
        super.onCleared()
        job!!.cancel()
    }


    private fun onError(message: String) {
//        errorMessage.value = message
//        loading.value = false
    }
}




