package com.codes_roots.imakeapp.presentaion.mypoints

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.view.isVisible
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.databinding.FragmentPointBinding
import com.codes_roots.imakeapp.helper.*
import com.codes_roots.imakeapp.presentaion.ClickHandler
import com.codes_roots.imakeapp.presentaion.homefragment.HomeFragment
import com.codes_roots.imakeapp.presentaion.mainactivity.MainActivity
import com.codes_roots.imakeapp.presentaion.mypoints.mvi.MainIntent
import com.codes_roots.imakeapp.presentaion.mypoints.viewmodel.MyPointsViewModel
import com.satafood.core.entities.userpoints.CountPointsModel

import kotlinx.coroutines.flow.collect

import javax.inject.Inject

class PointsFragment @Inject constructor() : Fragment()
{

    private lateinit var binding: FragmentPointBinding

    @Inject
    lateinit var Pref: PreferenceHelper

    @Inject
    lateinit var viewModelFactory: ViewModelProvider.Factory

    val viewModel by viewModels<MyPointsViewModel> { viewModelFactory }
    private var progr = 0
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        BaseApplication.appComponent.inject(this)


    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?,
    ): View {
        binding = FragmentPointBinding.inflate(inflater, container, false)
        val root: View = binding.root

        viewModel.transferPointsLD!!.observe(requireActivity()) {
            updateProgressBar(progr)
            if (it.message != null) {
                Error_MotionToast(it.message!!,requireActivity())
            }else {
                SUCCESS_MotionToast(requireActivity().getString(R.string.PointTransfer),context as MainActivity)
                ClickHandler().SwitchBetweenFragments(requireContext(),HomeFragment((context as MainActivity).fragmentFactory))
            }
            binding.totalPointsValue.text = "0"


        }
        return root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        viewModel.intents.trySend(MainIntent.Initialize(viewModel.state.value!!))
        binding.loading = true



        getAllData()
    }


    private fun getAllData() {
        lifecycleScope.launchWhenStarted {
            viewModel.state.collect {
                if (it != null) {
                    if (it.error != null) {
                        viewModel.intents.send(MainIntent.ErrorDisplayed(it))
                        binding.tvNoPoints.isVisible = true
                    } else {
                        if (it.progress == true) {
                            binding.layoutProgress.visibility = View.VISIBLE
                        } else {
                            try {
                                val minimum = it.data!!.setting!!.min_withdrawal!!
                                val totalPoints = it.data!!.userPoints!!.TotalPoints!!.toInt()

                                binding.transferPointsToWalletButton.setOnClickListener {
                                    val count = CountPointsModel(count = totalPoints)
                                    viewModel.transferPoints(count)
                                    binding.transferPointsToWalletButton.isEnabled = false
                                }
                                if (totalPoints >= minimum) {
//                                    binding.textViewProgress.text =
//                                        requireActivity().getString(R.string.canTransfer)
                                    progr = 100
                                    binding.transferPointsToWalletButton.isEnabled = true
                                    updateProgressBar(progr)
                                    binding.percent!!.text = "$progr %"
                                } else {
                                    binding.PointsLeft.text = (minimum - totalPoints).toString().plus(" ").plus(context!!.getString(R.string.points))
//                                    binding.textViewProgress.text =
//                                        requireActivity().getString(R.string.pointsLeft)
                                    val percentage = (totalPoints.toDouble() / minimum) * 100
                                    updateProgressBar(percentage.toInt())
                                    binding.transferPointsToWalletButton.isEnabled = false

                                    progr = percentage.toInt()
                                    binding.percent!!.text = "$progr %"
                                }
                            } catch (e: Exception) {
                                binding.textViewProgress.text =
                                    requireActivity().getString(R.string.no_wallets_to_points)
                            }
                            binding.data = it.data
                            binding.layoutProgress.visibility = View.GONE
                            binding.totalPointsValue.text = "0"
                        }
                    }
                }

            }
        }
    }

    private fun updateProgressBar(percentage: Int) {
        binding.progressBar.progress = percentage
    }
}



