package com.codes_roots.imakeapp.presentaion.mypoints.mvi


import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.repository.points_user.PointsDataRepo
import com.satafood.core.entities.userpoints.UserPointsModel
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first


/**
 * this is the model function in MVI, it's responsibility is to convert intents into views states
 */
suspend fun mapIntentToViewState(
    intent: MainIntent,
    Datarepo: PointsDataRepo,
    loadMainData: suspend () -> Flow<Result<UserPointsModel>> = { Datarepo.getUserPoints() },
) = when (intent) {
    is MainIntent.Initialize -> proceedWithInitialize(loadMainData, intent)
    is MainIntent.ErrorDisplayed -> intent.viewState.copy()
}


private suspend fun proceedWithInitialize(
    loadCart: suspend () -> Flow<Result<UserPointsModel>>,
    intent: MainIntent,
): MainViewState {
    val response = loadCart()
    val data = response.first()
    return runCatching {
        intent.viewState!!.copy(data = (data.getOrThrow()),
            noPointsYet = true,
            error = null,
            progress = false)
    }
        .getOrElse {
            intent.viewState!!.copy(error = UserError.NetworkError(it))
        }

}






