package com.codes_roots.imakeapp.presentaion.mypoints.viewmodel

import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.codes_roots.imakeapp.presentaion.mainactivity.BaseViewModel
import com.codes_roots.imakeapp.presentaion.mypoints.mvi.MainIntent
import com.codes_roots.imakeapp.presentaion.mypoints.mvi.MainViewState
import com.codes_roots.imakeapp.presentaion.mypoints.mvi.mapIntentToViewState
import com.codes_roots.imakeapp.repository.points_user.PointsDataRepo
import com.codes_roots.imakeapp.repository.points_user.PointsRemoteDataSource
import com.satafood.core.entities.userpoints.CountPointsModel
import kotlinx.coroutines.*
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.flow.receiveAsFlow
import javax.inject.Inject


class MyPointsViewModel @Inject constructor(
    private val DateRepoCompnay: PointsDataRepo,
    private val Datasources:PointsRemoteDataSource,
) : BaseViewModel<MainViewState>() {

    val intents: Channel<MainIntent> = Channel<MainIntent>(Channel.UNLIMITED)

    protected val uiState: MutableStateFlow<MainViewState?> = MutableStateFlow(MainViewState())

    val state: MutableStateFlow<MainViewState?> get() = uiState

    private var job: Job? = null

    var transferPointsLD: MutableLiveData<CountPointsModel>? = null


    val errorMessage = MutableLiveData<String>()
    val loading = MutableLiveData<Boolean>()

    private val viewStateMapper: suspend (MainIntent)

    -> MainViewState = { mapIntentToViewState(it, DateRepoCompnay) }

    val coroutineExceptionHandler = CoroutineExceptionHandler { _, throwable ->
        throwable.printStackTrace()
    }

    init {
        transferPointsLD = MutableLiveData()
        getIntent()

        // لازم ابعت viewstate كامل واتحكم بكل option حسب المطلوب وارساله مرة اخري واستقباله هنا او في ال view

        uiState.value = MainViewState().copy(progress = true)

        // launchNextViewStateJob(intents.value)
    }

    fun getIntent() {

        job = viewModelScope.launch() {

            intents.receiveAsFlow().collect {

                uiState.value = (viewStateMapper(it))

            }
        }
    }


    fun transferPoints(userPoints: CountPointsModel) {
        job = CoroutineScope(Dispatchers.IO+coroutineExceptionHandler).launch {
            val response = Datasources.transferPoints(userPoints!!)
            withContext(Dispatchers.Main+coroutineExceptionHandler) {
                if (response.isSuccessful) {
                    transferPointsLD?.postValue(response.body())

                } else {
                    onError("Error : ${response.message()} ")
                }
            }
        }

    }
    override fun onCleared() {
        super.onCleared()
        job!!.cancel()
    }

    private fun onError(message: String) {
        errorMessage.value = message
        loading.value = false
    }

}




