package com.codes_roots.imakeapp.presentaion.mywallet

import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.os.bundleOf
import androidx.fragment.app.Fragment
import androidx.fragment.app.setFragmentResult
import androidx.fragment.app.viewModels
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import com.codes_roots.imakeapp.databinding.FragmentWalletBinding
import com.codes_roots.imakeapp.helper.*
import com.codes_roots.imakeapp.presentaion.cartfragment.viewmodel.CartViewModel
import com.codes_roots.imakeapp.presentaion.mainactivity.MainActivity
import com.codes_roots.imakeapp.presentaion.mywallet.mvi.WalletIntent
import com.satafood.core.base.DataState
import com.satafood.core.entities.wallet.WalletModel
import kotlinx.android.synthetic.main.fragment_point.*
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.launch
import javax.inject.Inject

class WalletFragment @Inject constructor() : Fragment() {

    private var binding: FragmentWalletBinding by autoCleared()

    lateinit var walletAdapter: WalletAdapter

    @Inject
    lateinit var Pref: PreferenceHelper

    @Inject
    lateinit var viewModelFactory: ViewModelProvider.Factory

    val viewModel by viewModels<WalletViewModel> { viewModelFactory }

    val viewModelCart by viewModels<CartViewModel> { viewModelFactory }

    var walletClickable: Boolean = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        BaseApplication.appComponent.inject(this)
        arguments?.let {
            walletClickable = it.getBoolean("walletClickable")
        }

    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        // Inflate the layout for this fragment
        binding = FragmentWalletBinding.inflate(inflater, container, false)

        val root: View = binding.root

        binding.backBtn.setOnClickListener { requireActivity().onBackPressed() }
         binding.context = context as MainActivity
        return root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        setupWalletAdapter()

        observeStateFlow()
        lifecycleScope.launch {
            Pref.token?.let {
                viewModel.intentChannel.send(WalletIntent.GetWalletsIntent(it))
                viewModel.intentChannel.send(WalletIntent.GetUserCreditIntent(it))
            }
        }


    }

    private fun observeStateFlow() {
        lifecycleScope.launchWhenStarted {
            viewModel.wallets.collect {
                when (it.status) {
                    DataState.Status.LOADING -> binding.loading = true
                    DataState.Status.SUCCESS -> {
                        binding.loading = false
                        it.data?.let { wallets -> setWalletsList(wallets) }

                    }
                    DataState.Status.ERROR -> {
                        binding.loading = false
                    }
                }
            }
        }
        lifecycleScope.launchWhenStarted {
            viewModel.walletsCredit.collect {
                when (it.status) {
                    DataState.Status.LOADING -> binding.loading = true
                    DataState.Status.SUCCESS -> {
                        binding.loading = false
                        if (it.data?.TotalWallet != null) {
                            val x = it.data?.TotalWallet
                            Log.i("totalwalletVal", "observeStateFlow:$x ")
                            binding.totalWallet = x?.toFloat().toString()

                        } else {
                            binding.totalWallet = "0"
                        }
                    }
                    DataState.Status.ERROR -> {
                        binding.loading = false

                        //    Error_MotionToast(it.error?.throwableMessage.toString(), requireActivity())
                    }
                }
            }
        }
    }

    private fun setWalletsList(wallets: List<WalletModel>) {
        if (!wallets.isNullOrEmpty()) {
            walletAdapter.submitList(wallets)
            binding.walletsAvailable = true
        } else {
            binding.walletsAvailable = false
        }
    }

    private fun setupWalletAdapter() {
        walletAdapter = WalletAdapter(requireContext()) { wallet ->
            if (walletClickable) {
                setFragmentResult("WALLET_REQUEST", bundleOf("wallet" to wallet))
                requireActivity().supportFragmentManager.popBackStack();
            }
        }
        binding.rvWallets.apply {
            adapter = walletAdapter
            addItemDecoration(
                VerticalMarginItemDecoration(
                    dpToPx(10f, requireContext())
                )
            )
        }

    }


}