package com.codes_roots.imakeapp.presentaion.mywallet

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.satafood.core.base.DataState
import com.codes_roots.imakeapp.presentaion.mywallet.mvi.WalletIntent
import com.codes_roots.imakeapp.repository.wallet.GetUserCreditUseCase
import com.codes_roots.imakeapp.repository.wallet.GetWalletListUseCase
import com.satafood.core.entities.wallet.WalletModel
import com.satafood.core.entities.wallet.WalletCredit
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch
import javax.inject.Inject

class WalletViewModel @Inject constructor(
    private val getWalletListUseCase: GetWalletListUseCase,
    private val getUserCreditUseCase: GetUserCreditUseCase,

) :
    ViewModel() {

    val intentChannel = Channel<WalletIntent>(Channel.UNLIMITED)

    private val _wallets: MutableStateFlow<DataState<List<WalletModel>>> =
        MutableStateFlow(DataState(DataState.Status.LOADING))

    val wallets = _wallets.asStateFlow()

    private val _walletsCredit: MutableStateFlow<DataState<WalletCredit>> =
        MutableStateFlow(DataState<WalletCredit>(DataState.Status.LOADING))

    val walletsCredit = _walletsCredit.asStateFlow()

    init {
        processIntent()

    }

    private fun processIntent() {
        viewModelScope.launch {
            intentChannel.consumeAsFlow().collect {
                when (it) {
                    is WalletIntent.GetWalletsIntent -> getWallets()
                    is WalletIntent.GetUserCreditIntent -> getCredit()
                }
            }
        }
    }

    private fun getWallets() {
        viewModelScope.launch() {
            getWalletListUseCase()
                .buffer()
                .onEach { _wallets.emit(it) }
                .launchIn(viewModelScope)
        }
    }

    private fun getCredit() {
        viewModelScope.launch() {
            getUserCreditUseCase()
                .buffer()
                .onEach { _walletsCredit.emit(it) }
                .launchIn(viewModelScope)
        }
    }

    fun test (action: suspend() -> Unit){

    }
}