package com.codes_roots.imakeapp.presentaion.notification

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.view.isVisible
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.databinding.NotificaionFragmentBinding
import com.codes_roots.imakeapp.helper.BaseApplication
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.presentaion.ClickHandler
import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.presentaion.notification.mvi.MainIntent
import com.codes_roots.imakeapp.presentaion.notification.mvi.NotificationViewModel
import kotlinx.coroutines.flow.collect
import javax.inject.Inject


open class NotificationFragment @Inject constructor() : Fragment() {

    @Inject
    lateinit var viewModelFactory: ViewModelProvider.Factory

    private val viewModel by viewModels<NotificationViewModel> { viewModelFactory }

    lateinit var notificationAdapter: NotificationAdapter
    private lateinit var view: NotificaionFragmentBinding

    @Inject
    lateinit var pref: PreferenceHelper
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        BaseApplication.appComponent.inject(this)
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {

        view = DataBindingUtil.inflate(
            inflater
            , R.layout.notificaion_fragment, container, false )



        getAllData()
        notificationsRecycleView()
        ClickHandler().bottomNavItemSelected(requireContext(), R.id.home, false)
        viewModel.intents.trySend(MainIntent.Initialize(viewModel.state.value!!,pref.googleID,pref.UserId))
        return view.root
    }


    fun notificationsRecycleView() {
        notificationAdapter = NotificationAdapter(viewModel, requireContext())
        view.notificationRecycle.apply {
            adapter = notificationAdapter;
            isNestedScrollingEnabled = false
            setHasFixedSize(true)
        }
    }


    fun getAllData() {
        lifecycleScope.launchWhenStarted {
            viewModel.state.collect {
                if (it != null) {
                    if (it.error != null) {
                        it.error?.let { userError ->
                            when (userError) {
                                is UserError.InvalidId -> "Invalid id"
                                is UserError.NetworkError -> userError.throwable.message
                                UserError.ServerError -> "Server error"
                                UserError.Unexpected -> "Unexpected error"
                                is UserError.UserNotFound -> "User not found"
                                UserError.ValidationFailed -> "Validation failed"
                                else -> {}
                            }
                        }
                        viewModel.intents.trySend(MainIntent.ErrorDisplayed(it))
                    } else {
                        view.progress.isVisible = false

                        if (!it.data.isNullOrEmpty()) {

                            notificationAdapter.submitList(it.data)
                            view.noOrderFoundLayout.isVisible = false

                        } else
                            view.noOrderFoundLayout.isVisible = true
                    }
                }

            }
        }
    }
}