package com.codes_roots.imakeapp.presentaion.notification.mvi


import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.repository.homepage.DataRepo
import com.satafood.core.entities.notification.NotificationModelItem
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first


/**
 * this is the model function in MVI, it's responsibility is to convert intents into views states
 */
suspend fun mapIntentToViewState(intent: MainIntent, Datarepo: DataRepo,
                                 loadMainData: suspend () -> Flow<Result<ArrayList<NotificationModelItem>>> = { Datarepo.getNotifications(intent.userId!!,intent.googleId!!) },
) = when (intent) {
    is MainIntent.Initialize -> proceedWithInitialize(loadMainData,intent)
    is MainIntent.ErrorDisplayed -> intent.viewState.copy(error = null)
}


private suspend fun proceedWithInitialize(loadCart: suspend () -> Flow<Result<ArrayList<NotificationModelItem>>>, intent:MainIntent): MainViewState {
    val response =   loadCart()
    val data = response.first()
    return runCatching {
        intent.viewState!!.copy(data = (data.getOrThrow()),noFavouriteYet = true, error = null, progress = false)
    }
        .getOrElse {
            intent.viewState!!.copy(error = UserError.NetworkError(it))
        }
}




