package com.codes_roots.imakeapp.presentaion.paymentfragment

import android.content.Intent
import android.graphics.Color
import android.graphics.Paint
import android.graphics.PorterDuff
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.appcompat.content.res.AppCompatResources
import androidx.core.view.isVisible
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentFactory
import androidx.fragment.app.setFragmentResultListener
import androidx.fragment.app.viewModels
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.databinding.PaymentFragmentBinding
import com.codes_roots.imakeapp.helper.*
import com.codes_roots.imakeapp.paymob_activity.PaymobActivity
import com.codes_roots.imakeapp.presentaion.ClickHandler
import com.codes_roots.imakeapp.presentaion.cartfragment.mvi.CartViewState
import com.codes_roots.imakeapp.presentaion.cartfragment.viewmodel.CartViewModel
import com.codes_roots.imakeapp.presentaion.confirmedfragment.ConfirmedFragment
import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.presentaion.mainactivity.MainActivity
import com.codes_roots.imakeapp.presentaion.paymentfragment.mvi.CheckoutIntents
import com.codes_roots.imakeapp.presentaion.paymentfragment.mvi.CheckoutViewModel
import com.codes_roots.imakeapp.presentaion.register.otp_fragment.MobileVerificationFragment
import com.codes_roots.imakeapp.presentaion.splash.SplashScreen
import com.satafood.core.entities.google.GoogleModel
import com.satafood.core.entities.login.LoginModel
import com.satafood.core.entities.ordermodel.OrderDetail
import com.satafood.core.entities.ordermodel.OrderDetailsOption
import com.satafood.core.entities.ordermodel.OrderModel
import com.satafood.core.entities.paymentways.VoucherModel
import com.satafood.core.entities.wallet.WalletModel
import io.github.muddz.styleabletoast.StyleableToast
import kotlinx.coroutines.flow.collect
import java.util.*
import javax.inject.Inject
import kotlin.collections.ArrayList
import kotlin.math.roundToInt

private const val TAG = "PaymentFragmentServertest"

class PaymentFragment @Inject constructor(
    var fragmentFactory: FragmentFactory, var cartViewModel: CartViewModel,
) : Fragment() {

    lateinit var view: PaymentFragmentBinding
    var CartViewStates: CartViewState? = null
    var orderDetials = ArrayList<OrderDetail>()
    var paymentTypeId: Int? = 0
    var menuOptionID: List<OrderDetailsOption>? = null

    @Inject
    lateinit var cartViewModelFactory: ViewModelProvider.Factory

    val viewModel by viewModels<CheckoutViewModel> { cartViewModelFactory }

    @Inject
    lateinit var Pref: PreferenceHelper
    lateinit var paymentWayAdapter: PaymentWayAdapter

    var wallet: WalletModel? = null
    var walletId: Int? = null
    var subTotal: Double = 0.0
    var total: Double = 0.0
    var newTotal: Double = 0.0

    var taxes: Double = 0.0
    var discount: Double = 0.0

    var delivery_cost: Double = 0.0
    var voucher_id = "0"

    var branchId = 0
    var walletDiscount = 0.0
    var voucherDiscount = 0.0

    var isTakeaway = false


    override fun onCreate(savedInstanceState: Bundle?) {
        childFragmentManager.fragmentFactory = fragmentFactory
        super.onCreate(savedInstanceState)
        if (savedInstanceState == null) {
            BaseApplication.appComponent.inject(this)


        }
        getPayMobToken()
        observeOnResponse()

        try {
            branchId =
                cartViewModel.state.value?.ItemCartData?.restaurantDetails?.get(0)?.branch_id!!

            arguments?.let {

                subTotal = it.getDouble("total", 0.0)

            }

            setFragmentResultListener("WALLET_REQUEST") { key, bundle ->
                if (key == "WALLET_REQUEST") {
                    wallet = bundle.getSerializable("wallet") as WalletModel?
                    val subTotal = (total * 10.0).roundToInt() / 10.0
                    if (total >= wallet?.minimum!!) {
                        val checkoutConfigs =
                            GoogleModel(wallet_id = wallet!!.id, ordertotal = subTotal)

                        viewModel.intents.trySend(
                            CheckoutIntents.CheckWallet(
                                viewModel.state.value!!,
                                checkoutConfigs
                            )
                        )
                    } else
                        WARN_MotionToast(
                            requireContext().getString(R.string.minimum) + " " + wallet?.minimum,
                            requireActivity()
                        )
                }
            }

        } catch (e: Exception) {
        }
        cartViewModel.InActiveView.observe(requireActivity(), Observer {
            if (it == 1) {
                switchToVerificationFragmentWithData()
            }
        })
    }

    //
//
    private fun setTotalValue(total: Double) {

        view.total.apply {
            text = total.toString() + "  " + Pref.currency
            paintFlags = 0
        }
        view.newTotalValue.apply {
            this.isVisible = false
            view.newTotalValue.text = total.toString() + " " + Pref.currency
        }
    }

    private fun setNewTotalValue(total: Float) {
        view.total.apply {
            paintFlags = paintFlags or Paint.STRIKE_THRU_TEXT_FLAG
        }
        view.newTotalValue.isVisible = true
        view.newTotalValue.text = total.toString()
        newTotal = total.toDouble()

    }

    //
    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?,
    ): View {

        view = DataBindingUtil.inflate(inflater, R.layout.payment_fragment, container, false)
////
        CartViewStates = cartViewModel.state.value!!

        cartViewModel.VoucherLD?.value == null

        view.isTakeaway = isTakeaway
//
        view.subTotal = subTotal.toString()
//

        orderDetials.clear()
        val cartConfigs =
            GoogleModel(area_id = Pref.areaId, branch_id = Pref.BranchId, Subtotal = subTotal)

        viewModel.intents.trySend(
            CheckoutIntents.LoadData(
                viewModel.state.value!!,
                cartConfigs
            )
        )
        CartViewStates!!.CartItem!!.map { it ->
            try {
                orderDetials.add(
                    OrderDetail(
                        it.quintity,
                        it.id!!,
                        it.menu_options!!.map { menu_option ->
                            OrderDetailsOption(menu_option.id!!)
                        },
                        it.total!!, it.notes!!
                    )
                )
            } catch (e: Exception) {
            }
        }

        view.data = CartViewStates!!.ItemCartData
        view.listener = ClickHandler()

        view.pref = Pref

        view.context = context as MainActivity

        takeawayBtnVisibility(view)

        try {
            if (view.description.text == "")
                CartViewStates!!.ItemCartData!!.restaurantDetails?.first()
                    ?.cuisines!!.forEachIndexed { index, cuisine ->
                        if (CartViewStates!!.ItemCartData!!.restaurantDetails?.first()?.cuisines!!.lastIndex != index)
                            view.description.append(cuisine.name + " -")
                        else view.description.append(cuisine.name)
                    }
        } catch (e: Exception) {
        }

        view.completeInRestaurant.background.setColorFilter(
            Color.parseColor(Pref.appPrimaryColor),
            PorterDuff.Mode.SRC_ATOP
        )

        view.takeAway.background.setColorFilter(
            Color.parseColor(Pref.appPrimaryColor),
            PorterDuff.Mode.SRC_ATOP
        )

        view.takeAway2.background.setColorFilter(
            Color.parseColor(Pref.appPrimaryColor),
            PorterDuff.Mode.SRC_ATOP
        )
//        /// voucher
        view.useCodeButton.setOnClickListener {
            if (view.promoCode.length() > 4) {
                voucher_id = view.promoCode.text.toString()
                if (Pref.areaId != 0) {
                    view.animationView.isVisible = true
                    view.animationView.playAnimation()
                    try {
                        val voucherModel = VoucherModel(
                            code = view.promoCode.text.toString(),
                            ordertotal = total,
                            area_id = Pref.areaId.toString()
                        )
                        cartViewModel.checkVoucherRequest(voucherModel)
                    } catch (e: Exception) {
                    }
                } else {
                    WARN_MotionToast(
                        requireContext().getString(R.string.chooselocation),
                        requireActivity()
                    )
                }
            }
        }

        cartViewModel.VoucherLD!!.observe(requireActivity()) {
            view.animationView.cancelAnimation()
            view.animationView.isVisible = false
            if (it != null) {
                if (it.message == null) {
                    view.useCodeButton.isEnabled = false
                    setNewTotalValue(it.newTotal!!)

                } else
                    try {
                        WARN_MotionToast(it.message!!, requireActivity())
                    } catch (e: Exception) {
                    }
            }

        }
//
        // payment way cash or card
        if (!cartViewModel.isPaymentError) {
            cartViewModel.getPaymentWays()
        } else {
            StyleableToast.makeText(
                requireContext(),
                "Error loading payment methods, Please check your connection",
                Toast.LENGTH_SHORT, R.style.myInternetToast
            ).show()
        }



        cartViewModel.PaymentWaysLD.observe(viewLifecycleOwner) {
            paymentWayAdapter = PaymentWayAdapter(context, this, it!!)
            view.paymentRecycle.layoutManager =
                LinearLayoutManager(requireActivity())
            view.paymentRecycle.adapter = paymentWayAdapter
            view.paymentRecycle.setHasFixedSize(true)
        }
//


        view.backArrow.setOnClickListener { requireActivity().onBackPressed() }


        view.completeInRestaurant.setOnClickListener {
            view.completeInRestaurant.isEnabled = false
            orderRequest(isTakeaway)
        }

        view.takeAway.setOnClickListener {
            takeAwayButtonAction()
        }

        view.takeAway2.setOnClickListener { takeAwayButtonAction() }

        /// wallet
        view.ivRemoveCoupon.setOnClickListener {
            view.couponApplied = false
            view.tvCouponValue.text = "0" + Pref.currency
            view.useCodeButton.isEnabled = true
            val subTotal = (total * 10.0).roundToInt() / 10.0
            setTotalValue(subTotal)

        }

        cartViewModel.errorMessage.observe(viewLifecycleOwner) {
            if (it != null) {
                view.animationView.cancelAnimation()
                view.animationView.isVisible = false
                Error_MotionToast(it, context as MainActivity)
            }
        }
        getData()
//        }catch (e : Exception){
//            clickHandler.switchToActivity(requireContext(), SplashScreen())
//        }
        return view.root

    }

    private fun getData() {
        lifecycleScope.launchWhenStarted {
            viewModel.state.collect { it ->
                try {
                    if (it != null) {
                        try {
                            if (it.Errors != null) {
                                it.Errors.let {
                                    when (it) {
                                        is UserError.InvalidId -> "Invalid id"
                                        is UserError.NetworkError -> it.throwable.message
                                        is UserError.ServerError -> "Server error"
                                        is UserError.Unexpected -> "Unexpected error"
                                        is UserError.UserNotFound -> "User not found"
                                        is UserError.ValidationFailed -> "Validation failed"
                                        else -> "Unexpected error"
                                    }
                                }

                            } else {
                                if (it.checkOutData != null) {
                                    view.checkoutData = it.checkOutData
                                    val data = it.checkOutData!!.Confirgrations
                                    delivery_cost = data?.delivery_cost!!
                                    discount = data.discount!!
                                    total = data.Total!!
                                    taxes = data.totalTax!!
                                    val taxValue = (taxes * 10.0).roundToInt() / 10.0
                                    view.taxValue.text = taxValue.toString() + Pref.currency
                                }
                                if (it.CheckWallet != null) {
                                    walletId = it.CheckWallet?.id

                                    walletDiscount = it.CheckWallet!!.value!! // المخصوم من المحفظة

                                    setNewTotalValue(it.CheckWallet!!.newTotal!!)
//                                total = it.CheckWallet!!.newTotal!!
                                    view.couponApplied = true
                                    //   view.tv4.visibility = View.GONE

                                    //    view.discountValue.visibility = View.GONE

                                    view.tvCouponValue.text =
                                        walletDiscount.toString() + Pref.currency

                                }

                            }
                        } catch (e: Exception) {
                            Log.e(TAG, "getData: ${e.message}")
                        }
                    }
                } catch (e: Exception) {
                    Log.e(TAG, "getData: ${e.message}")
                }

            }

        }

    }

    private fun takeawayBtnVisibility(view: PaymentFragmentBinding) {
        //select locatin layout
        if (Pref.BillingId == 0) {
            view.whenNotChosen.visibility = View.VISIBLE
            view.whenChooseAddress.visibility = View.GONE
            if (Pref.takeAway == 1) {
                view.takeAway.visibility = View.GONE
                view.takeAway2.visibility = View.VISIBLE
            } else {
                view.takeAway.visibility = View.GONE
                view.takeAway2.visibility = View.GONE
            }
        } else {
            isTakeaway = false

            view.whenNotChosen.visibility = View.GONE
            view.whenChooseAddress.visibility = View.VISIBLE
            if (CartViewStates!!.ItemCartData!!.restaurantDetails?.get(0)?.has_takeway != null) {
                view.takeAway.visibility = View.VISIBLE
                view.takeAway2.visibility = View.GONE
            } else {
                view.takeAway2.visibility = View.GONE
                view.takeAway.visibility = View.GONE
            }
        }
    }

    private fun takeAwayButtonAction() {
        try {
            isTakeaway = true
            view.completeInRestaurant.visibility = View.VISIBLE

            total -= delivery_cost
            newTotal -= delivery_cost
            view.newTotalValue.text = newTotal.toString()
            val x = total.toString()
            view.total.text = x
            view.deliveryValue.text = "0"
            delivery_cost = 0.0
            view.takeAway2.backgroundTintList =
                AppCompatResources.getColorStateList(requireContext(), R.color.address_red)
            view.takeAway2.setTextColor(context!!.resources.getColor(R.color.white))
            view.takeAway.backgroundTintList =
                AppCompatResources.getColorStateList(requireContext(), R.color.address_red)
            view.takeAway.setTextColor(context!!.resources.getColor(R.color.white))
            view.whenNotChosen.visibility = View.VISIBLE
            view.takeAway2.visibility = View.VISIBLE
            view.takeAway.visibility = View.GONE

            view.whenChooseAddress.visibility = View.GONE
        } catch (e: Exception) {
            Error_MotionToast(e.toString(), requireActivity())
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        ResourceUtil().changeLang(Pref.lang!!, requireActivity())

        if (requestCode == 1000) {
            if (data != null) {
                takeAwayButtonAction()
            } else
                WARN_MotionToast(requireContext().getString(R.string.area), requireActivity())
        }
        var takeway = 0
        takeway = if (isTakeaway)
            1 else 0
        addOrder(takeway, Pref.OrderTotal.toDouble())
    }

    private fun observeOnResponse() {
        try {
            cartViewModel.OrderInfoLD?.observe(this) {
                if (it != null) {
                    val gson = Gson()
                    val type = object : TypeToken<OrderModel?>() {}.type
                    val newdata = gson.toJson(it, type)

                    (activity as MainActivity).mSocket?.emit("makeNewOrder", newdata)
                    SUCCESS_MotionToast("success", requireActivity())
                    sendNotificationToDevice(Pref.restaurantUserId, "سيتي رولز", "لديك طلب جديد")
                    Pref.clearCart()
                    //  cartViewModel.OrderInfoLD = null
                    ClickHandler().switchBetweenFragmentsWithoutBack(
                        requireContext(),
                        ConfirmedFragment()
                    )
                } else {
                    Error_MotionToast("error", requireActivity())
                }
            }
        } catch (e: Exception) {
            Error_MotionToast(e.toString(), requireActivity())
        }
    }

    private fun addOrder(takeaway: Int, total: Double) {
        val orderInfo = OrderModel(
            total = total,
            user_id = Pref.UserId,
            taxes = taxes,
            sub_total = subTotal,
            notes = CartViewStates!!.CartItem!!.first().notes,
            branch_id = Pref.BranchId,
            vendor_id = Pref.restaurantId,
            order_details = orderDetials,
            voucher_id = voucher_id,
            paymenttype_id = paymentTypeId,
            billing_address_id = Pref.BillingId,
            discount = discount,
            delivery_serivce = delivery_cost,
//            google_id = Pref.googleID,
            wallets_id = walletId,
            offer_id = 0,
            area_id = Pref.areaId,
            takeway = takeaway,
            offer_discount = 0.0,
            wallet_discount = walletDiscount
        )
        cartViewModel.addOrder(orderInfo)
    }

    fun orderRequest(takeaway: Boolean) {
        try {
            val total = if (!view.newTotalValue.isVisible)
                total
            else
                view.newTotalValue.text.toString().toDouble()
            if (Pref.BillingId != 0 || isTakeaway) { // check if user select address or takeway

                if (paymentTypeId != 0) {
                    if (paymentTypeId == 1) {
                        Pref.OrderTotal = total.toFloat()
                        val i = Intent(context, PaymobActivity::class.java)
                        (context as MainActivity).startActivityForResult(i, 100)
                    } else {
                        var takeway = 0
                        takeway = if (isTakeaway)
                            1 else 0
                        addOrder(takeway, total)
                    }
                } else {
                    view.completeInRestaurant.isEnabled = true
                    WARN_MotionToast(
                        context!!.getString(R.string.chooes_payment_way),
                        requireActivity()
                    )
                }
            } else {
                view.completeInRestaurant.isEnabled = true
                WARN_MotionToast(
                    context!!.getString(R.string.chooselocation),
                    requireActivity()
                )
            }
        } catch (e: Throwable) {
            WARN_MotionToast(
                context!!.getString(R.string.errorOccurred),
                requireActivity()
            )
        }
    }

    fun getPayMobToken() {
        cartViewModel.getPaymobToken(Constants.PayMobAuthKey)
        cartViewModel.PayMobTokenLD?.observe(this, androidx.lifecycle.Observer {
            Pref.PayMobToken = it?.token
            Log.i("tokenInfo", Pref.PayMobToken ?: "")
            Pref.MerchatId = it?.profile?.id
        })

    }

    private fun switchToVerificationFragmentWithData() {
        val fragment = MobileVerificationFragment()
        var bundle = Bundle()
        bundle.putInt("fromPaymentFragment", 1)
        fragment.arguments = bundle
        (context as MainActivity).supportFragmentManager.beginTransaction()
            .setCustomAnimations(R.anim.slide_top, 0, 0, R.anim.slide_down)
            .replace(R.id.main_frame, fragment).addToBackStack(null).commit()
    }

    fun sendNotificationToDevice(user_id: Int, title: String, message: String) {
        val notificationInfo = LoginModel(
            title = title, user_id = user_id, message = message
        )
        cartViewModel.sendNotificationToDevice(notificationInfo)
    }
}