package com.codes_roots.imakeapp.presentaion.paymentfragment.mvi


import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.repository.cart.CartDataRepo
import com.satafood.core.entities.cart.CartConfigurations
import com.satafood.core.entities.wallet.WalletModel
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first
import retrofit2.Response


/**
 * this is the model function in MVI, it's responsibility is to convert intents into views states
 */
suspend fun mapIntentToViewState(
    intent: CheckoutIntents,
    Datarepo: CartDataRepo,
    loadMainData: suspend () ->  Flow<Result<Response<CartConfigurations>>> = { Datarepo.getCartConfigurations(intent.googleModel!!) },
    checkWallet: suspend () -> Flow<Result<WalletModel>> = { Datarepo.checkWallet(intent.googleModel!!) },
) = when (intent) {
    is CheckoutIntents.LoadData -> proceedWithInitialize(loadMainData, intent)
    is CheckoutIntents.CheckWallet -> checkWalletInitialize(checkWallet, intent)
    is CheckoutIntents.DisplayErrors -> intent.viewState.copy()
}


private suspend fun proceedWithInitialize(
    loadCart: suspend () ->  Flow<Result<Response<CartConfigurations>>>,
    intent: CheckoutIntents,
): CheckOutViewState {
    val response = loadCart()
    val data = response.first()
    return runCatching {
        intent.viewState!!.copy(checkOutData = (data.getOrThrow().body()), Errors = null,
            progress = false)
    }
        .getOrElse {
            intent.viewState!!.copy(Errors = UserError.NetworkError(it.cause!!))
        }

}

private suspend fun checkWalletInitialize(
    loadCart: suspend () -> Flow<Result<WalletModel>>,
    intent: CheckoutIntents,
): CheckOutViewState {
    val response = loadCart()
    val data = response.first()
    return runCatching {
        intent.viewState!!.copy(CheckWallet = (data.getOrThrow()), Errors = null,
            progress = false)
    }
        .getOrElse {
            intent.viewState!!.copy(Errors =null)
        }

}






