package com.codes_roots.imakeapp.presentaion.profilefragment

import android.app.Activity
import android.net.Uri
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.activity.result.ActivityResult
import androidx.activity.result.contract.ActivityResultContracts
import androidx.core.view.isVisible
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.lifecycle.ViewModelProvider
import com.github.dhaval2404.imagepicker.ImagePicker
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.databinding.FragmentNewProfileBinding
import com.codes_roots.imakeapp.helper.BaseApplication
import com.codes_roots.imakeapp.helper.FileUtils
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.helper.SUCCESS_MotionToast
import com.codes_roots.imakeapp.presentaion.myaddress.viewmodel.UserAddressViewModel
import com.satafood.core.entities.login.User
import io.github.muddz.styleabletoast.StyleableToast
import okhttp3.MediaType.Companion.toMediaTypeOrNull
import okhttp3.MultipartBody
import okhttp3.RequestBody
import java.io.File
import javax.inject.Inject


class NewProfileFragment @Inject constructor() : Fragment() {


    @Inject
    lateinit var viewModelFactory: ViewModelProvider.Factory

    val viewModel by viewModels<UserAddressViewModel> { viewModelFactory }

    @Inject
    lateinit var Pref: PreferenceHelper
    var fileUri: Uri? = null

    private lateinit var view: FragmentNewProfileBinding

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        BaseApplication.appComponent.inject(this)
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?,

        ): View {

        view = DataBindingUtil.inflate(inflater, R.layout.fragment_new_profile, container, false)

        if (!viewModel.isAddressError){
            viewModel.getUsersData(Pref.UserId)
        }else{
            StyleableToast.makeText(requireContext(), "Error loading your profile, Please check your connection", Toast.LENGTH_SHORT, R.style.myInternetToast).show();
        }

        view.imageView2.setOnClickListener {
            addimage()
        }

        runCatching {
            viewModel.userLD?.observe(viewLifecycleOwner) {
                if (it != null)
                    view.data = it
                Pref.userPhoto = it?.profile?.get(0)?.photo

            }
        }.getOrElse { throw  it }
        runCatching {
            viewModel.editUserLD?.observe(viewLifecycleOwner) {
                if (it != null)
                    SUCCESS_MotionToast(
                        requireActivity().getString(R.string.confirmed),
                        requireActivity()  )
                view.progress.isVisible = false

                requireActivity().onBackPressed()
            }
        }.getOrElse { throw  it }

        view.btnSave.setOnClickListener {
            val username = view.editTextName.text.toString()
            val mobile = view.edtMobile.text.toString()
            val email = view.editTextEmail.text.toString()
            view.progress.isVisible = true
            val data = User(first_name = username, mobile = mobile, email = email)
            if (fileUri != null) {
                val photo_part = prepareFilePart("img", fileUri!!)
                viewModel.editUserData(photo_part, data)
            }else {
                viewModel.editUsersDataWithOutPhoto(data)
            }
        }

        view.ivBack.setOnClickListener { requireActivity().onBackPressed() }

        return view.root
    }

    fun addimage() {
        ImagePicker.with(this)
            .compress(1024)         //Final image size will be less than 1 MB(Optional)
            .maxResultSize(1080, 1080)
            .crop()    //Final image resolution will be less than 1080 x 1080(Optional)
            .createIntent { intent ->
                startForProfileImageResult.launch(intent)
            }
    }


    private fun prepareFilePart(name: String, fileUri: Uri): MultipartBody.Part {
        var file: File? = null
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.KITKAT) {
            file = FileUtils.getFile(requireContext(), fileUri)
        }
        val requestFile = RequestBody.create(
            "image".toMediaTypeOrNull(),
            file!!
        )
        return MultipartBody.Part.createFormData(name, file.name, requestFile)
    }

    private val startForProfileImageResult =
        registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { result: ActivityResult ->
            val resultCode = result.resultCode
            val data = result.data
            if (resultCode == Activity.RESULT_OK) {
                //Image Uri will not be null for RESULT_OK
                fileUri = data?.data!!
                //  mProfileUri = fileUri
                view.rivProfile2.setImageURI(fileUri)

            } else if (resultCode == ImagePicker.RESULT_ERROR) {
                Toast.makeText(requireContext(), ImagePicker.getError(data), Toast.LENGTH_SHORT)
                    .show()
            } else {
                Toast.makeText(requireContext(), "Task Cancelled", Toast.LENGTH_SHORT).show()
            }
        }

}