package com.codes_roots.imakeapp.presentaion.register.loginfragment

import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProvider
import com.google.android.gms.tasks.OnCompleteListener
import com.google.firebase.messaging.FirebaseMessaging
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.databinding.LoginFragmentBinding
import com.codes_roots.imakeapp.helper.BaseApplication
import com.codes_roots.imakeapp.helper.FormValidationLogin
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.presentaion.ClickHandler
import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.MainViewModel
import com.codes_roots.imakeapp.presentaion.register.RegisterActivity
import com.codes_roots.imakeapp.presentaion.register.forgetpassword.ForgetPasswordFragment
import com.codes_roots.imakeapp.presentaion.register.otp_fragment.MobileVerificationFragment
import com.codes_roots.imakeapp.presentaion.register.viewmodel.AuthViewModel
import com.satafood.core.entities.login.LoginModel
import javax.inject.Inject


class LoginFragment: Fragment() {

    @Inject
    lateinit var viewModelFactory: ViewModelProvider.Factory

    val viewModel by viewModels<AuthViewModel> {viewModelFactory}

    private val homeViewModel by viewModels<MainViewModel> { viewModelFactory }

    var firebaseToken : String ? = ""

    @Inject
    lateinit var Pref: PreferenceHelper

    lateinit var view: LoginFragmentBinding
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (savedInstanceState == null) {
            BaseApplication.appComponent.inject(this)
        }
    }
    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View{
        view = DataBindingUtil.inflate(inflater,
            R.layout.login_fragment, container, false)
        FirebaseMessaging.getInstance().token.addOnCompleteListener(OnCompleteListener { task ->
            if (!task.isSuccessful) {
                Log.d("TAG", "token:/// ${task.exception}")
                return@OnCompleteListener
            }
            // Get new FCM registration token
            firebaseToken = task.result

            // Log and toast
            Log.d("TAG", "token:///" + firebaseToken)

        })

        FormValidationLogin().passwordFocusListener(requireContext(),view)
        FormValidationLogin().nameFocusListener(requireContext(),view)

        view.login.setOnClickListener {
         FormValidationLogin().submitForm(view,context as RegisterActivity,this)

         }

        view.forgetPW.setOnClickListener {
           ClickHandler().switchToForgentPassword(requireContext(),ForgetPasswordFragment())
        }

        viewModel.authLD?.observe(requireActivity() , Observer {
            if (it.token == null){
                Toast.makeText(context , "خطأ في كلمة المرور او الاسم", Toast.LENGTH_SHORT).show()
            }else{
               Pref.userName = it.user?.username
               Pref.token= (it.token)
                Pref.email = it.user!!.email
                Pref.UserId  =  it.user?.id!!
               Pref.userPhoto  =  it.user?.photo!!
               Pref.code = it.user!!.code
               registerTokenRequest(it.user?.id!!)
             (context as RegisterActivity).setResult(10000)
             (context as RegisterActivity).finish()
            }
        })
        viewModel.errorMessage.observe(requireActivity(), Observer {
            Toast.makeText(context , "خطأ في كلمة المرور او الاسم", Toast.LENGTH_SHORT).show()
        })
        viewModel.InActiveView.observe(requireActivity(), Observer {

            if (it == 1) {
                switchToVerificationFragmentWithData()
            }
        })
   return view.root
    }
    private fun switchToVerificationFragmentWithData(){
        val fragment = MobileVerificationFragment()
        (context as RegisterActivity).supportFragmentManager.beginTransaction()
            .setCustomAnimations(R.anim.slide_top, 0, 0, R.anim.slide_down)
            .replace(R.id.main_frame, fragment).addToBackStack(null).commit()
    }
    fun registerTokenRequest(userId : Int) {
        val registerTokenInfo = LoginModel(
            token = firebaseToken , user_id = userId)
        homeViewModel.registerToken(registerTokenInfo)
    }


}