package com.codes_roots.imakeapp.presentaion.register.viewmodel

import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import com.meem.physio.util.SingleLiveEvent
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.repository.authenticate.RemoteDataSource
import com.satafood.core.entities.login.ForgetPasswordModel
import com.satafood.core.entities.login.LoginModel
import com.satafood.core.entities.login.User
import io.reactivex.disposables.CompositeDisposable
import kotlinx.coroutines.*
import org.json.JSONException
import org.json.JSONObject
import javax.inject.Inject

class AuthViewModel @Inject constructor(
    private val Datasources: RemoteDataSource,
) : ViewModel() {

    @Inject
    lateinit var Pref: PreferenceHelper
    private var job: Job? = null
    var mCompositeDisposable = CompositeDisposable()
    var processVisibility = MutableLiveData(false)

    var rateJob: Job? = null
    var authLD: MutableLiveData<LoginModel>? = null
    var forgetPasswordLD: SingleLiveEvent<ForgetPasswordModel>? = null
    var otpLD: MutableLiveData<LoginModel>? = null

    var data: User? = null

    var errorMessage = MutableLiveData<String>()
    var InActiveView = MutableLiveData<Int>()

    val loading = MutableLiveData<Boolean>()

    val coroutineExceptionHandler = CoroutineExceptionHandler { _, throwable ->
        throwable.printStackTrace()
    }

    init {

        errorMessage = MutableLiveData()
        InActiveView = MutableLiveData()
        otpLD = MutableLiveData()
        authLD = MutableLiveData()
        forgetPasswordLD = SingleLiveEvent()
    }


    //authentication
    fun facebookLogin(facebookModel: User?) {
        job = CoroutineScope(Dispatchers.IO).launch {
            val response = Datasources.facebookResponse(facebookModel!!)
            withContext(Dispatchers.Main) {
                if (response.isSuccessful) {
                    authLD?.postValue(response.body())

                } else {
                    onError("Error : ${response.message()} ")
                }
            }
        }

    }

    //authentication
    fun login(loginModel: User?) {
        job = CoroutineScope(Dispatchers.IO + coroutineExceptionHandler).launch {
            val response = Datasources.getLoginResponse(loginModel!!)
            withContext(Dispatchers.Main + coroutineExceptionHandler) {
                if (response.isSuccessful) {
                    authLD?.postValue(response.body())

                } else {
                    if (response.code() == 405) {
                        var jsonObject: JSONObject? = null
                        try {
                            jsonObject = JSONObject(response.errorBody()!!.string())
                            val mobile: String = jsonObject.getString("mobile")
                            var email : String = jsonObject.getString("email")
                            var token : String = jsonObject.getString("token")

                            Pref.userPhone = mobile
                            Pref.email = email
                            Pref.token = token

                            InActiveView.postValue(1)
                          } catch (e: JSONException) {
                        e.printStackTrace()
                    }
                        }else {
                            onError("Error : ${response.message()} ")

                        }

                }
            }
        }

    }
    fun forgetPassword(loginModel: ForgetPasswordModel?) {
        job = CoroutineScope(Dispatchers.IO + coroutineExceptionHandler).launch {
            val response = Datasources.forgetPasswordResponse(loginModel!!)
            withContext(Dispatchers.Main + coroutineExceptionHandler) {
                if (response.isSuccessful) {
                    forgetPasswordLD?.postValue(response.body())

                } else {
                    onError("Error : ${response.message()} ")
                }
            }
        }

    }

    fun activeUserAccount(userId: Int, loginModel: LoginModel) {
        job = CoroutineScope(Dispatchers.IO + coroutineExceptionHandler).launch {
            val response = Datasources.activeUserAccount(userId, loginModel)
            withContext(Dispatchers.Main + coroutineExceptionHandler) {
                if (response.isSuccessful) {
                    authLD?.postValue(response.body())

                } else {
                    onError("Error : ${response.message()} ")
                }
            }
        }

    }

    //getRegisterResponse
    fun register(registerModel: User?) {
        job = CoroutineScope(Dispatchers.IO + coroutineExceptionHandler).launch {
            val response = Datasources.getRegisterResponse(registerModel!!)
            withContext(Dispatchers.Main + coroutineExceptionHandler) {
                if (response.isSuccessful) {
                    authLD?.postValue(response.body())

                } else {
                    if (response.code() == 400) {
                        var jsonObject: JSONObject? = null
                        try {
                            jsonObject = JSONObject(response.errorBody()!!.string())
                            val userMessage: JSONObject = jsonObject.getJSONObject("data")
                                if (userMessage.has("username")) {
                                   var usernameValidation =  userMessage.getJSONArray("username")
                                    errorMessage.postValue(usernameValidation.getJSONObject(0).getString("message"))


                                }
                            if (userMessage.has("email")) {
                                var emailValidation =  userMessage.getJSONArray("email")
                                errorMessage.postValue(emailValidation.getJSONObject(0).getString("message"))


                            }
                            if (userMessage.has("mobile")) {
                                var mobileValidation =  userMessage.getJSONArray("mobile")
                                errorMessage.postValue(mobileValidation.getJSONObject(0).getString("message"))
                            }

                        } catch (e: JSONException) {
                            e.printStackTrace()
                        }
                    }


                }
            }
        }

    }//getVerifyEmailResponse



    fun checkAgainVerifyEmail(registerModel: User?) {
        job = CoroutineScope(Dispatchers.IO + coroutineExceptionHandler).launch {
            val response = Datasources.checkAgainVerifyEmail(registerModel!!)
            withContext(Dispatchers.Main + coroutineExceptionHandler) {
                if (response.isSuccessful) {
                    otpLD?.postValue(response.body())

                } else {
                    errorMessage.postValue("user Not Found or code incorrect")

                  //  onError("Error : ${response.message()} ")
                }
            }
        }

    }

    fun checkVerifyEmail(registerModel: LoginModel?) {
        job = CoroutineScope(Dispatchers.IO + coroutineExceptionHandler).launch {
            val response = Datasources.checkVerifyEmail(registerModel!!)
            withContext(Dispatchers.Main + coroutineExceptionHandler) {
                if (response.isSuccessful) {
                    otpLD?.postValue(response.body())

                } else {
                    errorMessage.postValue("user Not Found or code incorrect")
//                    onError("Error : ${response.body()!!.message} ")
                }
            }
        }

    }


    private fun onError(message: String) {
        errorMessage.value = message
        loading.value = false
    }

    override fun onCleared() {
        super.onCleared()
        mCompositeDisposable.dispose()
        mCompositeDisposable.clear()
        rateJob?.cancel()
    }
}