package com.codes_roots.imakeapp.presentaion.reviews_fragment

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.view.isVisible
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.databinding.ReviewsVendorFragmentBinding
import com.codes_roots.imakeapp.helper.BaseApplication
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.presentaion.reviews_fragment.mvi.MainIntent
import com.codes_roots.imakeapp.presentaion.reviews_fragment.mvi.ReviewsViewModel
import com.codes_roots.imakeapp.presentaion.vendordetailsfragment.viewmodel.RestaurantDetailsViewModel
import kotlinx.coroutines.flow.collect
import java.math.RoundingMode
import java.text.DecimalFormat
import java.util.*
import javax.inject.Inject

class ReviewsVendorFragment @Inject constructor
    (var restaurantViewModel: RestaurantDetailsViewModel) : Fragment() {


    @Inject
    lateinit var viewModelFactory: ViewModelProvider.Factory

    val viewModel by viewModels<ReviewsViewModel> { viewModelFactory }

    @Inject
    lateinit var Pref: PreferenceHelper

    lateinit var reviewsAdapter: ReviewsAdapter

    private lateinit var view: ReviewsVendorFragmentBinding

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (savedInstanceState == null) {
            BaseApplication.appComponent.inject(this)
        }

    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?,
    ): View? {

        view = DataBindingUtil.inflate(inflater, R.layout.reviews_vendor_fragment, container, false)
        viewModel.intents.trySend(
            MainIntent.LoadData(viewModel.state.value!!.copy(progress = true, data = null),restaurantViewModel.data?.id))

        view.restaurantData = restaurantViewModel.data

        view.description.apply {
            isSelected = true
            text = ""
            if (restaurantViewModel.data?.cuisines?.isNotEmpty() == true)
                restaurantViewModel.data!!.cuisines?.map { cuisine ->
                    view.description.append(cuisine.name + " - ")
                }
            else
                text = restaurantViewModel.data?.restaurantName
        }

        view.backArrow.setOnClickListener {
            requireActivity().onBackPressed()
        }


        getData()
        setupPaymentWaysRecyclerView()


        return view.root
    }

    private fun getData() {
        lifecycleScope.launchWhenStarted {
            viewModel.state.collect {  it ->
                if (it != null) {
                    if (it.Errors != null) {
                        it.Errors.let {
                            when (it) {
                                is UserError.InvalidId -> "Invalid id"
                                is UserError.NetworkError -> it.throwable.message
                                UserError.ServerError -> "Server error"
                                UserError.Unexpected -> "Unexpected error"
                                is UserError.UserNotFound -> "User not found"
                                UserError.ValidationFailed -> "Validation failed"
                                else -> {}
                            }
                        }
                    } else {
                        if (!it.data?.rates.isNullOrEmpty()) {
                            view.data = it.data
                            view.noReviews.isVisible = false
                            reviewsAdapter.submitList(it.data?.rates!!)
                            view.noReviews.isVisible = it.data!!.rates.isNullOrEmpty()
//                                setupPaymentWaysRecyclerView()

                            view.numberOfRates.text  =roundOffDecimal(it.data!!.avg.TotalRate).toString().plus( " " + context?.getString(R.string.ratee))


                        }else{
                            view.noReviews.isVisible = true
                            view.animationView.playAnimation()
                        }
                    }

                }
            }

        }

    }

    private fun roundOffDecimal(number: Double): Double{
        val rounded = number.toBigDecimal().setScale(1, RoundingMode.UP).toDouble()
        return rounded
    }

    private fun setupPaymentWaysRecyclerView()  {
        reviewsAdapter = ReviewsAdapter()
        view.rateRecycleView.apply {
            adapter = reviewsAdapter
            setHasFixedSize(true)
        }

    }

}






