package com.codes_roots.imakeapp.presentaion.reviews_fragment.mvi


import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.repository.retaurantdetails.DataRepo
import com.satafood.core.entities.reviews.ReviewsModel
import com.satafood.core.entities.search_model.SearchModel
import com.satafood.core.mapper.search_model_mapper.SearchModelMapper
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first

/**
 * this is the model function in MVI, it's responsibility is to convert intents into views states
 */
suspend fun mapIntentToViewState(
    intent: MainIntent,
    Datarepo: DataRepo,
    loadMainData: suspend () -> Flow<Result<ReviewsModel>> = { Datarepo.getRestaurantReviews(intent.restauratnId!!) },

    ) = when (intent) {
    is MainIntent.LoadData -> proceedWithInitialize(loadMainData, intent)
    is MainIntent.DisplayErrors -> intent.viewState.copy()
}

private suspend fun proceedWithInitialize(
    loadSearchData: suspend () -> Flow<Result<ReviewsModel>>,

    intent: MainIntent,
): MainViewState {
    val response = loadSearchData()
    val data = response.first()
    return runCatching {
        intent.viewState!!.copy(data = data.getOrThrow(),
            Errors = null,
            progress = false)
    }
        .getOrElse {
            intent.viewState!!.copy(Errors = UserError.NetworkError(it))
        }
}






