package com.codes_roots.imakeapp.presentaion.search_fragment

import android.annotation.SuppressLint
import android.app.Activity
import android.content.ActivityNotFoundException
import android.content.Context.INPUT_METHOD_SERVICE
import android.content.Intent
import android.os.Bundle
import android.speech.RecognizerIntent
import android.util.Log
import android.view.KeyEvent
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.InputMethodManager
import android.widget.Toast
import androidx.core.view.isVisible
import androidx.core.widget.doOnTextChanged
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.databinding.SearchFragmentBinding
import com.codes_roots.imakeapp.helper.BaseApplication
import com.codes_roots.imakeapp.helper.Error_MotionToast
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.presentaion.ClickHandler
import com.codes_roots.imakeapp.presentaion.homefragment.HomeViewModel
import com.codes_roots.imakeapp.presentaion.homefragment.adapters.Best_Item_Adapter
import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.presentaion.mainactivity.MainActivity
import com.codes_roots.imakeapp.presentaion.mywallet.mvi.HomeIntent
import com.codes_roots.imakeapp.presentaion.search_fragment.adapter.Items_Adapter
import com.codes_roots.imakeapp.presentaion.search_fragment.adapter.StoresAdapter
import com.codes_roots.imakeapp.presentaion.search_fragment.mvi.MainIntent
import com.codes_roots.imakeapp.presentaion.search_fragment.mvi.SearchViewModel
import com.codes_roots.imakeapp.presentaion.vendordetailsfragment.viewmodel.RestaurantDetailsViewModel
import com.satafood.core.base.DataState
import com.satafood.core.entities.google.GoogleModel
import com.satafood.core.mapper.search_model_mapper.ItemResultMapper
import com.satafood.core.mapper.search_model_mapper.StoresResultMapper
import kotlinx.android.synthetic.main.fragment_point.view.*

import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.launch

import java.util.*
import javax.inject.Inject


class SearchFragment @Inject constructor() : Fragment() {

    @Inject
    lateinit var viewModelFactory: ViewModelProvider.Factory

    val viewModel by viewModels<SearchViewModel> { viewModelFactory }

    @Inject
    lateinit var Pref: PreferenceHelper

    private val REQUEST_CODE_STT = 102
    var lastText = ""
    var storesModel: ArrayList<StoresResultMapper>? = null
    var itemsModel: ArrayList<ItemResultMapper>? = null
    private lateinit var view: SearchFragmentBinding

    lateinit var itemsAdapter: Items_Adapter

    lateinit var storesAdapter: StoresAdapter
    private val viewModelRestaurants by viewModels<RestaurantDetailsViewModel> { viewModelFactory }

    private val homeViewModel by viewModels<HomeViewModel> { viewModelFactory }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        if (savedInstanceState == null) {
            BaseApplication.appComponent.inject(this)
        }
    }

    fun itemsRecycleView() {
        itemsAdapter = Items_Adapter(requireContext()) {
            lifecycleScope.launch {
                it.id?.let { id ->
                    homeViewModel.intentChannel.send(HomeIntent.GetItemDetails(id))
                }
            }
        }
        view.itemsRecycleView.apply {
            adapter = itemsAdapter;
            isNestedScrollingEnabled = false
        }
    }

    fun storesRecycleView() {
        storesAdapter = StoresAdapter(requireContext())
        view.storesRecycleView.apply {
            adapter = storesAdapter;

        }
    }


    @SuppressLint("ClickableViewAccessibility")
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?,
    ): View {


        view = DataBindingUtil.inflate(inflater, R.layout.search_fragment, container, false)

        view.backBtn.setOnClickListener {
            requireActivity().onBackPressed()
        }
        view.listener = ClickHandler()
        view.context = context as MainActivity
        view.viewModel = viewModel

        itemsRecycleView()

        storesRecycleView()


        //  view.searchBar.setError("assad")
        view.searchIc.setOnClickListener {
            if (lastText != view.searchBar.text.toString()) {
                if (view.searchBar.text!!.isNotEmpty()) {
                    startSearchView()
                    clearData()
                    val googleInfo =
                        GoogleModel(
                            google_id = Pref.googleID,
                            name = view.searchBar.text.toString()
                        )
                    viewModel.intents.trySend(
                        MainIntent.LoadData(
                            viewModel.state.value!!.copy(
                                progress = true,
                                data = null
                            ), googleInfo
                        )
                    )

                    hideSoftKeyboard()

                }
                lastText = view.searchBar.text.toString()
            }
        }
        view.searchBar.doOnTextChanged { text, start, before, count ->
            if (view.searchBar.text!!.length >= 3) {
                startSearchView()
                clearData()
                val googleInfo =
                    GoogleModel(google_id = Pref.googleID, name = view.searchBar.text.toString())
                viewModel.intents.trySend(
                    MainIntent.LoadData(
                        viewModel.state.value!!.copy(progress = true, data = null),
                        googleInfo
                    )
                )

            }
        }

        // edit text enter key listener
        view.searchBar.setOnKeyListener(object : View.OnKeyListener {
            override fun onKey(v: View?, keyCode: Int, event: KeyEvent): Boolean {
                // if the event is a key down event on the enter button
                if (event.action == KeyEvent.ACTION_DOWN &&
                    keyCode == KeyEvent.KEYCODE_ENTER
                ) {
                    if (view.searchBar.text!!.isNotEmpty()) {
                        if (lastText != view.searchBar.text.toString()) {
                            // hide soft keyboard programmatically
                            clearData()
                            hideSoftKeyboard()
                            startSearchView()
                            val googleInfo =
                                GoogleModel(
                                    google_id = Pref.googleID,
                                    name = view.searchBar.text.toString()
                                )
                            viewModel.intents.trySend(
                                MainIntent.LoadData(
                                    viewModel.state.value!!.copy(progress = true),
                                    googleInfo
                                )
                            )
                            // clear focus and hide cursor from edit text
                            view.searchBar.clearFocus()

                            view.searchBar.isCursorVisible = false

                        }
                    }
                    lastText = view.searchBar.text.toString()
                    return true

                }

                return false
            }

        })

        view.microphone.setOnClickListener {
            speak()
        }

        getData()

        return view.root

    }

    private fun speak() {
        val sttIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
        sttIntent.putExtra(
            RecognizerIntent.EXTRA_LANGUAGE_MODEL,
            RecognizerIntent.LANGUAGE_MODEL_FREE_FORM
        )
        sttIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, (context as MainActivity).Pref.lang)
        sttIntent.putExtra(RecognizerIntent.EXTRA_PROMPT, "Speak now!")
        try {
            activity?.startActivityForResult(sttIntent, REQUEST_CODE_STT)
        } catch (e: ActivityNotFoundException) {
            e.printStackTrace()
            Toast.makeText(
                requireContext(),
                "Your device does not support STT.",
                Toast.LENGTH_LONG
            ).show()
        }
    }

    private fun clearData() {
        if (itemsModel != null) {
            itemsModel!!.clear()
            view.itemsRecycleView.adapter!!.notifyDataSetChanged()
        }

        if (storesModel != null) {
            storesModel!!.clear()
            view.storesRecycleView.adapter!!.notifyDataSetChanged()
        }

    }

    private fun getData() {
        lifecycleScope.launchWhenStarted {
            viewModel.state.collect { it ->
                if (it != null) {
                    try {
                        if (it.Errors != null) {
                            it.Errors.let {
                                when (it) {
                                    is UserError.InvalidId -> "Invalid id"
                                    is UserError.NetworkError -> it.throwable.message
                                    UserError.ServerError -> "Server error"
                                    UserError.Unexpected -> "Unexpected error"
                                    is UserError.UserNotFound -> "User not found"
                                    UserError.ValidationFailed -> "Validation failed"
                                    else -> {}
                                }
                            }
                            viewModel.intents.send(MainIntent.DisplayErrors(it))
                        } else {
                            if (it.data != null) {
                                if (it.data!!.storesModel.isNullOrEmpty() && it.data!!.itemsModel.isNullOrEmpty()) {
                                    view.noOrderFoundLayout.isVisible = true
                                } else {
                                    if (!it.data!!.storesModel.isNullOrEmpty()) {
                                        view.storesRecycleView.isVisible = true

                                        storesModel =
                                            it.data!!.storesModel as ArrayList<StoresResultMapper>?
                                        storesAdapter.submitList(storesModel)
                                        view.storesTitle.isVisible = true
                                        view.noOrderFoundLayout.isVisible = false
                                    } else {
                                        view.storesRecycleView.isVisible = false
                                        view.storesTitle.isVisible = false
                                    }
                                    if (!it.data!!.itemsModel.isNullOrEmpty()) {
                                        view.itemsRecycleView.isVisible = true

                                        itemsModel =
                                            it.data!!.itemsModel as ArrayList<ItemResultMapper>?
                                        itemsAdapter.submitList(itemsModel)
                                        view.noOrderFoundLayout.isVisible = false
                                        view.itemsTitle.isVisible = true

                                    } else {
                                        view.itemsRecycleView.isVisible = false
                                        view.itemsTitle.isVisible = false
                                    }

                                }
                            } else {
                                view.noOrderFoundLayout.isVisible = false


                            }
                        }
                        stopSearchView()
                        lifecycleScope.launchWhenCreated {
                            homeViewModel.itemDetails.observe(viewLifecycleOwner) {
                                when (it!!.status) {
                                    DataState.Status.LOADING -> {
//                                        view.progress.isVisible = true
                                    }

                                    DataState.Status.SUCCESS -> {
//                                        view.progress.isVisible = false
                                        if (!it.data.isNullOrEmpty())
                                            it.data?.first()?.let { it1 ->
                                                ClickHandler().openItemDetailsFragment(
                                                    requireContext(),
                                                    it1,
                                                    it1.menu_categories!!.vendor_id!!, viewModelRestaurants, 1
                                                )
                                            }
                                        else
                                            Error_MotionToast(
                                                "no data for this item",
                                                requireActivity()
                                            )
                                    }
                                    DataState.Status.ERROR -> {
                                        Error_MotionToast(it.error?.throwableMessage.toString(), requireActivity())
                                    }
                                }
                            }
                        }


                    } catch (e: Exception) {

                        Error_MotionToast("", requireActivity())
                    }
                }

            }

        }
        stopSearchView()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        when (requestCode) {
            REQUEST_CODE_STT -> {
                if (resultCode == Activity.RESULT_OK && data != null) {
                    val result = data.getStringArrayListExtra(RecognizerIntent.EXTRA_RESULTS)
                    result?.let {
                        val recognizedText = it[0]
                        view.searchBar.setText(recognizedText)
                        startSearchView()
                        clearData()
                        val googleInfo =
                            GoogleModel(google_id = Pref.googleID, name = recognizedText)
                        viewModel.intents.trySend(
                            MainIntent.LoadData(
                                viewModel.state.value!!.copy(progress = true),
                                googleInfo
                            )
                        )


                    }
                }
            }

        }
    }

    fun startSearchView() {
        view.noOrderFoundLayout.isVisible = false
        view.progressBar.isVisible = true
        view.shimmerView.startShimmerAnimation()
        view.shimmerView.isVisible = true
    }

    fun stopSearchView() {
        view.progressBar.isVisible = false
        view.shimmerView.stopShimmerAnimation()
        view.shimmerView.isVisible = false
    }

    // extension function to hide soft keyboard programmatically
    fun hideSoftKeyboard() {
        (requireActivity().getSystemService(INPUT_METHOD_SERVICE) as InputMethodManager).apply {
            hideSoftInputFromWindow(requireActivity().currentFocus?.windowToken, 0)
        }
    }

}