package com.codes_roots.imakeapp.presentaion.splash


import android.content.Intent
import android.graphics.Color
import android.net.Uri
import android.os.Bundle
import android.util.Log
import androidx.activity.viewModels
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.app.AppCompatDelegate
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import com.codes_roots.imakeapp.databinding.ActivityCoreSplashBinding
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.helper.WARN_MotionToast
import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.presentaion.ThemeColors
import com.codes_roots.imakeapp.presentaion.splash.mvi.MainIntent
import com.codes_roots.imakeapp.presentaion.splash.mvi.SplashViewModel
import dagger.android.AndroidInjection
import dagger.android.AndroidInjector
import dagger.android.DispatchingAndroidInjector
import dagger.android.HasAndroidInjector
import kotlinx.coroutines.flow.collect
import javax.inject.Inject


class CoreSplashScreen : AppCompatActivity(), HasAndroidInjector {

    private val SPLASH_DISPLAY_LENGTH = 10000 //splash screen will be shown for 2 seconds

    @Inject
    lateinit var Pref: PreferenceHelper

    @Inject
    lateinit var viewModelFactory: ViewModelProvider.Factory

    val viewModel by viewModels<SplashViewModel> { viewModelFactory }
    var binding: ActivityCoreSplashBinding? = null
    private var uri: Uri? = null

    public override fun onCreate(icicle: Bundle?) {
        AndroidInjection.inject(this)
        super.onCreate(icicle)
        binding = ActivityCoreSplashBinding.inflate(layoutInflater)
        setContentView(binding!!.root)
        ThemeColors(this)
        uri = getIntent().getData()
        AppCompatDelegate.setDefaultNightMode(AppCompatDelegate.MODE_NIGHT_NO);
        viewModel.intents.trySend(MainIntent.LoadData(viewModel.state.value!!.copy()))
        getData()
    }

    @Inject
    lateinit var androidInjector: DispatchingAndroidInjector<Any>
    override fun androidInjector(): AndroidInjector<Any> {
        return androidInjector
    }


    private fun getRgbFromHex(hex: String) {
        val initColor = Color.parseColor(hex)
        val r = Color.red(initColor)
        val g = Color.green(initColor)
        val b = Color.blue(initColor)
        buttonClick(r, g, b)
    }

    fun buttonClick(red: Int, green: Int, blue: Int) {
        ThemeColors.setNewThemeColor(this@CoreSplashScreen, red, green, blue)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == 1) {
            if (resultCode != RESULT_OK) {
                Log.e("MY_APP", "Update flow failed! Result code: $resultCode")
                WARN_MotionToast("canceled", this)
                // If the update is cancelled or fails,
                // you can request to start the update again.
            }
        }
    }

    private fun getData() {
        lifecycleScope.launchWhenStarted {
            viewModel.state.collect { it ->
                if (it != null) {
                    if (it.Errors != null) {
                        it.Errors.let {
                            when (it) {
                                is UserError.InvalidId -> "Invalid id"
                                is UserError.NetworkError -> it.throwable.message
                                UserError.ServerError -> "Server error"
                                UserError.Unexpected -> "Unexpected error"
                                is UserError.UserNotFound -> "User not found"
                                UserError.ValidationFailed -> "Validation failed"
                                else -> {}
                            }
                        }
                    } else {
                        if (!it.data.isNullOrEmpty()) {
                        it.data?.map {
                            when(it.type) {
                               "logo" -> {
                                  Pref.appLogo = it.value
                               }
                                "secondary_color" -> {
                                    val secondaryColor = it.value!!
                                    if (secondaryColor.contains("#")) {
                                        Pref.secondaryColor = secondaryColor
                                    } else {
                                        Pref.secondaryColor = "#".plus(secondaryColor)
                                    }
                                }
                                "image header" -> {
                                    Pref.appHeader = it.value
                                }
                                "primary_color" -> {
                                    val primaryColor = it.value!!
                                    if (primaryColor.contains("#")) {
                                        getRgbFromHex(primaryColor)
                                        Pref.appPrimaryColor = primaryColor
                                    } else {
                                        getRgbFromHex(("#".plus(primaryColor)))
                                        Pref.appPrimaryColor = "#".plus(primaryColor)
                                    }
                                }
                                "currency" -> {
                                    Pref.currency = it.value

                                }
                                "text_color" -> {}
                                "font_family_1" -> {}

                            }

                        }

                        }
                    }

                }
            }

        }

    }
}