package com.codes_roots.imakeapp.presentaion.splash


import android.content.Intent
import android.graphics.Color
import android.graphics.PorterDuff
import android.os.Bundle
import android.os.Handler
import androidx.appcompat.app.AppCompatActivity
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.presentaion.mainactivity.MainActivity
import com.codes_roots.imakeapp.presentaion.map_activity.MapActivity
import dagger.android.AndroidInjection
import dagger.android.AndroidInjector
import dagger.android.DispatchingAndroidInjector
import dagger.android.HasAndroidInjector
import javax.inject.Inject

import android.net.ConnectivityManager
import android.util.Log
import androidx.appcompat.app.AppCompatDelegate
import com.codes_roots.imakeapp.helper.ResourceUtil
import com.codes_roots.imakeapp.helper.WARN_MotionToast
import com.codes_roots.imakeapp.helper.setImageResource
import com.codes_roots.imakeapp.presentaion.ClickHandler
import com.codes_roots.imakeapp.presentaion.choose_language.LanguageActivity
import com.codes_roots.imakeapp.presentaion.ThemeColors
import kotlinx.android.synthetic.main.activity_splash.*


class SplashScreen : AppCompatActivity(), HasAndroidInjector {

    private val SPLASH_DISPLAY_LENGTH = 2000 //splash screen will be shown for 2 seconds

    @Inject
    lateinit var Pref: PreferenceHelper

    public override fun onCreate(icicle: Bundle?) {
        AndroidInjection.inject(this)
        super.onCreate(icicle)
        setContentView(R.layout.activity_splash)
        AppCompatDelegate.setDefaultNightMode(AppCompatDelegate.MODE_NIGHT_NO);
        ThemeColors(this)
        setImageResource(logo, Pref.appLogo)
        background.background.setColorFilter(
            Color.parseColor(Pref.appPrimaryColor),
            PorterDuff.Mode.SRC_ATOP
        )
        ResourceUtil().changeLang(Pref.lang!!, this)
        Handler().postDelayed({
            if (!Pref.langChosen) {
                ClickHandler().switchToActivity(this, LanguageActivity())
            } else {
                if (Pref.userLocation != "") {
                    val mainIntent = Intent(this, MainActivity::class.java)
                    startActivity(mainIntent)
                    finish()
                } else {
                    val mainIntent = Intent(this, MapActivity::class.java)
                    startActivity(mainIntent)
                    finish()
                }
            }
        }, SPLASH_DISPLAY_LENGTH.toLong())
    }
//        else
//            Error_MotionToast(this.getString(R.string.text_no_internet), this)
//
//    }

    @Inject
    lateinit var androidInjector: DispatchingAndroidInjector<Any>
    override fun androidInjector(): AndroidInjector<Any> {
        return androidInjector
    }

    private fun isNetworkConnected(): Boolean {
        val cm = this.getSystemService(CONNECTIVITY_SERVICE) as ConnectivityManager
        val activeNetwork = cm.activeNetworkInfo
        return activeNetwork != null && activeNetwork.isConnectedOrConnecting
    }


    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == 1) {
            if (resultCode != RESULT_OK) {
                Log.e("MY_APP", "Update flow failed! Result code: $resultCode")
                WARN_MotionToast("canceled", this)
                // If the update is cancelled or fails,
                // you can request to start the update again.
            }
        }
    }


}