package com.codes_roots.imakeapp.presentaion.splash.mvi

import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.repository.splash.SplashDataRepo
import com.satafood.core.entities.vendor_attributes.VendorAttributesModelItem
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first

/**
 * this is the model function in MVI, it's responsibility is to convert intents into views states
 */
suspend fun mapIntentToViewState(
    intent: MainIntent,
    Datarepo: SplashDataRepo,
    loadMainData: suspend () -> Flow<Result<ArrayList<VendorAttributesModelItem>>> = { Datarepo.getVendorAttributes(
      ) },

    ) = when (intent) {
    is MainIntent.LoadData -> proceedWithInitialize(loadMainData, intent)
    is MainIntent.DisplayErrors -> intent.viewState.copy()
}

private suspend fun proceedWithInitialize(
    loadSearchData: suspend () -> Flow<Result<ArrayList<VendorAttributesModelItem>>>,

    intent: MainIntent,
): MainViewState {
    val response = loadSearchData()
    val data = response.first()
    return runCatching {
        intent.viewState!!.copy(data = data.getOrThrow(),
            Errors = null,
            progress = false)
    }
        .getOrElse {
            intent.viewState!!.copy(Errors = UserError.NetworkError(it))
        }
}






