package com.codes_roots.imakeapp.presentaion.stores_fragment.mvi

import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.repository.RestaurantsList.StoresListDataRepo
import com.satafood.core.entities.homepage.Cuisines
import com.satafood.core.entities.homepage.sliders.SliderHomePageModelItem
import com.satafood.core.entities.restaurantslist.BranchesModelListItem
import com.satafood.core.mapper.restaurantslist.*
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first

suspend fun mapResIntentToViewState(
    intent: RestaurantsListIntent,
    Datarepo: StoresListDataRepo,
    loadMainData: suspend () -> Flow<Result<List<Cuisines>>> = { Datarepo.getCuisinesList(intent.catid!!) },
    LoadBranchesData: suspend () -> Flow<Result<List<BranchesModelListItem>>> = {
        Datarepo.getBranchesList(intent.AreaId!!,
            intent.catid!!)
    },
    LoadBranchesByGoogleID: suspend () -> Flow<Result<List<BranchesModelListItem>>> = {
        Datarepo.getBranchesByGoogle(intent.googleModel!!,intent.page ?: 0)
    },
    getBranchesByAreasByCat: suspend () -> Flow<Result<List<BranchesModelListItem>>> = {
        Datarepo.getBranchesByAreasByCat(intent.googleModel!!,intent.page ?: 0)
    },
    LoadBranchesSliders: suspend () -> Flow<Result<List<SliderHomePageModelItem>>> = {
        Datarepo.getCategoriesSliders(intent.sliderModel!!)
    },
    LoadVendorOffersData: suspend () -> Flow<Result<List<BranchesModelListItem>>> = {
        Datarepo.getVendorOffersData(intent.googleModel!!)
    },
    AddFavorite: suspend () ->  Flow<Result<Boolean>> = { Datarepo.AddFavorite(intent.restauratnId!!, intent.AreaId!!) },

    LoadNearestBranchesData: suspend () -> Flow<Result<List<BranchesModelListItem>>> = {
        Datarepo.getNearestBranchesData(intent.googleModel!!)
    },
    LoadNewOfferData: suspend () -> Flow<Result<List<BranchesModelListItem>>> = {
        Datarepo.getNewOfferData(intent.googleModel!!)
    },
    LoadFreeDliveryBranchesData: suspend () -> Flow<Result<List<BranchesModelListItem>>> = {
        Datarepo.getFreeDliveryBranches(intent.googleModel!!)
    },
    LoadMostOrderedBranchesData: suspend () -> Flow<Result<List<BranchesModelListItem>>> = {
        Datarepo.getMostOrderedBranches(intent.googleModel!!)
    },

    ) = when (intent) {
    is RestaurantsListIntent.LoadData -> getDataByGoogleID(LoadBranchesData, intent)

    is RestaurantsListIntent.DisplayErrors -> intent.viewState.copy(Errors = null)

    is RestaurantsListIntent.FilterBranchesData -> FilterData(intent, intent.CuisinesId!!)
    is RestaurantsListIntent.SortByName -> sortByName(intent)
    is RestaurantsListIntent.SortByDeliveryCost -> sortBranchesByDeliveryCost(intent)
    is RestaurantsListIntent.SortByDeliveryRating -> sortBranchesByRate(intent);
    is RestaurantsListIntent.SortByDeliveryTime -> sortBranchesByDeliveryTime(intent)
// is RestaurantsListIntent.SearchByName -> SearchBranchesByName(intent, intent.RestaurantName!!)


    is RestaurantsListIntent.FilterByTopRated -> filterTopRatedVendors(intent)
    is RestaurantsListIntent.FilterOpenedVendors -> filterOpenedVendors(intent)
    is RestaurantsListIntent.FilterFreeDeliveryVendors -> filterFreeDeliveryVendors(intent)

    is RestaurantsListIntent.NoRestaurant -> intent.viewState.copy(noRestaurantFound = true)


    is RestaurantsListIntent.LoadingPaging -> getDataByGoogleID(LoadBranchesByGoogleID, intent)

    is RestaurantsListIntent.LoadDataByGoogleID -> getDataByGoogleID(LoadBranchesByGoogleID, intent)

    is RestaurantsListIntent.LoadDataByCatId -> getDataByCatID(getBranchesByAreasByCat, intent)

    is RestaurantsListIntent.LoadingCuisines -> getCuisines(loadMainData, intent)

    is RestaurantsListIntent.LoadingSliderData -> getSliderData(LoadBranchesSliders, intent)

    is RestaurantsListIntent.LoadVendorOffersData -> getVendorOffersData(LoadVendorOffersData,
        intent)
    is RestaurantsListIntent.LoadNearestBranchesData -> getNearestBranchesData(
        LoadNearestBranchesData,
        intent)
    is RestaurantsListIntent.LoadNewOfferData -> getNewOfferData(LoadNewOfferData, intent)
    is RestaurantsListIntent.LoadFreeDliveryBranchesData -> getFreeDeliveryBranchesData(
        LoadFreeDliveryBranchesData,
        intent)
    is RestaurantsListIntent.LoadMostOrderedBranchData -> getVendorOffersData(
        LoadMostOrderedBranchesData,
        intent)
    is RestaurantsListIntent.AddFavorite -> proceedAddFavorite(AddFavorite, intent)
    is RestaurantsListIntent.GetSliderData -> getSliderData(LoadBranchesSliders,intent)
}

private suspend fun proceedAddFavorite(
    AddFavorite: suspend () -> Flow<Result<Boolean>>, intent: RestaurantsListIntent,
): RestaurantsListViewStateIntent {
    return runCatching {
        var response = AddFavorite().first()
        return intent.viewState!!.copy(IsFave = response.getOrThrow())
    }.getOrElse {
        intent.viewState!!.copy(Errors = UserError.NetworkError(it))

    }
}


private suspend fun getDataByGoogleID(
    loadBranchesDataByGoogleID: suspend () -> Flow<Result<List<BranchesModelListItem>>>,
    intent: RestaurantsListIntent,
): RestaurantsListViewStateIntent {
    val BranchesData = loadBranchesDataByGoogleID().first()
    return runCatching {
        intent.viewState!!.copy(BranchesData = brancheslistusecase(BranchesData.getOrThrow()),
            FilterdBranchesData = null,
            Errors = null,
            Progress = false,
            isFilter = false,
            noRestaurantFound = false)
    }
        .getOrElse {
            intent.viewState!!.copy(Errors = UserError.NetworkError(it))
        }
}
private suspend fun getDataByCatID(
    loadBranchesDataByGoogleID: suspend () -> Flow<Result<List<BranchesModelListItem>>>,
    intent: RestaurantsListIntent,
): RestaurantsListViewStateIntent {
    val BranchesData = loadBranchesDataByGoogleID().first()
    return runCatching {
        intent.viewState!!.copy(BranchesData = null,
            FilterdBranchesData = brancheslistusecase(BranchesData.getOrThrow()),
            Errors = null,
            isFilter = true,
            Progress = false,
            noRestaurantFound = false)
    }
        .getOrElse {
            intent.viewState!!.copy(Errors = UserError.NetworkError(it))
        }
}

private suspend fun getCuisines(
    loadCart: suspend () -> Flow<Result<List<Cuisines>>>,
    intent: RestaurantsListIntent,
): RestaurantsListViewStateIntent {

    val CuisinesData = loadCart().first()
    return runCatching {
        intent.viewState!!.copy(
            cuisinesData = cuisinesUseCase(CuisinesData.getOrThrow()),
            Errors = null,
            Progress = false,
            noRestaurantFound = false)
    }
        .getOrElse {
            intent.viewState!!.copy(Errors = UserError.NetworkError(it))
        }

}


private suspend fun getSliderData(
    loadSliderPage: suspend () -> Flow<Result<List<SliderHomePageModelItem>>>,
    intent: RestaurantsListIntent,
): RestaurantsListViewStateIntent {

    val loadSliderPage = loadSliderPage()
    val loadSliderPageData = loadSliderPage.first()
    return runCatching {
        intent.viewState!!.copy(
            sliderData = loadSliderPageData.getOrThrow(),
            Errors = null,
            Progress = false,
            noRestaurantFound = false)
    }
        .getOrElse {
            intent.viewState!!.copy(Errors = UserError.NetworkError(it))
        }

}

private suspend fun getVendorOffersData(
    loadBranchesDataByGoogleID: suspend () -> Flow<Result<List<BranchesModelListItem>>>,
    intent: RestaurantsListIntent,
): RestaurantsListViewStateIntent {

    val BranchesData = loadBranchesDataByGoogleID().first()

    return runCatching {
        intent.viewState!!.copy(BranchesData = brancheslistusecase(BranchesData.getOrThrow()),
            FilterdBranchesData = brancheslistusecase(BranchesData.getOrThrow()),
            Errors = null, Progress = false, noRestaurantFound = false)
    }
        .getOrElse {
            intent.viewState!!.copy(Errors = UserError.NetworkError(it))
        }

}


private suspend fun getNearestBranchesData(
    loadNearestBranchesData: suspend () -> Flow<Result<List<BranchesModelListItem>>>,
    intent: RestaurantsListIntent,
): RestaurantsListViewStateIntent {

    val BranchesData = loadNearestBranchesData().first()

    return runCatching {
        intent.viewState!!.copy(BranchesData = brancheslistusecase(BranchesData.getOrThrow()),
            FilterdBranchesData = brancheslistusecase(BranchesData.getOrThrow()),
            Errors = null, Progress = false, noRestaurantFound = false)
    }
        .getOrElse {
            intent.viewState!!.copy(Errors = UserError.NetworkError(it))
        }

}

private suspend fun getNewOfferData(
    loadNearestBranchesData: suspend () -> Flow<Result<List<BranchesModelListItem>>>,
    intent: RestaurantsListIntent,
): RestaurantsListViewStateIntent {
    val BranchesData = loadNearestBranchesData().first()
    return runCatching {
        intent.viewState!!.copy(BranchesData = brancheslistusecase(BranchesData.getOrThrow()),
            FilterdBranchesData = brancheslistusecase(BranchesData.getOrThrow()),
            Errors = null, Progress = false, noRestaurantFound = false)
    }
        .getOrElse {
            intent.viewState!!.copy(Errors = UserError.NetworkError(it))
        }

}

private suspend fun getFreeDeliveryBranchesData(
    loadNearestBranchesData: suspend () -> Flow<Result<List<BranchesModelListItem>>>,
    intent: RestaurantsListIntent,
): RestaurantsListViewStateIntent {

    val BranchesData = loadNearestBranchesData().first()

    return runCatching {
        intent.viewState!!.copy(BranchesData = brancheslistusecase(BranchesData.getOrThrow()),
            FilterdBranchesData = brancheslistusecase(BranchesData.getOrThrow()),
            Errors = null, Progress = false, noRestaurantFound = false)
    }
        .getOrElse {
            intent.viewState!!.copy(Errors = UserError.NetworkError(it))
        }

}

private fun FilterData(
    intent: RestaurantsListIntent,
    CuisinesId: Int,
): RestaurantsListViewStateIntent {
    var filterdata = FilterBranchesListByCuisines(CuisinesId, intent.viewState!!.BranchesData!!)
    return intent.viewState!!.copy(BranchesData = brancheslistusecase(filterdata), isFilter = true)
}

private fun sortByName(intent: RestaurantsListIntent): RestaurantsListViewStateIntent {
    var filterdata = sortBranchesByName(intent.viewState!!.BranchesUseCaseData)
    return intent.viewState!!.copy(BranchesData = brancheslistusecase(FiltredData = filterdata), isFilter = true)
}

private fun sortBranchesByRate(intent: RestaurantsListIntent): RestaurantsListViewStateIntent {
    var filterdata = sortBranchesByRate(intent.viewState!!.BranchesUseCaseData)
    return intent.viewState!!.copy(BranchesData = brancheslistusecase(FiltredData = filterdata), isFilter = true)
}

private fun sortBranchesByDeliveryCost(intent: RestaurantsListIntent): RestaurantsListViewStateIntent {
    var filterdata = sortBranchesByDeliveryCost(intent.viewState!!.BranchesUseCaseData!!)
    return intent.viewState!!.copy(BranchesData = brancheslistusecase(FiltredData = filterdata), isFilter = true)
}

private fun sortBranchesByDeliveryTime(intent: RestaurantsListIntent): RestaurantsListViewStateIntent {
    var filterdata = sortBranchesByDeliveryTime(intent.viewState!!.BranchesUseCaseData)
    return intent.viewState!!.copy(BranchesData = brancheslistusecase( FiltredData = filterdata), isFilter = true)
}

private fun SearchBranchesByName(
    intent: RestaurantsListIntent,
    RestaurantName: String,
): RestaurantsListViewStateIntent {
    val filterdata = SearchBranchesByname(RestaurantName, intent.viewState!!.BranchesUseCaseData!!)
    return intent.viewState!!.copy(BranchesData = brancheslistusecase(FiltredData = filterdata))

}

private fun filterTopRatedVendors(intent: RestaurantsListIntent): RestaurantsListViewStateIntent {
    val filterdata = filterTopRatedVendors(intent.viewState!!.BranchesUseCaseData)
    return intent.viewState!!.copy(BranchesData = brancheslistusecase( FiltredData = filterdata), isFilter = true)
}

private fun filterOpenedVendors(intent: RestaurantsListIntent): RestaurantsListViewStateIntent {
    val filterdata = filterOpenedVendors(intent.viewState!!.BranchesUseCaseData)
    return intent.viewState!!.copy(BranchesData = brancheslistusecase( FiltredData= filterdata), isFilter = true)
}

private fun filterFreeDeliveryVendors(intent: RestaurantsListIntent): RestaurantsListViewStateIntent {
    val filterdata = filterFreeDeliveryVendors(intent.viewState!!.BranchesUseCaseData)
    return intent.viewState!!.copy(BranchesData = brancheslistusecase( FiltredData=filterdata), isFilter = true)
}