package com.codes_roots.imakeapp.presentaion.stores_fragment.mvi

import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.satafood.core.entities.google.GoogleModel
import com.satafood.core.entities.google.SliderModel
import com.satafood.core.entities.homepage.sliders.SliderHomePageModelItem
import com.satafood.core.mapper.restaurantslist.BranchesDataUseCase
import com.satafood.core.mapper.restaurantslist.brancheslistusecase
import com.satafood.core.mapper.restaurantslist.cuisinesUseCase

data class RestaurantsListViewStateIntent(
    val cuisinesData: cuisinesUseCase? = null,
    val BranchesData: brancheslistusecase? = null,
    var BranchesUseCaseData: ArrayList<BranchesDataUseCase>? = null,
    var filterText: String? = null,
    val sliderData: List<SliderHomePageModelItem>? = null,
    val FilterdBranchesData: brancheslistusecase? = null,
    val Progress: Boolean? = null,
    val Errors: UserError? = null,
    var IsFave: Boolean? = null,
    val isFilter: Boolean? = false,
    val isLocationChanged: Boolean? = false,
    val noRestaurantFound: Boolean? = false,

    )


sealed class RestaurantsListIntent(
    open val viewState: RestaurantsListViewStateIntent? = null,
    open val AreaId: Int? = null,
    open val restauratnId: Int? = null,
    open val catid: Int? = null,
    open val googleModel: GoogleModel? = null,
    open val sliderModel: SliderModel? = null,
    open val page: Int? = null,
) {

    class LoadData(
        override val viewState: RestaurantsListViewStateIntent?,
        override val AreaId: Int? = null,
        override val catid: Int?
    ) : RestaurantsListIntent()

    class AddFavorite(
        override val viewState: RestaurantsListViewStateIntent?,
        override val restauratnId: Int?,
        override val AreaId: Int?
    ) : RestaurantsListIntent()

    class LoadVendorOffersData(
        override val viewState: RestaurantsListViewStateIntent?,
        override val googleModel: GoogleModel?
    ) : RestaurantsListIntent()

    class LoadNearestBranchesData(
        override val viewState: RestaurantsListViewStateIntent?,
        override val googleModel: GoogleModel?
    ) : RestaurantsListIntent()

    class LoadNewOfferData(
        override val viewState: RestaurantsListViewStateIntent?,
        override val googleModel: GoogleModel?
    ) : RestaurantsListIntent()

    class LoadFreeDliveryBranchesData(
        override val viewState: RestaurantsListViewStateIntent?,
        override val googleModel: GoogleModel?
    ) : RestaurantsListIntent()

    class LoadMostOrderedBranchData(
        override val viewState: RestaurantsListViewStateIntent?,
        override val googleModel: GoogleModel?
    ) : RestaurantsListIntent()

    class LoadDataByGoogleID(
        override val viewState: RestaurantsListViewStateIntent?,
        override val googleModel: GoogleModel?,
        override val catid: Int? = null,
        override val sliderModel: SliderModel?,
        override val page: Int?
    ) : RestaurantsListIntent()

    class LoadDataByCatId(
        override val viewState: RestaurantsListViewStateIntent?,
        override val googleModel: GoogleModel?,
        override val page: Int?
    ) : RestaurantsListIntent()

    class LoadingPaging(
        override val viewState: RestaurantsListViewStateIntent?,
        override val googleModel: GoogleModel?,
        override val catid: Int? = null,
        override val sliderModel: SliderModel?,
        override val page: Int?
    ) : RestaurantsListIntent()

    class LoadingCuisines(
        override val viewState: RestaurantsListViewStateIntent?,
        override val googleModel: GoogleModel?,
        override val catid: Int? = null,
    ) : RestaurantsListIntent()

    class GetSliderData(
        override val viewState: RestaurantsListViewStateIntent?,
        override val googleModel: GoogleModel?,
        override val catid: Int? = null,
        override val sliderModel: SliderModel?
    ) : RestaurantsListIntent()

    class LoadingSliderData(
        override val viewState: RestaurantsListViewStateIntent?,
        override val googleModel: GoogleModel?,
        override val catid: Int? = null,
        override val sliderModel: SliderModel?,
        override val page: Int?
    ) : RestaurantsListIntent()

    class DisplayErrors(override val viewState: RestaurantsListViewStateIntent) :
        RestaurantsListIntent()

    class FilterBranchesData(
        override val viewState: RestaurantsListViewStateIntent? = null,
        val CuisinesId: Int? = null
    ) : RestaurantsListIntent()

    class SortByName(override val viewState: RestaurantsListViewStateIntent? = null) :
        RestaurantsListIntent()

    class SortByDeliveryCost(override val viewState: RestaurantsListViewStateIntent? = null) :
        RestaurantsListIntent()

    class SortByDeliveryRating(override val viewState: RestaurantsListViewStateIntent? = null) :
        RestaurantsListIntent()

    class SortByDeliveryTime(override val viewState: RestaurantsListViewStateIntent? = null) :
        RestaurantsListIntent()

    //    class SearchByName(override val viewState: RestaurantsListViewStateIntent? = null,val RestaurantName:String? = null): RestaurantsListIntent()
    class NoRestaurant(override val viewState: RestaurantsListViewStateIntent) :
        RestaurantsListIntent()

    class FilterByTopRated(override val viewState: RestaurantsListViewStateIntent) :
        RestaurantsListIntent()

    class FilterOpenedVendors(override val viewState: RestaurantsListViewStateIntent) :
        RestaurantsListIntent()

    class FilterFreeDeliveryVendors(override val viewState: RestaurantsListViewStateIntent) :
        RestaurantsListIntent()

}