package com.codes_roots.imakeapp.presentaion.support

import android.content.Context
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.appcompat.content.res.AppCompatResources
import androidx.recyclerview.widget.AsyncListDiffer
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.RecyclerView
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.databinding.TypeItemBinding
import com.satafood.core.entities.support.QuestionCategoryItem

class QuestionTypesAdapter (val context: Context, private val itemClickListener: ItemClickListener) : RecyclerView.Adapter<QuestionTypesAdapter.ViewHolder>() {

    interface ItemClickListener {
        fun onItemClick(position: Int)
    }

    fun getItemAt(position: Int): QuestionCategoryItem = questionTypes[position]

    var item_index = -1


    inner class ViewHolder(val binding: TypeItemBinding) : RecyclerView.ViewHolder(binding.root)

    private val diffCallback = object : DiffUtil.ItemCallback<QuestionCategoryItem>(){
        override fun areItemsTheSame(oldItem: QuestionCategoryItem, newItem: QuestionCategoryItem): Boolean {
            return oldItem.id == newItem.id
        }

        override fun areContentsTheSame(oldItem: QuestionCategoryItem, newItem: QuestionCategoryItem): Boolean {
            return oldItem == newItem
        }
    }

    private val differ = AsyncListDiffer(this, diffCallback)

    var questionTypes : List<QuestionCategoryItem>
        get() = differ.currentList
        set(value) { differ.submitList(value)}

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        return ViewHolder(TypeItemBinding.inflate(
            LayoutInflater.from(parent.context),
            parent,
            false
        ))
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        holder.binding.apply {
            val type = questionTypes[position]
            questionType.text = type.name
            root.setOnClickListener {
                itemClickListener.onItemClick(position)
                item_index = position
                notifyDataSetChanged()
            }
            if (item_index == position) {
                questionType.setTextColor(context.getColor(R.color.white))
                questionType.background = AppCompatResources.getDrawable(context,R.drawable.gradient_round_bg)
            }
            else{
                questionType.setTextColor(context.getColor(R.color.grey_font))
                questionType.background = AppCompatResources.getDrawable(context,R.drawable.orders_tv_bg)
            }
        }
    }

    override fun getItemCount() = questionTypes.size

}