package com.codes_roots.imakeapp.presentaion.support

import android.annotation.SuppressLint
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import com.codes_roots.imakeapp.databinding.FragmentSupportBinding
import com.codes_roots.imakeapp.helper.*
import com.codes_roots.imakeapp.presentaion.support.mvi.SupportIntent
import com.satafood.core.entities.support.questionsItem
import kotlinx.coroutines.launch
import javax.inject.Inject

// TODO: Rename parameter arguments, choose names that match
// the fragment initialization parameters, e.g. ARG_ITEM_NUMBER
private const val ARG_PARAM1 = "param1"
private const val ARG_PARAM2 = "param2"

class SupportFragment @Inject constructor() : Fragment() {

    private var binding: FragmentSupportBinding by autoCleared()

//    lateinit var queriesAdapter: QueriesAdapter
    private lateinit var questionTypesAdapter : QuestionTypesAdapter
    private lateinit var questionsAdapter: CommonQuestionsAdapter
    private lateinit var filteredList : List<questionsItem>


    @Inject
    lateinit var Pref: PreferenceHelper

    @Inject
    lateinit var viewModelFactory: ViewModelProvider.Factory

    val viewModel by viewModels<SupportViewModel> { viewModelFactory }


    // TODO: Rename and change types of parameters
    private var param1: String? = null
    private var param2: String? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        BaseApplication.appComponent.inject(this)
        arguments?.let {
            param1 = it.getString(ARG_PARAM1)
            param2 = it.getString(ARG_PARAM2)
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        // Inflate the layout for this fragment
        binding = FragmentSupportBinding.inflate(inflater, container, false)
        val root: View = binding.root


        setupQuestionTypeRecycle()
        setupQuestionsRecycle()

//        viewModel.questionsItemLD.observe(requireActivity()) {
//            if (it != null)
//                questionsAdapter.questions = it
//        }
//
//
//        viewModel.questionCategoryLD.observe(viewLifecycleOwner){
//            questionTypesAdapter.questionTypes = it
//        }

        viewModel.questionsType.observe(viewLifecycleOwner){
            questionTypesAdapter.questionTypes = it
        }

        viewModel.questions.observe(viewLifecycleOwner){
            questionsAdapter.questions = it
            filteredList = ArrayList(it)
        }

        binding.backBtn.setOnClickListener { requireActivity().onBackPressed()  }

        viewModel.isFilter.observe(viewLifecycleOwner){
            try {
                if (it == true){
                    addTextListenerForFiltered()
                }else{
                    addTextListener()
                }
            }catch (e:Exception){
                Log.e("supportFilteredSearch", "onItemClick: supportSearch error : ${e.message}", )

            }

        }


        return root
    }

    private fun setupQuestionsRecycle() = binding.questionTypeRv.apply {
        questionTypesAdapter = QuestionTypesAdapter(requireContext() ,object : QuestionTypesAdapter.ItemClickListener {
            override fun onItemClick(position: Int) {
                getQuestionsForThatType(position)
                viewModel.isFilter.value = true
            }
        })

        adapter = questionTypesAdapter
        layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)

    }

    @SuppressLint("NotifyDataSetChanged")
    private fun getQuestionsForThatType(position: Int) {
        val item = questionTypesAdapter.getItemAt(position).id
        viewModel.getQuestionsByType(item!!)

        viewModel.questionsFiltered.observe(viewLifecycleOwner){
            questionsAdapter.questions = it
            questionsAdapter.notifyDataSetChanged()
        }

    }

    private fun setupQuestionTypeRecycle() = binding.rvQueries.apply {
        questionsAdapter = CommonQuestionsAdapter(requireContext())
        adapter = questionsAdapter
        layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

    }

    companion object {

        @JvmStatic
        fun newInstance(param1: String, param2: String) =
            SupportFragment().apply {
                arguments = Bundle().apply {
                    putString(ARG_PARAM1, param1)
                    putString(ARG_PARAM2, param2)
                }
            }
    }


    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

//        setupSupportAdapter()

        lifecycleScope.launch {
            Pref.token?.let {
                viewModel.intentChannel.send(SupportIntent.GetQueries)
            }
        }
//        observeStateFlow()
    }

    private fun addTextListener(){
        binding.searchBar.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {
            }

            @SuppressLint("NotifyDataSetChanged")
            override fun onTextChanged(query: CharSequence?, p1: Int, p2: Int, p3: Int) {
                viewModel.tempList.clear()
                val queries = query.toString().lowercase()
                try {
                    if (queries.isNotEmpty()){
                        viewModel.search(queries)
                        questionsAdapter.questions = viewModel.tempList
                        binding.rvQueries.adapter!!.notifyDataSetChanged()
                    }else{
                        viewModel.tempList.clear()
                        viewModel.questions.value?.let { viewModel.tempList.addAll(it) }
                        questionsAdapter.questions = viewModel.tempList
                        binding.rvQueries.adapter!!.notifyDataSetChanged()

                    }
                }catch (e:Exception){
                    Log.e("supportFilteredSearch", "onItemClick: supportSearch error : ${e.message}", )
                }


            }

            override fun afterTextChanged(p0: Editable?) {
            }

        })
    }

    private fun addTextListenerForFiltered(){
        binding.searchBar.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {
            }

            @SuppressLint("NotifyDataSetChanged")
            override fun onTextChanged(query: CharSequence?, p1: Int, p2: Int, p3: Int) {
                viewModel.tempListFilter.clear()
                viewModel.tempList.clear()
                val queries = query.toString().lowercase()
                try {
                    if (queries.isNotEmpty()){
                        viewModel.searchFiltered(queries)
                        questionsAdapter.questions = viewModel.tempListFilter
                        binding.rvQueries.adapter!!.notifyDataSetChanged()
                    }else{
                        viewModel.tempListFilter.clear()
                        viewModel.tempList.clear()
                        viewModel.questionsFiltered.value?.let { viewModel.tempListFilter.addAll(it) }
                        questionsAdapter.questions = viewModel.tempListFilter
                        binding.rvQueries.adapter!!.notifyDataSetChanged()

                    }

                }catch (e:Exception){
                    Log.e("supportFilteredSearch", "onItemClick: supportFilteredSearch error : ${e.message}", )
                }
            }
            override fun afterTextChanged(p0: Editable?) {
            }

        })
    }

//    private fun observeStateFlow() {
//        lifecycleScope.launchWhenStarted {
//            viewModel.queries.collect {
//                when (it.status) {
//                    DataState.Status.LOADING -> binding.loading = true
//                    DataState.Status.SUCCESS -> {
//                        Log.d("TAG", "observeStateFlow: $it")
//                        binding.loading = false
//                        it.data?.let { queriesAdapter.submitList(it) }
//                    }
//                    DataState.Status.ERROR -> {
//                        binding.loading = false
//                    }
//                }
//            }
//        }
//
//    }

//    private fun observeStateFlow2() {
//        lifecycleScope.launchWhenStarted {
//            viewModel.questions.collect {
//                when (it.status) {
//                    DataState.Status.LOADING -> binding.loading = true
//                    DataState.Status.SUCCESS -> {
//                        Log.d("TAG", "observeStateFlow: $it")
//                        binding.loading = false
//                        it.data?.let { queriesAdapter.submitList(it) }
//                    }
//                    DataState.Status.ERROR -> {
//                        binding.loading = false
//                    }
//                }
//            }
//        }
//
//    }
//    private fun setupSupportAdapter() {
//        queriesAdapter = QueriesAdapter(viewModel, { query ->
//            Log.d("TAG", "setupSupportAdapter: $query")
//        },requireContext())
//
//        binding.rvQueries.apply {
//            adapter = queriesAdapter
//            addItemDecoration(
//                VerticalMarginItemDecoration(
//                    dpToPx(10f, requireContext())
//                )
//            )
//        }
//
//    }
}