package com.codes_roots.imakeapp.presentaion.support

import android.util.Log
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.codes_roots.imakeapp.presentaion.support.mvi.SupportIntent
import com.codes_roots.imakeapp.repository.support.SupportDataSource
import com.satafood.core.base.DataState
import com.satafood.core.entities.support.Complaints
import com.satafood.core.entities.support.Query
import com.satafood.core.entities.support.QuestionCategoryItem
import com.satafood.core.entities.support.questionsItem
import com.satafood.core.mapper.main.GetQueriesUseCase
import kotlinx.coroutines.*
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.flow.*
import java.util.*
import javax.inject.Inject
import kotlin.Exception

class SupportViewModel @Inject constructor(
    private val getQueriesUseCase: GetQueriesUseCase, private val Datasources: SupportDataSource
) : ViewModel() {


    val intentChannel = Channel<SupportIntent?>(Channel.UNLIMITED)
    private var job: Job? = null

    private val _queries: MutableStateFlow<DataState<List<Query>>> =

        MutableStateFlow(DataState<List<Query>>(DataState.Status.LOADING))

    val queries = _queries.asStateFlow()


    val errorMessage = MutableLiveData<String>()
    val loading = MutableLiveData<Boolean>()

    private var _questions = MutableLiveData<MutableList<questionsItem>>()
    val questions : LiveData<MutableList<questionsItem>>
        get() = _questions

    val compalaintsLD = MutableLiveData<Complaints>()
//    val questionsItemLD = MutableLiveData<List<questionsItem>>()
//    val questionCategoryLD = MutableLiveData<List<QuestionCategoryItem>>()


    private var _questionsFiltered = MutableLiveData<MutableList<questionsItem>>()
    val questionsFiltered : LiveData<MutableList<questionsItem>>
        get() = _questionsFiltered

    private var _questionsType = MutableLiveData<List<QuestionCategoryItem>>()
    val questionsType : LiveData<List<QuestionCategoryItem>>
        get() = _questionsType

    var tempList = mutableListOf<questionsItem>()
    var tempListFilter = mutableListOf<questionsItem>()

    //var filteredQuestions = mutableListOf<questionsItem>()

    var isFilter = MutableLiveData<Boolean>(false)


    init {
        processIntent()
        getAllCommonQuestions()
        getAllQuestionTypes()
    }

    private fun processIntent() {
        viewModelScope.launch {
            intentChannel.consumeAsFlow().collect {
                when (it) {
                    is SupportIntent.GetQueries -> getQueries()

                }
            }
        }
    }

    private fun getQueries() {
        viewModelScope.launch() {
            getQueriesUseCase()
                .buffer()
                .onEach { _queries.emit(it) }
                .launchIn(viewModelScope)
        }
    }


    private fun onError(message: String) {
        errorMessage.value = message
        loading.value = false
    }

    fun addComplaints(compalaints:Complaints) {
        job = CoroutineScope(Dispatchers.IO).launch {
            val response = Datasources.addComplaints(compalaints)
            withContext(Dispatchers.Main) {
                if (response.isSuccessful) {
                    compalaintsLD.postValue(response.body())

                } else {
                    onError("Error : ${response.message()} ")
                }
            }
        }

    }


    fun getQuestionsByType(itemId : Int){
        viewModelScope.launch {
            val response = Datasources.getCommonQuestions(itemId)
            try {
                if (response.isSuccessful){
                    _questionsFiltered.value = response.body()
                }
            }catch (e:Exception){
                Log.e("CategoriesVMServerTest", "getAllCategories Method: $e")
                onError("CategoriesVMServerTestOnError : ${response.message()}")

            }
//            for (i in _questions.value!!){
//                if (i.customer_support_cat_id == itemId){
//                    filteredQuestions.add(i)
//                }
//            }
//            _questionsFiltered.value = filteredQuestions
        }
    }

    private fun getAllCommonQuestions(){
        viewModelScope.launch {
            try {
                val response = Datasources.getAllCommonQuestions()
                if (response.isSuccessful){
                    _questions.value = response.body()
                }
            }catch (e: Exception){
                Log.e("CategoriesVMServerTest", "getAllCategories Method: $e")
            }
        }
    }

    private fun getAllQuestionTypes(){
        viewModelScope.launch {
            try {
                val response = Datasources.getQuestionsCategories()
                if (response.isSuccessful){
                    _questionsType.value = response.body()
                }
            }catch (e: Exception){
                Log.e("CategoriesVMServerTest", "getAllCategories Method: $e")
            }
        }
    }

    fun search(query: String){
        viewModelScope.launch {
            for (i in _questions.value!!){
                if (i.questions.lowercase(Locale.getDefault()).contains(query)){
                    tempList.add(i)
                }
            }
        }
    }

    fun searchFiltered(query: String){
        viewModelScope.launch {
            for (i in _questionsFiltered.value!!){
                if (i.questions.lowercase(Locale.getDefault()).contains(query)){
                    tempListFilter.add(i)
                }
            }
        }
    }


}