package com.codes_roots.imakeapp.presentaion.vendordetailsfragment

import android.annotation.SuppressLint
import android.app.Activity
import android.content.ActivityNotFoundException
import android.content.Intent
import android.graphics.Color
import android.graphics.PorterDuff
import android.os.Bundle
import android.speech.RecognizerIntent
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.InputMethodManager
import android.widget.AbsListView
import android.widget.Toast
import androidx.annotation.NonNull
import androidx.constraintlayout.motion.utils.ViewState
import androidx.core.view.isVisible
import androidx.core.widget.doOnTextChanged
import androidx.databinding.DataBindingUtil
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentFactory
import androidx.fragment.app.viewModels
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.LinearSmoothScroller
import androidx.recyclerview.widget.RecyclerView
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.databinding.RestaurantDetailsFragmentBinding
import com.codes_roots.imakeapp.helper.BaseApplication
import com.codes_roots.imakeapp.helper.Error_MotionToast
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.helper.WARN_MotionToast
import com.codes_roots.imakeapp.presentaion.ClickHandler
import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.presentaion.mainactivity.MainActivity
import com.codes_roots.imakeapp.presentaion.vendordetailsfragment.adapter.MenuRowAdapter
import com.codes_roots.imakeapp.presentaion.vendordetailsfragment.adapter.VendorCategoryAdapter
import com.codes_roots.imakeapp.presentaion.vendordetailsfragment.mvi.MainIntent
import com.codes_roots.imakeapp.presentaion.vendordetailsfragment.viewmodel.RestaurantDetailsViewModel
import com.satafood.core.entities.restaurantdetails.MenuCategoriesItem
import kotlinx.android.synthetic.main.restaurant_details_fragment.*
import kotlinx.android.synthetic.main.vendor_details_bar.*
import kotlinx.coroutines.flow.collect
import net.yslibrary.android.keyboardvisibilityevent.KeyboardVisibilityEvent
import net.yslibrary.android.keyboardvisibilityevent.KeyboardVisibilityEventListener
import java.math.RoundingMode
import java.util.*
import javax.inject.Inject


open class VendorDetailsFragment @Inject constructor(var fragmentFactory: FragmentFactory) :
    Fragment() {
    private val REQUEST_CODE_STT = 102

    @Inject
    lateinit var viewModelFactory: ViewModelProvider.Factory

    @Inject
    lateinit var Pref: PreferenceHelper

    lateinit var view: RestaurantDetailsFragmentBinding


    lateinit var categoryAdapter: VendorCategoryAdapter

    lateinit var menuAdapter: MenuRowAdapter


    var filteredData: List<MenuCategoriesItem>? = null

    private val viewModel by viewModels<RestaurantDetailsViewModel> { viewModelFactory }

    override fun onCreate(savedInstanceState: Bundle?) {
        childFragmentManager.fragmentFactory = fragmentFactory
        super.onCreate(savedInstanceState)
        BaseApplication.appComponent.inject(this)
        //ClickHandler().bottomNavItemSelected(requireContext(), R.id.home, false)
    }

    fun categoryRecycleView() {
        categoryAdapter = VendorCategoryAdapter(viewModel,this, requireContext())
        view.menuCategoryRecycleView.apply {
            adapter = categoryAdapter
            setHasFixedSize(true)
        }

    }

    fun menuRecycleView() {
        menuAdapter = MenuRowAdapter(
            viewModel, requireArguments().getInt("RestaurantـId"),
            requireContext()
        )
        view.menuRecycleView.apply {
            var viewState = viewModel.state.value
            adapter = menuAdapter
            layoutManager = LinearLayoutManager(context)
            addOnScrollListener(object : RecyclerView.OnScrollListener() {
                override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                    super.onScrolled(recyclerView, dx, dy)
                    val lastVisibleItem =
                        (Objects.requireNonNull(recyclerView.layoutManager) as LinearLayoutManager).findFirstVisibleItemPosition()
                    Log.e("Position", lastVisibleItem.toString())
                    if (viewState?.filterKey == false) {
                        viewModel.intents.trySend(
                            MainIntent.ItemSwitching(
                                viewState.copy(
                                    position = lastVisibleItem
                                )
                            )
                        )
                    }
                }
            })

            setHasFixedSize(true)
        }

    }


    @SuppressLint("ClickableViewAccessibility")
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?,
    ): View? {

        var view: RestaurantDetailsFragmentBinding = DataBindingUtil.inflate(
            inflater,
            R.layout.restaurant_details_fragment, container, false
        )
        this.view = view
        categoryRecycleView()
        menuRecycleView()
        view.top.context = context as MainActivity
        view.top.viewModel = viewModel


        view.context = context as MainActivity

        //start shimmer loading
        view.shimmerViewContainer.startShimmerAnimation()
        view.top.shimmerViewRounded.startShimmerAnimation()
        view.top.shimmerView.startShimmerAnimation()
        view.shimmerTextContainer.startShimmerAnimation()

       viewModel.state.value?.position = requireArguments().getInt("category_id")
        Log.i("position", "category: " + viewModel.state.value?.position.toString())
        //call server data
        viewModel.intents.trySend(
            MainIntent
                .LoadData(
                    viewModel.state.value!!.copy(filterKey = true),
                    requireArguments().getInt("RestaurantـId"),
                    Pref.areaId
                )
        )


        view.ButtonClick.background.setColorFilter(
            Color.parseColor(Pref.appPrimaryColor),
            PorterDuff.Mode.SRC_ATOP
        )


        //  view.searchBar.setError("assad")
//        view.searchBar.doOnTextChanged { text, start, before, count ->
//            viewModel.intents.trySend(
//                MainIntent.SearchByName(
//                    viewModel.state.value!!.copy(filterKey = true),
//                    text.toString()
//                )
//            )
//        }

        KeyboardVisibilityEvent.setEventListener(requireActivity(),
            KeyboardVisibilityEventListener {
                view.top.root.isVisible = !it
            })
        showSoftKeyboard(view.searchBar)
        view.microphone.setOnClickListener {
            val sttIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
            sttIntent.putExtra(
                RecognizerIntent.EXTRA_LANGUAGE_MODEL,
                RecognizerIntent.LANGUAGE_MODEL_FREE_FORM
            )
            Log.i("lang", "onCreateView: " + Locale.getDefault().toString())
            sttIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, (context as MainActivity).Pref.lang)
            sttIntent.putExtra(RecognizerIntent.EXTRA_PROMPT, "Speak now!")
            try {
                startActivityForResult(sttIntent, REQUEST_CODE_STT)
            } catch (e: ActivityNotFoundException) {
                e.printStackTrace()
                Toast.makeText(
                    requireContext(),
                    "Your device does not support STT.",
                    Toast.LENGTH_LONG
                ).show()
            }
        }

        view.ButtonClick.isVisible = Pref.retriveCartItemsValue().isNotEmpty()
        view.TotalItem.text = Pref.retriveCartItemsValue().sumOf { it.total!! }.toString() + Pref.currency
        view.cartSize.text = Pref.retriveCartItemsValue().size.toString()


        //   هندلة الحالة لو رجعت بخطأ في المطاعم فقط
        lifecycleScope.launchWhenStarted {
            viewModel.state.collect { ViewState ->
                val restaurantDetailsData = ViewState?.data?.details?.data
                kotlin.runCatching {
                    if (ViewState?.Errors != null) {
                        ViewState.Errors.let {
                            when (it) {
                                is UserError.NetworkError -> WARN_MotionToast(
                                    requireActivity().getString(
                                        R.string.chooseArea
                                    ), requireActivity()
                                )
                                is UserError.TokenNeeded -> WARN_MotionToast(
                                    requireActivity().getString(
                                        R.string.loginFirst
                                    ), requireActivity()
                                )

                            }
                        }
                        viewModel.intents.send(MainIntent.DisplayErrors(ViewState))

                    } else {
                        viewModel.data = ViewState!!.data!!.details

                        if (view.top.description.text == ""){
                            view.top.description.apply {
                                viewModel.data!!.cuisines!!.forEachIndexed { index, cuisine ->
                                    if (viewModel.data!!.cuisines!!.lastIndex != index)
                                        append(cuisine.name + "-")
                                    else append(cuisine.name)
                                }
                            }
                        }


                        view.top.listener = ClickHandler()
                        view.listener = ClickHandler()
                        view.progress.isVisible = ViewState.progress ?: false
                        view.progress.isVisible = ViewState.progress ?: false
                        shimmer_text_container.isVisible = ViewState.progress ?: false
                        shimmer_view.isVisible = ViewState.progress ?: false
                        view.shimmerViewContainer.isVisible = ViewState.progress ?: false
                        view.top.shimmerViewRounded.isVisible = ViewState.progress ?: false
                        view.top.discount.isVisible =
                            !restaurantDetailsData!!.offerDescription!!.contains("false")
                        view.top.data = ViewState.data!!.details
                        Pref.restaurantUserId = restaurantDetailsData.user_id!!
                        categoryAdapter.submitList(restaurantDetailsData.menu_categories)
                        if (restaurantDetailsData.menu_categories!!.isNotEmpty()) {
                            menuAdapter.submitList(ViewState.ItemFiltred)
                        } else {
                            Error_MotionToast("", requireActivity())
                        }

                        Pref.BranchId = restaurantDetailsData.branch_id!!
                        val starsNumber = restaurantDetailsData.rate!!
                        val rounded =
                            starsNumber.toBigDecimal().setScale(1, RoundingMode.UP).toDouble()
//                    view.numberOfStars.text = rounded.toString()


//                    view.title.text = ViewState.title


                    }

                }.getOrElse {
                    // Error_MotionToast(it.cause.toString(),requireActivity())
                }
            }
        }

        viewModel.quantity!!.observe(requireActivity()) {
            view.top.cartSize.text = it.toString()
            view.cartSize.text = it.toString()
            view.ButtonClick.isVisible = true
            view.TotalItem.text = Pref.retriveCartItemsValue().sumOf { it.total!! }.toString()+ Pref.currency
        }

        return view.root
    }

    fun RecyclerView.smoothSnapToPosition(
        position: Int,
        snapMode: Int = LinearSmoothScroller.SNAP_TO_START
    ) {
        val smoothScroller = object : LinearSmoothScroller(this.context) {
            override fun getVerticalSnapPreference(): Int = snapMode
            override fun getHorizontalSnapPreference(): Int = snapMode
        }
        smoothScroller.targetPosition = position
        layoutManager?.startSmoothScroll(smoothScroller)
    }

    private fun collapseAppBar() {
        // Collapse the AppBarLayout with animation
        view.appbar.setExpanded(false, true)
    }

    open fun showSoftKeyboard(view: View) {
        val inputMethodManager: InputMethodManager =
            activity!!.getSystemService(Activity.INPUT_METHOD_SERVICE) as InputMethodManager
        view.requestFocus()
        inputMethodManager.showSoftInput(view, 0)
    }

//    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
//        super.onActivityResult(requestCode, resultCode, data)
//        when (requestCode) {
//            REQUEST_CODE_STT -> {
//                if (resultCode == Activity.RESULT_OK && data != null && viewModel.state.value != null) {
//                    val result = data.getStringArrayListExtra(RecognizerIntent.EXTRA_RESULTS)
//                    result?.let {
//                        val recognizedText = it[0]
//                        view.searchBar.setText(recognizedText)
//                        viewModel.intents.trySend(
//                            MainIntent.SearchByName(
//                                viewModel.state.value!!,
//                                recognizedText.toString()
//                            )
//                        )
//
//                    }
//                }
//            }
//        }
//    }

    override fun onResume() {
        super.onResume()
        view.top.cartSize.text = Pref.retriveCartItemsValue().size.toString()

    }

}