package com.codes_roots.imakeapp.presentaion.vendordetailsfragment.adapter

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.Color
import android.graphics.PorterDuff
import android.util.Log

import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.core.content.ContextCompat
import androidx.databinding.DataBindingUtil
import androidx.lifecycle.lifecycleScope
import androidx.lifecycle.viewModelScope
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.LinearSmoothScroller
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.databinding.RestaurantMenuCategoryAdapterBinding
import com.codes_roots.imakeapp.presentaion.ClickHandler
import com.codes_roots.imakeapp.presentaion.mainactivity.MainActivity
import com.codes_roots.imakeapp.presentaion.vendordetailsfragment.VendorDetailsFragment
import com.codes_roots.imakeapp.presentaion.vendordetailsfragment.mvi.MainIntent
import com.codes_roots.imakeapp.presentaion.vendordetailsfragment.viewmodel.RestaurantDetailsViewModel
import com.satafood.core.entities.restaurantdetails.MenuCategory
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.launch

class VendorCategoryAdapter(
    var viewModel: RestaurantDetailsViewModel,
    var fragment: VendorDetailsFragment,
    var context: Context?
) : ListAdapter<MenuCategory, CustomViewHolder>(CuisineDiffCallback()) {
    var row_index: Int? = 0

    override fun onBindViewHolder(holder: CustomViewHolder, position: Int) {
        holder.bind(context, currentList[position])
        var viewState = viewModel.state.value

        holder.binding.name.setOnClickListener {
            row_index = position
            viewModel.intents.trySend(MainIntent.ItemSwitching(viewState!!.copy(
                position = position, filterKey = true)))

        }
        /// Text  underLine  when selected

        viewModel.viewModelScope.launch {
            viewModel.state.collect { ViewState ->
                if (ViewState?.position == position) {
                    holder.binding.line.apply {

                    setBackgroundColor(Color.parseColor((context as MainActivity).Pref.appPrimaryColor));

                    }
                    holder.binding.name.setTextColor(Color.parseColor( (context as MainActivity).Pref.appPrimaryColor))
                    if (ViewState.filterKey == true) {
                        fragment.view.menuRecycleView.smoothSnapToPosition(ViewState.position)
                    }
                    fragment.view.menuCategoryRecycleView.smoothSnapToPosition(ViewState.position)
                    Log.i("position", "ViewStatecategory: " + ViewState.position.toString())

                }
                else {
                    holder.binding.line.apply {
                      setBackgroundColor(0x00000000);
                    }
                    holder.binding.name.setTextColor(ContextCompat.getColor(context!!, R.color.grey_font))
                }
            }
        }

    }

    override fun onCreateViewHolder(p0: ViewGroup, p1: Int): CustomViewHolder {
        val binding: RestaurantMenuCategoryAdapterBinding = DataBindingUtil.inflate(
            LayoutInflater.from(p0.context),
            R.layout.restaurant_menu_category_adapter, p0, false
        )
        return CustomViewHolder(binding)
    }

    fun RecyclerView.smoothSnapToPosition(
        position: Int,
        snapMode: Int = LinearSmoothScroller.SNAP_TO_START
    ) {
        val smoothScroller = object : LinearSmoothScroller(this.context) {
            override fun getVerticalSnapPreference(): Int = snapMode
            override fun getHorizontalSnapPreference(): Int = snapMode
        }

        smoothScroller.targetPosition = position

        layoutManager?.startSmoothScroll(smoothScroller)
    }
}

private class CuisineDiffCallback : DiffUtil.ItemCallback<MenuCategory>() {

    override fun areItemsTheSame(oldItem: MenuCategory, newItem: MenuCategory) =
        oldItem.id == newItem.id

    @SuppressLint("DiffUtilEquals")
    override fun areContentsTheSame(oldItem: MenuCategory, newItem: MenuCategory) =
        oldItem == newItem
}

class CustomViewHolder(
    val binding: RestaurantMenuCategoryAdapterBinding,
) : RecyclerView.ViewHolder(binding.root) {

    fun bind(context: Context?, data: MenuCategory) {
        binding.listener = ClickHandler()
        binding.data = data
        binding.context = context as MainActivity?
    }


}
