package com.codes_roots.imakeapp.presentaion.vendordetailsfragment.mvi

import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.repository.retaurantdetails.DataRepo
import com.satafood.core.entities.restaurantdetails.MenuCategoriesItem
import com.satafood.core.entities.restaurantdetails.StoreDetailsModel
import com.satafood.core.mapper.restaurantdetails.FilterItemsListByCategory
import com.satafood.core.mapper.restaurantdetails.MenuCategoryMapper
import com.satafood.core.mapper.restaurantdetails.StoreDetailsMapper
import com.satafood.core.mapper.restaurantdetails.SearchBranchesByname
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first


suspend fun mapIntentToViewState(
    intent: MainIntent,
    Datarepo: DataRepo,
    loadMainData: suspend () -> Flow<Result<StoreDetailsModel>> = { Datarepo.getStoresDetailsData(intent.restauratnId!!, intent.AreaId!!) },
    loadAdsMainData: suspend () -> Flow<Result<ArrayList<MenuCategoriesItem>>> = { Datarepo.getItemAdsData(intent.item_id!!) },
    AddFavorite: suspend () ->  Flow<Result<Boolean>> = { Datarepo.AddFavorite(intent.restauratnId!!, intent.AreaId!!) },

    ) = when (intent) {
    is MainIntent.LoadData -> proceedWithInitialize(loadMainData, intent)
    is MainIntent.DisplayErrors -> intent.viewState.copy(Errors = null)
    is MainIntent.ItemSwitching -> FilterData(intent)
    is MainIntent.AddFavorite -> proceedAddFavorite(AddFavorite,intent)
    is MainIntent.AddItemInItemDetails -> TODO()
    is MainIntent.LoadAdsData -> itemAdsInitialize(loadAdsMainData,intent)
    is MainIntent.SearchByName -> SearchBranchesByName(intent,intent.RestaurantName!!)

}
private suspend fun proceedAddFavorite( AddFavorite : suspend () -> Flow<Result<Boolean>>   ,intent: MainIntent
): MainViewState
{
    return runCatching {
        var response =  AddFavorite().first()
        return intent.viewState!!.copy(IsFave = response.getOrThrow(), progress = false)
    }.getOrElse {
        intent.viewState!!.copy(Errors = UserError.TokenNeeded(it))

    }
}
private suspend fun proceedWithInitialize(
    loadCart: suspend () -> Flow<Result<StoreDetailsModel>>,
    intent: MainIntent
): MainViewState {
    var response = loadCart()
    var data = response.first()
    return runCatching {
        var data = data.getOrThrow()
        intent.viewState!!.copy(
            IsFave = data.restaurantDetails?.first()?.IsFavorite,
            data = StoreDetailsMapper(data),
            itemdata = data.restaurantDetails?.first()?.menu_categories ,
            ItemFiltred =data.restaurantDetails?.first()?.menu_categories,
            Errors = null,
            title = data.restaurantDetails?.first()?.menu_categories?.first()?.name,
            progress = false,
            offer = data.restaurantDetails?.first()?.offerDescription.equals("false")
        )
    }
        .getOrElse {
            intent.viewState!!.copy(Errors = UserError.NetworkError(it))
        }
}
private suspend fun itemAdsInitialize(
    loadCart: suspend () -> Flow<Result<ArrayList<MenuCategoriesItem>>>,
    intent: MainIntent
): MainViewState {
    var response = loadCart()
    var data = response.first()
    return runCatching {
        intent.viewState!!.copy(
            Errors = null,
            progress = false,


        )
    }
        .getOrElse {
            intent.viewState!!.copy(Errors = UserError.NetworkError(it))
        }
}




private  fun FilterData(intent: MainIntent): MainViewState
{
    return intent.viewState!!.copy()
}


private  fun SearchBranchesByName(intent:MainIntent,RestaurantName:String): MainViewState
{
    val filterdata = SearchBranchesByname(RestaurantName,intent.viewState!!.data!!.details!!.data!!.menu_categories!!)
    return intent.viewState!!.copy()
}
