package com.codes_roots.imakeapp.presentaion.wallet_details

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import com.codes_roots.imakeapp.databinding.FragmentWalletDetailsBinding
import com.codes_roots.imakeapp.helper.BaseApplication
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.helper.VerticalMarginItemDecoration
import com.codes_roots.imakeapp.helper.dpToPx
import com.sata.satafood.presentaion.wallet_details.mvi.MainIntent
import com.codes_roots.imakeapp.presentaion.wallet_details.mvi.WalletDetailsViewModel
import com.satafood.core.entities.walletdetails.WalletDetailsModelItem
import kotlinx.android.synthetic.main.fragment_point.*
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.launch
import javax.inject.Inject

class WalletDetailsFragment @Inject constructor() : Fragment() {

    private lateinit var binding: FragmentWalletDetailsBinding

    lateinit var walletAdapter: WalletTransactionsAdapter

    @Inject
    lateinit var Pref: PreferenceHelper

    @Inject
    lateinit var viewModelFactory: ViewModelProvider.Factory

    val viewModel by viewModels<WalletDetailsViewModel> { viewModelFactory }

    var walletId: Int ?= null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        BaseApplication.appComponent.inject(this)
        arguments?.let {
            walletId = it.getInt("walletId")
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        // Inflate the layout for this fragment
        binding = FragmentWalletDetailsBinding.inflate(inflater, container, false)

        val root: View = binding.root

    //    binding.backBtn.setOnClickListener { requireActivity().onBackPressed() }
        getAllData()
        setupWalletAdapter()
        return root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        val walletId  = WalletDetailsModelItem(id = walletId)
        lifecycleScope.launch {
            Pref.token?.let {
                viewModel.intents.trySend(MainIntent.Initialize(viewModel.state.value!!,walletId!!))
            }
        }
    }


    private fun setupWalletAdapter() {
        walletAdapter = WalletTransactionsAdapter(requireContext()) { wallet ->

        }
        binding.rvWallets.apply {
            adapter = walletAdapter
            addItemDecoration(
                VerticalMarginItemDecoration(
                    dpToPx(10f, requireContext())
                )
            )
        }

    }

    private fun setWalletsList(wallets: List<WalletDetailsModelItem>) {
        if (!wallets.isNullOrEmpty()) {
            walletAdapter.submitList(wallets)
            binding.walletsAvailable = true
        } else {
            binding.walletsAvailable = false
            binding.animationView.playAnimation()
        }
    }
    private fun getAllData() {
        lifecycleScope.launchWhenStarted {
            viewModel.state.collect {
                if (it != null) {
                    if (it.error != null) {
                        viewModel.intents.send(MainIntent.ErrorDisplayed(it))
                    } else {
                        if (it.progress == true) {
                        } else {
                            setWalletsList(it.data!!)
                        }
                    }
                }

            }
        }
    }




}