package com.codes_roots.imakeapp.presentaion.wallet_details.mvi


import com.codes_roots.imakeapp.presentaion.homefragment.viewmodel.UserError
import com.codes_roots.imakeapp.repository.wallet_details.WalletTransactionsDataRepo
import com.sata.satafood.presentaion.wallet_details.mvi.MainIntent
import com.satafood.core.entities.walletdetails.WalletDetailsModelItem
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first


/**
 * this is the model function in MVI, it's responsibility is to convert intents into views states
 */
suspend fun mapIntentToViewState(
    intent: MainIntent,
    Datarepo: WalletTransactionsDataRepo,
    loadMainData: suspend () -> Flow<Result<ArrayList<WalletDetailsModelItem>>> = { Datarepo.getWalletTransactions(
        intent.walletId!!
    ) },
) = when (intent) {
    is MainIntent.Initialize -> proceedWithInitialize(loadMainData, intent)
    is MainIntent.ErrorDisplayed -> intent.viewState.copy()
}


private suspend fun proceedWithInitialize(
    loadCart: suspend () -> Flow<Result<ArrayList<WalletDetailsModelItem>>>,
    intent: MainIntent,
): MainViewState {
    val response = loadCart()
    val data = response.first()
    return runCatching {
        intent.viewState!!.copy(data = (data.getOrThrow()),

            error = null,
            progress = false)
    }
        .getOrElse {
            intent.viewState!!.copy(error = UserError.NetworkError(it))
        }

}






