package com.codes_roots.imakeapp.presentaion.webview

import android.os.Build
import android.os.Bundle
import android.view.KeyEvent
import android.view.View
import android.webkit.WebSettings
import android.webkit.WebView
import android.webkit.WebViewClient
import androidx.appcompat.app.AppCompatActivity
import com.codes_roots.imakeapp.R
import com.codes_roots.imakeapp.helper.PreferenceHelper
import com.codes_roots.imakeapp.helper.ResourceUtil
import dagger.android.AndroidInjection
import dagger.android.AndroidInjector
import dagger.android.DispatchingAndroidInjector
import dagger.android.HasAndroidInjector
import kotlinx.android.synthetic.main.web_view.*
import javax.inject.Inject


class WebViewActivity: AppCompatActivity() , HasAndroidInjector {

    @Inject
    lateinit var preferenceHelper: PreferenceHelper


    override fun onCreate(savedInstanceState: Bundle?) {
        AndroidInjection.inject(this)
        super.onCreate(savedInstanceState)
        setContentView(R.layout.web_view)


        webView.webViewClient = WebViewClient()
        webView.settings.setSupportZoom(true)
        webView.settings.javaScriptEnabled = true

        val extras = intent.extras
        val url = extras?.getString("url")!!

        val webview = findViewById<View>(R.id.webView) as WebView
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            ResourceUtil().changeLang(preferenceHelper.lang!!, this)
        }

        val webSettings = webview.settings
        webSettings.javaScriptEnabled = true
        webSettings.domStorageEnabled = true
        webSettings.javaScriptCanOpenWindowsAutomatically = true;
        webSettings.setLayoutAlgorithm(WebSettings.LayoutAlgorithm.NORMAL);
        if (Build.VERSION.SDK_INT >= 11){
            webView.setLayerType(View.LAYER_TYPE_SOFTWARE, null);
        }
        webSettings.setAppCacheMaxSize(5 * 1048576);
        webSettings.databaseEnabled = true
            webview.requestFocus();


        webview.loadUrl(url)


        when {
            Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP -> {
                webSettings.mixedContentMode = 0;
                webView.setLayerType(View.LAYER_TYPE_HARDWARE, null);
            }
            Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT -> {
                webView.setLayerType(View.LAYER_TYPE_HARDWARE, null);
            }
            else -> {
                webView.setLayerType(View.LAYER_TYPE_SOFTWARE, null);
            }
        }

    }
    override fun onKeyDown(keyCode: Int, event: KeyEvent): Boolean {
        if (event.action === KeyEvent.ACTION_DOWN) {
            when (keyCode) {
                KeyEvent.KEYCODE_BACK -> {
                    if (webView.canGoBack()) {
                        webView.goBack()
                    } else {
                        finish()
                    }
                    return true
                }
            }
        }
        return super.onKeyDown(keyCode, event)
    }

    @Inject
    lateinit var androidInjector: DispatchingAndroidInjector<Any>
    override fun androidInjector(): AndroidInjector<Any> {
        return androidInjector
    }

}