package com.codes_roots.imakeapp.repository.RestaurantsList

import com.codes_roots.imakeapp.datalayer.APIServices
import com.satafood.core.entities.google.GoogleModel
import com.satafood.core.entities.google.SliderModel
import com.satafood.core.entities.homepage.Cuisines
import com.satafood.core.entities.homepage.sliders.SliderHomePageModelItem
import com.satafood.core.entities.restaurantslist.BranchesModelListItem
import com.satafood.core.entities.seemore_section.VendorOffersDataItem
import javax.inject.Inject

class RestListRemoteDataSource @Inject constructor(
    private val ApiService: APIServices,
) : RestaurantListDataSource {

    override suspend fun getCuisinesListResponse(category_id: Int): ArrayList<Cuisines> {
        runCatching { return ApiService.GetCuisinesData(category_id) }.getOrElse { throw it }
    }

    override suspend fun AddFavorite(restaurant_id: Int, UserId: Int): Boolean {
        runCatching {  return  ApiService.AddFavorite(restaurant_id, UserId)}.getOrElse { throw it }
    }

    override suspend fun getBranchesListResponse(
        AreasId: Int,
        catid: Int,
    ): ArrayList<BranchesModelListItem> {
        runCatching { return ApiService.GetBranchesData(AreasId, catid) }.getOrElse { throw it }
    }

    override suspend fun getBranchesListByGoogleID(googleModel: GoogleModel,page:Int?): ArrayList<BranchesModelListItem> {
        runCatching { return ApiService.getBranchesByGoogle(googleModel,page) }.getOrElse { throw it }
    }

    override suspend fun getBranchesByAreasByCat(googleModel: GoogleModel,page:Int?): ArrayList<BranchesModelListItem> {
        runCatching { return ApiService.getBranchesByAreasByCat(googleModel,page) }.getOrElse { throw it }
    }

    override suspend fun getVendorOffersData(googleModel: GoogleModel?): ArrayList<VendorOffersDataItem> {
        runCatching { return ApiService.getVendorOffersData(googleModel) }.getOrElse { throw it }
    }


    override suspend fun getNearestBranchesData(googleModel: GoogleModel?):
            ArrayList<BranchesModelListItem> {
        runCatching { return ApiService.getNearestBranchesData(googleModel) }.getOrElse { throw it }
    }


    override suspend fun getNewOfferData(googleModel: GoogleModel?):
            ArrayList<BranchesModelListItem> {
        runCatching { return ApiService.getNewOfferData(googleModel) }.getOrElse { throw it }
    }

    override suspend fun getFreeDliveryBranches(googleModel: GoogleModel?):
            ArrayList<BranchesModelListItem> {
        runCatching { return ApiService.getFreeDliveryBranches(googleModel) }.getOrElse { throw it }
    }

    override suspend fun getMostOrderedBranches(googleModel: GoogleModel?):
            ArrayList<BranchesModelListItem> {
        runCatching { return ApiService.getMostOrderedBranches(googleModel) }.getOrElse { throw it }
    }

    override suspend fun getCategoriesSliders(sliderModel: SliderModel?): ArrayList<SliderHomePageModelItem> {
        runCatching { return ApiService.getCategoriesSliders(sliderModel!!) }.getOrElse { throw it }
    }


}