package com.codes_roots.imakeapp.repository.RestaurantsList

import com.codes_roots.imakeapp.di.IoDispatcher
import com.satafood.core.entities.google.GoogleModel
import com.satafood.core.entities.google.SliderModel
import com.satafood.core.entities.homepage.Cuisines
import com.satafood.core.entities.homepage.sliders.SliderHomePageModelItem
import com.satafood.core.entities.restaurantslist.BranchesModelListItem
import kotlinx.coroutines.*
import kotlinx.coroutines.flow.*
import java.io.IOException
import javax.inject.Inject

class StoresListDataRepo @Inject constructor(
    private val Datasources: RestaurantListDataSource,

    @IoDispatcher private val ioDispatcher: CoroutineDispatcher,
) {
    val coroutineExceptionHandler = CoroutineExceptionHandler{_, throwable ->
        throwable.printStackTrace()
    }

    ///getRestaurantsListResponse
    fun getCuisinesList(catid: Int): Flow<Result<List<Cuisines>>> =
        flow {
            emit(Datasources.getCuisinesListResponse(catid))
        }
            .map { Result.success(it) }
            .retry(4) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(1000)
                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher+coroutineExceptionHandler)

    fun AddFavorite(restaurant_id: Int, UserId:Int): Flow<Result<Boolean>> =
        flow {
            emit(Datasources.AddFavorite(restaurant_id,UserId))
        }
            .map { Result.success(it) }
            .retry(4){ t -> (t is IOException).also { if (it) {
                delay(1000)
            }}}
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher)



    suspend fun getBranchesList(AreasId: Int, catid: Int): Flow<Result<List<BranchesModelListItem>>> =
        withContext(Dispatchers.Main +coroutineExceptionHandler) {
        flow {
            emit(Datasources.getBranchesListResponse(AreasId, catid))
        }

            .map { Result.success(it) }
            .retry(4) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(1000)
                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher+coroutineExceptionHandler)    }



    fun getBranchesByGoogle(googleModel: GoogleModel,page : Int): Flow<Result<List<BranchesModelListItem>>> =
        flow {
            emit(Datasources.getBranchesListByGoogleID(googleModel,page))
        }
            .map { Result.success(it) }
            .retry(1) { t ->
                (t is IOException).also {

                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher+coroutineExceptionHandler)

    fun getBranchesByAreasByCat(googleModel: GoogleModel,page : Int): Flow<Result<List<BranchesModelListItem>>> =
        flow {
            emit(Datasources.getBranchesByAreasByCat(googleModel,page))
        }
            .map { Result.success(it) }
            .retry(1) { t ->
                (t is IOException).also {

                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher+coroutineExceptionHandler)


    fun getCategoriesSliders(sliderModel: SliderModel): Flow<Result<ArrayList<SliderHomePageModelItem>>> =
        flow {
            emit(Datasources.getCategoriesSliders(sliderModel))
        }
            .map { Result.success(it) }
            .retry(1) { t ->
                (t is IOException).also {

                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher+coroutineExceptionHandler)


    fun getVendorOffersData(googleModel: GoogleModel?): Flow<Result<List<BranchesModelListItem>>> =
        flow {
            emit(Datasources.getVendorOffersData(googleModel))
        }
            .map {
                Result.success(it.map {
                    it.restaurants!!
                })
            }
            .retry(4) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(1000)
                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher+coroutineExceptionHandler)

    fun getNearestBranchesData(googleModel: GoogleModel?): Flow<Result<List<BranchesModelListItem>>> =
        flow {
            emit(Datasources.getNearestBranchesData(googleModel))
        }
            .map {
                Result.success(it.map {
                    it
                })
            }
            .retry(4) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(1000)
                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher+coroutineExceptionHandler)

    fun getNewOfferData(googleModel: GoogleModel?): Flow<Result<List<BranchesModelListItem>>> =
        flow {
            emit(Datasources.getNewOfferData(googleModel))
        }
            .map {
                Result.success(it.map {
                    it
                })
            }
            .retry(4) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(1000)
                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher+coroutineExceptionHandler)


    fun getFreeDliveryBranches(googleModel: GoogleModel?): Flow<Result<List<BranchesModelListItem>>> =
        flow {
            emit(Datasources.getFreeDliveryBranches(googleModel))
        }
            .map {
                Result.success(it.map {
                    it
                })
            }
            .retry(4) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(1000)
                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher+coroutineExceptionHandler)

    fun getMostOrderedBranches(googleModel: GoogleModel?): Flow<Result<List<BranchesModelListItem>>> =
        flow {
            emit(Datasources.getMostOrderedBranches(googleModel))
        }
            .map {
                Result.success(it.map {
                    it
                })
            }
            .retry(4) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(1000)
                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher+coroutineExceptionHandler)


}
