package com.codes_roots.imakeapp.repository.cart

import com.codes_roots.imakeapp.di.IoDispatcher
import com.satafood.core.entities.cart.CartConfigurations
import com.satafood.core.entities.google.GoogleModel
import com.satafood.core.entities.restaurantdetails.StoreDetailsModel
import com.satafood.core.entities.wallet.WalletModel
import kotlinx.coroutines.CoroutineDispatcher
import kotlinx.coroutines.delay
import kotlinx.coroutines.flow.*
import retrofit2.Response
import java.io.IOException
import javax.inject.Inject

class CartDataRepo @Inject constructor(
    private val Datasources: CartRemoteDataSource,
    @IoDispatcher private val ioDispatcher: CoroutineDispatcher
)
{

    fun getCartConfigurations(googleModel: GoogleModel): Flow<Result<Response<CartConfigurations>>> =
        flow {
            emit(Datasources.getCartConfig(googleModel))
             }
            .map { Result.success(it) }
            .retry(4){ t -> (t is IOException).also { if (it) {
                delay(1000)
            }}}
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher)



    fun checkWallet(googleModel: GoogleModel): Flow<Result<WalletModel>> =
        flow {
            emit(Datasources.checkWallet(googleModel))
             }
            .map { Result.success(it) }
            .retry(4){ t -> (t is IOException).also { if (it) {
                delay(1000)
            }}}
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher)





    fun getCartList(restaurant_id: Int,AreaId: Int, list:Array<Int>): Flow<Result<StoreDetailsModel>> =
        flow {
            emit(Datasources.GetCartItem(restaurant_id,AreaId,list))
        }
            .map { Result.success(it) }
            .retry(4){ t -> (t is IOException).also { if (it) {
                delay(1000)
            }}}
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher)


}