package com.codes_roots.imakeapp.repository.cart

import android.util.Log
import com.codes_roots.imakeapp.datalayer.APIServices
import com.satafood.core.entities.cart.CartConfigurations
import com.satafood.core.entities.google.GoogleModel
import com.satafood.core.entities.login.LoginModel
import com.satafood.core.entities.ordermodel.OrderModel
import com.satafood.core.entities.paymentways.PaymentWaysItem
import com.satafood.core.entities.paymentways.VoucherModel
import com.satafood.core.entities.paymob.MakePaymentOrderIntegration
import com.satafood.core.entities.paymob.PayMobToken
import com.satafood.core.entities.paymob.Payment
import com.satafood.core.entities.paymob.Token
import com.satafood.core.entities.restaurantdetails.StoreDetailsModel
import com.satafood.core.entities.wallet.WalletModel
import retrofit2.Response
import javax.inject.Inject

class CartRemoteDataSource @Inject constructor(
    private val ApiService: APIServices
) : CartDataSource {

    override suspend fun getCartConfig(googleModel: GoogleModel): Response<CartConfigurations> {
       kotlin.runCatching {  return ApiService.getCartConfig(googleModel) }.getOrElse { throw it }
    }

    override suspend fun checkWallet(googleModel: GoogleModel): WalletModel {
       kotlin.runCatching {  return ApiService.checkWallet(googleModel) }.getOrElse { throw it }
    }


    override suspend fun GetCartItem(
        restaurant_id: Int, AreaId: Int, Itemlist: Array<Int>
    ): StoreDetailsModel {
        Log.d("Itemlist", "GetCartItem:" + Itemlist)
        runCatching {  return  ApiService.getCartItems(restaurant_id, AreaId,Itemlist)}.getOrElse { throw it }
    }
    override suspend fun sendNotificationToDevice(loginModel: LoginModel): Response<LoginModel> {
        return runCatching {    ApiService.sendNotificationToDevice(loginModel)}.getOrElse { throw it }
    }

    override suspend fun getPaymentWays(): Response<ArrayList<PaymentWaysItem>> {
        runCatching {  return  ApiService.getPaymentWays()}.getOrElse { throw it }
    }

    override suspend fun checkVoucherRequest(voucherModel: VoucherModel): Response<VoucherModel> {
        runCatching {  return  ApiService.checkVoucherRequest(voucherModel)}.getOrElse { throw it }
    }
    override suspend fun MakePaymentOrderIntegration(loginModel: HashMap<String,Any>): Response<MakePaymentOrderIntegration?> {

        return runCatching {    ApiService.MakePaymentOrderIntegration("https://accept.paymob.com/api/ecommerce/orders",loginModel)}.getOrElse { throw it }
    }

    override suspend fun GetPaymentkey(auth : Payment): Response<Payment> {
        return ApiService.GetPaymentkey("https://accept.paymob.com/api/acceptance/payment_keys",auth)
    }
    override suspend fun getPaymobToken(auth : PayMobToken): Response<Token> {
        return ApiService.getPaymobToken("https://accept.paymob.com/api/auth/tokens",auth)
    }
    override suspend fun AddOrder(orderInfo: OrderModel): Response<OrderModel> {
        runCatching {  return  ApiService.AddOrder(orderInfo)  }.getOrElse { throw it }
    }



}