package com.codes_roots.imakeapp.repository.homepage


import com.codes_roots.imakeapp.di.IoDispatcher
import com.satafood.core.entities.add_driver_request.DriverRequestModel
import com.satafood.core.entities.favourite.FavouriteModelItem
import com.satafood.core.entities.gallery.GalleryModel
import com.satafood.core.entities.google.GoogleModel
import com.satafood.core.entities.homepage.CitiesModelItem
import com.satafood.core.entities.homepage.SectionsModelItem
import com.satafood.core.entities.homepage.sliders.SliderHomePageModelItem
import com.satafood.core.entities.mainpage.HomePageModel
import com.satafood.core.entities.myorders.MyOrderModelItem
import com.satafood.core.entities.myorders.OrderStatus
import com.satafood.core.entities.notification.NotificationModelItem
import com.satafood.core.entities.ordermodel.OrderModel
import com.satafood.core.entities.social_media.SocialMediaModelItem
import com.satafood.core.entities.static_pages.StaticPagesItem
import kotlinx.coroutines.*
import kotlinx.coroutines.flow.*
import retrofit2.Response
import java.io.IOException
import javax.inject.Inject


class DataRepo @Inject constructor(
    private val Datasources: DataSource,
    @IoDispatcher private val ioDispatcher: CoroutineDispatcher,
) {


    val dispatcher = ioDispatcher

    // // انا كنت ناسي اعمل emit في الcatch عشان كدا كان بيرجع هنا نفس الrespone في الحالتين
    // استخدمت Result في ال follow عشان بترجع الobject , error في نفس الclass وده مش بيحصل في ال sealed
    val TestCoroutine: Flow<Result<ArrayList<SectionsModelItem>>> =
        flow {
            emit(Datasources.getMainResponse())
        }
            .map { Result.success(it) }
            .retry(retries = 2) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(10000)
                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(dispatcher)

    fun getMainSlidersData(googleModel: GoogleModel?): Flow<Result<ArrayList<SliderHomePageModelItem?>>> =
        flow {
            emit(Datasources.getMainSlidersData(googleModel))
        }
            .map { Result.success(it) }
            .catch {
                    throwable ->
                emit(Result.failure(throwable))
            }
            .flowOn(dispatcher)

    fun getUserLastOrderForRating(): Flow<Result<MyOrderModelItem?>> =
        flow {
            emit(Datasources.getUserLastOrderForRating())
        }
            .map { Result.success(it) }
            .catch {
                    throwable ->
                emit(Result.failure(throwable))
            }
            .flowOn(dispatcher)

    fun changeOrderRateStatus(orderId: Int,rated: MyOrderModelItem): Flow<Result<MyOrderModelItem?>> =
        flow {
            emit(Datasources.changeOrderRateStatus(orderId,rated))
        }
            .map { Result.success(it) }
            .catch {
                    throwable ->
                emit(Result.failure(throwable))
            }
            .flowOn(dispatcher)


    fun getHomePageData(googleModel: GoogleModel?,user_id: Int?): Flow<Result<HomePageModel?>> =
        flow {
            emit(Datasources.getHomePageData(googleModel,user_id))
        }
            .map { Result.success(it) }
            .retry(retries = 2) { t ->
                (t is IOException).also {
                    if (it) {


                        delay(10000)

                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(dispatcher)

    fun getBranchesGallery(url: String?,query: GalleryModel?): Flow<Result<GalleryModel?>> =
        flow {
            emit(Datasources.getBranchesGallery(url!!,query!!))
        }
            .map { Result.success(it) }
            .retry(retries = 2) { t ->
                (t is IOException).also {
                    if (it) {


                        delay(10000)

                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(dispatcher)


    suspend fun GetCities(): Flow<Result<ArrayList<CitiesModelItem>?>> =
        flow {
            emit(Datasources.GetCities())
        }
            .map { Result.success(it) }

            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(dispatcher)


    suspend fun DeleteFavortie(id: Int): Flow<Result<Boolean>> =
        flow {
            emit(Datasources.DeleteFavorite(id))
        }
            .map { Result.success(it) }
            .retry(retries = 2) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(1000)
                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(dispatcher)


    //GetTownData
    suspend fun getTownData(cityId: Int): Flow<Result<ArrayList<CitiesModelItem>?>> =
        flow {
            emit(Datasources.GetTownData(cityId))
        }
            .map { Result.success(it) }
            .retry(2) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(10000)
                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(dispatcher)

    suspend fun getAreasData(cityId: Int): Flow<Result<ArrayList<CitiesModelItem>>> =
        flow {
            emit(Datasources.GetAreaData(cityId))
        }
            .map { Result.success(it) }
            .retry(2) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(10000)
                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(dispatcher)


    /// getFavorite
    fun getFavorite(userId: Int): Flow<Result<ArrayList<FavouriteModelItem>>> =
        flow {
            emit(Datasources.getFavorite(userId))
        }
            .map { Result.success(it) }
            .retry(2) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(10000)
                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(dispatcher)


    /// getNotifications
    fun getNotifications(user_id:Int,googleId:String,): Flow<Result<ArrayList<NotificationModelItem>>> =
        flow {
            emit(Datasources.getNotifications(user_id,googleId))
        }
            .map { Result.success(it) }
            .retry(2) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(10000)
                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(dispatcher)

    /// getNotifications
    fun getStaticPages(): Flow<Result<ArrayList<StaticPagesItem>>> =
        flow {
            emit(Datasources.getStaticPages())
        }
            .map { Result.success(it) }
            .retry(2) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(10000)
                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(dispatcher)
    fun getSocialIcons(): Flow<Result<ArrayList<SocialMediaModelItem>>> =
        flow {
            emit(Datasources.getSocialIcons())
        }
            .map { Result.success(it) }
            .retry(2) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(10000)
                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(dispatcher)

    /// getNotifications
    fun addDriverRequest(driverRequestModel: DriverRequestModel): Flow<Result<Response<DriverRequestModel>?>> =
        flow {
            emit(Datasources.addDriverRequest(driverRequestModel))
        }
            .map { Result.success(it) }
            .retry(2) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(10000)
                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(dispatcher)





    //getUserOrders
    val getUserOrders: Flow<Result<ArrayList<MyOrderModelItem>>> =
        flow {
            emit(Datasources.getUserOrders())
        }
            .map { Result.success(it) }
            .retry(retries = 2) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(10000)
                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(dispatcher)


    suspend fun getFilteredUserOrders(order_status_id : OrderModel): Flow<Result<ArrayList<MyOrderModelItem>>> =
        flow {
            emit(Datasources.getFilteredUserOrders(order_status_id))
        }
            .map { Result.success(it) }
            .retry(retries = 2) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(10000)
                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable))
            }
            .flowOn(dispatcher)




    //changeOrderStatus
    val getCategoriesData: Flow<Result<SectionsModelItem>> =
        flow {
            emit(Datasources.getCategoriesData())
        }
            .map { Result.success(it) }
            .retry(retries = 2) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(10000)
                    }
                }
            }
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(dispatcher)

    suspend fun changeOrderStatus(order_id: Int, data: OrderStatus): Flow<Result<OrderStatus>> =
        flow {
            emit(Datasources.changeOrderStatus(order_id, data))
        }
            .map {
                Result.success(it)
            }
            .retry(retries = 2) { t ->
                (t is IOException).also {
                    if (it) {
                        delay(10000)
                    }
                }
            }
            .catch {

                    throwable ->
                emit(Result.failure(throwable))
            }

            .flowOn(dispatcher)



}


