package com.codes_roots.imakeapp.repository.homepage


import com.codes_roots.imakeapp.datalayer.APIServices
import com.satafood.core.entities.add_driver_request.DriverRequestModel
import com.satafood.core.entities.favourite.FavouriteModelItem
import com.satafood.core.entities.gallery.GalleryModel
import com.satafood.core.entities.google.GoogleModel
import com.satafood.core.entities.homepage.CitiesModelItem
import com.satafood.core.entities.homepage.SectionsModelItem
import com.satafood.core.entities.homepage.sliders.SliderHomePageModelItem
import com.satafood.core.entities.login.LoginModel
import com.satafood.core.entities.mainpage.HomePageModel
import com.satafood.core.entities.myorders.MyOrderModelItem
import com.satafood.core.entities.myorders.OrderStatus
import com.satafood.core.entities.notification.NotificationModelItem
import com.satafood.core.entities.ordermodel.OrderModel
import com.satafood.core.entities.seemore_section.CategoryBannerItem
import com.satafood.core.entities.social_media.SocialMediaModelItem
import com.satafood.core.entities.static_pages.StaticPagesItem
import retrofit2.Response
import javax.inject.Inject

/**
Created by Prokash Sarkar on Tue, January 19, 2021
 **/


class RemoteDataSource @Inject constructor(private val ApiService: APIServices) : DataSource {

    override suspend fun getMainResponse(): ArrayList<SectionsModelItem> =
        runCatching { ApiService.GetMainData() }
            .getOrElse { throw it }


    override suspend fun getMainSlidersData(googleModel: GoogleModel?): ArrayList<SliderHomePageModelItem?> =
        runCatching { ApiService.getMainSlidersData(googleModel) }
            .getOrElse { throw it }


    override suspend fun getUserLastOrderForRating(): MyOrderModelItem =
        runCatching { ApiService.getUserLastOrderForRating() }
            .getOrElse { throw it }

    override suspend fun getCategoriesData(): SectionsModelItem =
        runCatching { ApiService.getCategoriesData() }
            .getOrElse { throw it }


    override suspend fun changeOrderRateStatus(
        orderId: Int,
        rated: MyOrderModelItem
    ): MyOrderModelItem =
        runCatching { ApiService.changeOrderRateStatus(orderId, rated) }
            .getOrElse { throw it }


    override suspend fun registerToken(loginModel: LoginModel): Response<Int> {
        runCatching { return ApiService.registerToken(loginModel) }
            .getOrElse { throw it }
    }


    override suspend fun getHomePageData(googleModel: GoogleModel?, user_id: Int?): HomePageModel? =
        runCatching { ApiService.getHomePageData(googleModel, user_id!!) }
            .getOrElse { throw it }


    override suspend fun getBranchesGallery(
        url: String,
        query: GalleryModel
    ): GalleryModel =
        runCatching { ApiService.getBranchesGallery("https://imake-app.com:1112/graphql", GalleryModel(query = "{gallery{ id photo }}"))  }
            .getOrElse { throw it }


    override suspend fun DeleteFavorite(id: Int): Boolean {
        runCatching { return ApiService.DELETEFavorite(id) }
            .getOrElse { throw it }
    }


    override suspend fun GetCities(): ArrayList<CitiesModelItem>? =
        runCatching { ApiService.GetCitiesData() }
            .getOrElse { throw it }


    override suspend fun GetTownData(city_id: Int): ArrayList<CitiesModelItem>? {
        runCatching { return ApiService.GetTownData(city_id).body() }
            .getOrElse { throw it }
    }

    override suspend fun GetAreaData(area_id: Int): ArrayList<CitiesModelItem> {
        runCatching {
            return ApiService.GetAreaData(area_id)
        }
            .getOrElse { throw it }
    }

    override suspend fun getUserOrders(): ArrayList<MyOrderModelItem> {
        runCatching { return ApiService.getUserOrders() }.getOrElse { throw it }
    }


    override suspend fun getFilteredUserOrders(order_status_id: OrderModel): ArrayList<MyOrderModelItem> {
        runCatching { return ApiService.getFilteredUserOrders(order_status_id) }.getOrElse { throw it }
    }

    override suspend fun changeOrderStatus(order_id: Int, data: OrderStatus): OrderStatus {
        runCatching { return ApiService.changeOrderStatus(order_id, data) }.getOrElse { throw it }
    }


    override suspend fun getFavorite(user_id: Int): ArrayList<FavouriteModelItem> {
        runCatching { return ApiService.getFavorite() }.getOrElse { throw it }
    }

    override suspend fun getCategoriesBanner(): ArrayList<CategoryBannerItem> {
        runCatching { return ApiService.getCategoriesBanner() }.getOrElse { throw it }
    }

    override suspend fun getStaticPages(): ArrayList<StaticPagesItem> {
        runCatching { return ApiService.getStaticPages() }.getOrElse { throw it }
    }

   override suspend fun getSocialIcons(): ArrayList<SocialMediaModelItem> {
        runCatching { return ApiService.getSocialIcons() }.getOrElse { throw it }
    }

    override suspend fun addDriverRequest(driverRequestModel: DriverRequestModel?): Response<DriverRequestModel> {
        runCatching { return ApiService.addDriverRequest(driverRequestModel) }.getOrElse { throw it }
    }


    override suspend fun getNotifications(
        user_id: Int,
        googleId: String,
    ): ArrayList<NotificationModelItem> {
        runCatching { return ApiService.getNotifications(user_id, googleId) }.getOrElse { throw it }
    }

}