package com.codes_roots.imakeapp.repository.main

import android.util.Log
import com.google.firebase.messaging.FirebaseMessaging
import com.codes_roots.imakeapp.datalayer.ExceptionsMapper
import com.codes_roots.imakeapp.helper.executeRemoteAPI
import com.satafood.core.base.DataState
import com.satafood.core.entities.main.Token
import com.satafood.core.repositories.IFCMRepository
import javax.inject.Inject

class FCMRepositoryImpl @Inject constructor(
    private val mainDataSource: MainDataSource,
    private val exceptionsMapper: ExceptionsMapper,
    private val firebaseMessaging: FirebaseMessaging
): IFCMRepository {

    override fun subscribeTopic(
        topics: String,
        onSubscribedListener: (DataState<Unit>) -> Unit
    ) {
        Log.d("TAG", "observeLiveData: $topics")
        firebaseMessaging.subscribeToTopic(topics.toString())
            .addOnSuccessListener {
                onSubscribedListener(DataState.success(Unit))
            }
            .addOnFailureListener {
                Log.d("TAG", "observeLiveData: error: $it")
                onSubscribedListener(
                    DataState(
                        DataState.Status.ERROR,
                        null,
                        DataState.HttpError(exceptionsMapper.mapFromException(it))
                    )
                )
            }

    }

    override fun updateUserToken(userId: Int, token: Token) {
        executeRemoteAPI{
            mainDataSource.updateUserToken(userId, token)
        }
    }
}