package com.codes_roots.imakeapp.repository.points_user

import com.codes_roots.imakeapp.di.IoDispatcher
import com.satafood.core.entities.userpoints.UserPointsModel
import kotlinx.coroutines.CoroutineDispatcher
import kotlinx.coroutines.delay
import kotlinx.coroutines.flow.*
import java.io.IOException
import javax.inject.Inject

class PointsDataRepo @Inject constructor(
    private val Datasources: PointsRemoteDataSource,
    @IoDispatcher private val ioDispatcher: CoroutineDispatcher
) {

    suspend  fun getUserPoints(): Flow<Result<UserPointsModel>> =
        flow {
            emit(Datasources.getUserPoints())
        }
            .map { Result.success(it) }
            .retry(4){ t -> (t is IOException).also { if (it) {
                delay(1000)
            }}}
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher)






}