package com.codes_roots.imakeapp.repository.retaurantdetails

import com.codes_roots.imakeapp.di.IoDispatcher
import com.satafood.core.entities.restaurantdetails.MenuCategoriesItem
import com.satafood.core.entities.restaurantdetails.StoreDetailsModel
import com.satafood.core.entities.reviews.ReviewsModel
import kotlinx.coroutines.CoroutineDispatcher
import kotlinx.coroutines.delay
import kotlinx.coroutines.flow.*
import java.io.IOException
import javax.inject.Inject

class DataRepo @Inject constructor(
    private val Datasources: RemoteDataSource,
    @IoDispatcher private val ioDispatcher: CoroutineDispatcher
)
{


    fun getStoresDetailsData(restaurant_id: Int, area_id: Int): Flow<Result<StoreDetailsModel>> =

        flow {
            emit(Datasources.getRestaurantDetails(restaurant_id,area_id))
             }
            .map { Result.success(it) }
            .retry(4){ t -> (t is IOException).also { if (it) {
                delay(1000)
            }}}
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher)

    fun getRestaurantReviews(restaurant_id: Int): Flow<Result<ReviewsModel>> =

        flow {
            emit(Datasources.getRestaurantReviews(restaurant_id))
        }
            .map { Result.success(it) }
            .retry(4){ t -> (t is IOException).also { if (it) {
                delay(1000)
            }}}
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher)

    fun getItemAdsData(item_id: Int): Flow<Result<ArrayList<MenuCategoriesItem>>> =
        flow {
            emit(Datasources.GetItemById(item_id))
             }
            .map { Result.success(it) }
            .retry(4){ t -> (t is IOException).also { if (it) {
                delay(1000)
            }}}
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher)





    fun AddFavorite(restaurant_id: Int, UserId:Int): Flow<Result<Boolean>> =
        flow {
            emit(Datasources.AddFavorite(restaurant_id,UserId))
        }
            .map { Result.success(it) }
            .retry(4){ t -> (t is IOException).also { if (it) {
                delay(1000)
            }}}
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher)

}