package com.codes_roots.imakeapp.repository.retaurantdetails

import com.codes_roots.imakeapp.datalayer.APIServices
import com.satafood.core.entities.addresses.UserAddressesItem
import com.satafood.core.entities.favourite.FavouriteModelItem
import com.satafood.core.entities.login.LoginModel
import com.satafood.core.entities.login.User
import com.satafood.core.entities.myorders.MyOrderModelItem
import com.satafood.core.entities.profile.ProfileModel
import com.satafood.core.entities.rate.RateModel
import com.satafood.core.entities.restaurantdetails.MenuCategoriesItem
import com.satafood.core.entities.restaurantdetails.StoreDetailsModel
import com.satafood.core.entities.reviews.ReviewsModel
import okhttp3.MultipartBody
import retrofit2.Response
import javax.inject.Inject

class RemoteDataSource @Inject constructor(
    private val ApiService: APIServices,
) : DataSource {


    override suspend fun getRestaurantDetails(
        restaurant_id: Int, area_id: Int,
    ): StoreDetailsModel {
        runCatching {
            return ApiService.getRestaurantDetails(restaurant_id, area_id)
        }.getOrElse { throw it }
    }
    override suspend fun getRestaurantReviews(
        restaurant_id: Int,
    ): ReviewsModel {
        runCatching {
            return ApiService.getRestaurantReviews(restaurant_id)
        }.getOrElse { throw it }
    }

    override suspend fun GetItemById(item_id: Int): ArrayList<MenuCategoriesItem> {
        runCatching { return ApiService.GetItemById(item_id) }.getOrElse { throw it }
    }

    override suspend fun checkFavorite(restaurant_id:Int):Response< ArrayList<FavouriteModelItem>> {
        runCatching {    return ApiService.checkFavorite(restaurant_id)}.getOrElse { throw it }
    }

    override suspend fun preOrder(orderInfo: MyOrderModelItem): Response<Double> {
        runCatching {  return  ApiService.preOrder(orderInfo)  }.getOrElse { throw it }
    }

    override suspend fun sendNotificationToDevice(loginModel: LoginModel): Response<LoginModel> {
        runCatching { return ApiService.sendNotificationToDevice(loginModel) }.getOrElse { throw it }
    }


    override suspend fun AddFavorite(restaurant_id: Int, UserId: Int): Boolean {
        runCatching { return ApiService.AddFavorite(restaurant_id, UserId) }.getOrElse { throw it }

    }


    //ArrayList<UserAddressesItem>
    override suspend fun getUserAddresses(): Response<ArrayList<UserAddressesItem>> {
        runCatching { return ApiService.getUsersAddress() }.getOrElse { throw it }
    }

    //ArrayList<UserAddressesItem>
    override suspend fun checkRestaurantAddress(branchId:Int): Response<ArrayList<UserAddressesItem>> {
        runCatching { return ApiService.checkRestaurantAddress(branchId) }.getOrElse { throw it }
    }

    override suspend fun addUsersAddress(addressInfo: UserAddressesItem): Response<UserAddressesItem> {
        runCatching { return ApiService.addUsersAddress(addressInfo) }.getOrElse { throw it }

    }

    override suspend fun editUsersAddress(
        addressId: Int,
        addressInfo: UserAddressesItem,
    ): Response<UserAddressesItem> {
        runCatching {
            return ApiService.editUsersAddress(addressId,
                addressInfo)
        }.getOrElse { throw it }

    }

    override suspend fun deleteUsersAddress(addressId: Int): Response<Int> {
        runCatching { return ApiService.deleteUsersAddress(addressId) }.getOrElse { throw it }

    }

    override suspend fun getUsersData(userId: Int): Response<ProfileModel> {
        runCatching { return ApiService.getUsersData(userId) }.getOrElse { throw it }

    }

    override suspend fun editUserData(file: MultipartBody.Part?, user: User): Response<User> {
        runCatching { return ApiService.editUsersData( user,file) }.getOrElse { throw it }
    }



    override suspend fun editUsersDataWithOutPhoto(user: User): Response<User> {
        runCatching { return ApiService.editUsersDataWithOutPhoto( user) }.getOrElse { throw it }
    }


    override suspend fun rate(rate: RateModel): Response<RateModel> {
        runCatching { return ApiService.rate(rate) }.getOrElse { throw it }
    }


}