package com.codes_roots.imakeapp.repository.splash

import com.codes_roots.imakeapp.di.IoDispatcher
import com.satafood.core.entities.google.GoogleModel
import com.satafood.core.entities.search_model.SearchModel
import com.satafood.core.entities.vendor_attributes.VendorAttributesModelItem
import kotlinx.coroutines.CoroutineDispatcher
import kotlinx.coroutines.delay
import kotlinx.coroutines.flow.*
import java.io.IOException
import javax.inject.Inject

class SplashDataRepo @Inject constructor(
    private val Datasources: SplashRemoteDataSource,
    @IoDispatcher private val ioDispatcher: CoroutineDispatcher
)
{

    fun getVendorAttributes(): Flow<Result<ArrayList<VendorAttributesModelItem>>> =
        flow {
            emit(Datasources.getVendorAttributes())
             }
            .map { Result.success(it) }
            .retry(4){ t -> (t is IOException).also { if (it) {
                delay(1000)
            }}}
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher)






}