package com.codes_roots.imakeapp.repository.wallet_details

import com.codes_roots.imakeapp.di.IoDispatcher
import com.satafood.core.entities.walletdetails.WalletDetailsModelItem
import kotlinx.coroutines.CoroutineDispatcher
import kotlinx.coroutines.delay
import kotlinx.coroutines.flow.*
import java.io.IOException
import javax.inject.Inject

class WalletTransactionsDataRepo @Inject constructor(
    private val Datasources: WalletDetailsRemoteDataSource,
    @IoDispatcher private val ioDispatcher: CoroutineDispatcher
)
{

    fun getWalletTransactions(walletId:WalletDetailsModelItem): Flow<Result<ArrayList<WalletDetailsModelItem>>> =
        flow {
            emit(Datasources.getWalletTransactions(walletId))
             }
            .map { Result.success(it) }
            .retry(4){ t -> (t is IOException).also { if (it) {
                delay(1000)
            }}}
            .catch { throwable -> emit(Result.failure(throwable)) }
            .flowOn(ioDispatcher)


}